@@ -71,7 +71,7 @@ class CHost extends CHostGeneral {
 	 * @param string|array  $options['selectGraphs']                       Return a "graphs" property with host graphs.
 	 * @param string|array  $options['selectApplications']                 Return an "applications" property with host applications.
 	 * @param string|array  $options['selectMacros']                       Return a "macros" property with host macros.
-	 * @param string|array  $options['selectScreens']                      Return a "screens" property with host screens.
+	 * @param string|array  $options['selectDashboards']                   Return a "dashboards" property with host dashboards.
 	 * @param string|array  $options['selectInterfaces']                   Return an "interfaces" property with host interfaces.
 	 * @param string|array  $options['selectInventory']                    Return an "inventory" property with host inventory data.
 	 * @param string|array  $options['selectHttpTests']                    Return an "httpTests" property with host web scenarios.
@@ -154,7 +154,7 @@ class CHost extends CHostGeneral {
 			'selectGraphs'						=> null,
 			'selectApplications'				=> null,
 			'selectMacros'						=> null,
-			'selectScreens'						=> null,
+			'selectDashboards'					=> null,
 			'selectInterfaces'					=> null,
 			'selectInventory'					=> null,
 			'selectHttpTests'					=> null,
@@ -235,9 +235,10 @@ class CHost extends CHostGeneral {
 		if (!is_null($options['interfaceids'])) {
 			zbx_value2array($options['interfaceids']);
 
-			$sqlParts['from']['interface'] = 'interface hi';
+			$sqlParts['left_join']['interface'] = ['alias' => 'hi', 'table' => 'interface', 'using' => 'hostid'];
+			$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
+
 			$sqlParts['where'][] = dbConditionInt('hi.interfaceid', $options['interfaceids']);
-			$sqlParts['where']['hi'] = 'h.hostid=hi.hostid';
 		}
 
 		// itemids
@@ -453,8 +454,8 @@ class CHost extends CHostGeneral {
 			zbx_db_search('hosts h', $options, $sqlParts);
 
 			if (zbx_db_search('interface hi', $options, $sqlParts)) {
-				$sqlParts['from']['interface'] = 'interface hi';
-				$sqlParts['where']['hi'] = 'h.hostid=hi.hostid';
+				$sqlParts['left_join']['interface'] = ['alias' => 'hi', 'table' => 'interface', 'using' => 'hostid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
 		}
 
@@ -480,8 +481,8 @@ class CHost extends CHostGeneral {
 			$this->dbFilter('hosts h', $options, $sqlParts);
 
 			if ($this->dbFilter('interface hi', $options, $sqlParts)) {
-				$sqlParts['from']['interface'] = 'interface hi';
-				$sqlParts['where']['hi'] = 'h.hostid=hi.hostid';
+				$sqlParts['left_join']['interface'] = ['alias' => 'hi', 'table' => 'interface', 'using' => 'hostid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 			}
 		}
 
@@ -548,11 +549,8 @@ class CHost extends CHostGeneral {
 					')';
 				}
 
-				$sqlParts['left_join']['hosts_templates'] = [
-					'from' => 'hosts_templates ht2',
-					'on' => 'h.hostid=ht2.hostid'
-				];
-				$sqlParts['left_table'] = $this->tableName();
+				$sqlParts['left_join'][] = ['alias' => 'ht2', 'table' => 'hosts_templates', 'using' => 'hostid'];
+				$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 
 				$operator = ($options['evaltype'] == TAG_EVAL_TYPE_AND_OR) ? ' AND ' : ' OR ';
 				$sqlParts['where'][] = '('.implode($operator, $where).')';
@@ -575,7 +573,7 @@ class CHost extends CHostGeneral {
 
 		// Return count or grouped counts via direct SQL count.
 		if ($options['countOutput'] && !$this->requiresPostSqlFiltering($options)) {
-			$res = DBselect($this->createSelectQueryFromParts($sqlParts), $options['limit']);
+			$res = DBselect(self::createSelectQueryFromParts($sqlParts), $options['limit']);
 			while ($host = DBfetch($res)) {
 				if ($options['groupCount']) {
 					$result[] = $host;
@@ -588,7 +586,7 @@ class CHost extends CHostGeneral {
 			return $result;
 		}
 
-		$result = zbx_toHash($this->customFetch($this->createSelectQueryFromParts($sqlParts), $options), 'hostid');
+		$result = zbx_toHash($this->customFetch(self::createSelectQueryFromParts($sqlParts), $options), 'hostid');
 
 		// Return count for post SQL filtered result sets.
 		if ($options['countOutput']) {
@@ -643,11 +641,8 @@ class CHost extends CHostGeneral {
 
 		if ((!$options['countOutput'] && $this->outputIsRequested('inventory_mode', $options['output']))
 				|| ($options['filter'] && array_key_exists('inventory_mode', $options['filter']))) {
-			$sqlParts['left_join'][] = [
-				'from' => 'host_inventory hinv',
-				'on' => $this->tableAlias().'.'.$this->pk().'=hinv.hostid'
-			];
-			$sqlParts['left_table'] = $this->tableName();
+			$sqlParts['left_join'][] = ['alias' => 'hinv', 'table' => 'host_inventory', 'using' => 'hostid'];
+			$sqlParts['left_table'] = ['alias' => $this->tableAlias, 'table' => $this->tableName];
 		}
 
 		return $sqlParts;
@@ -699,8 +694,7 @@ class CHost extends CHostGeneral {
 		$this->validateCreate($hosts);
 
 		$hostids = [];
-		$ins_tags = [];
-		foreach ($hosts as $host) {
+		foreach ($hosts as &$host) {
 			// If visible name is not given or empty it should be set to host name.
 			if (!array_key_exists('name', $host) || !trim($host['name'])) {
 				$host['name'] = $host['host'];
@@ -710,6 +704,7 @@ class CHost extends CHostGeneral {
 			$hostid = reset($hostid);
 			$host['hostid'] = $hostid;
 			$hostids[] = $hostid;
+			$host['groups'] = zbx_toArray($host['groups']);
 
 			// Save groups. Groups must be added before calling massAdd() for permission validation to work.
 			$groupsToAdd = [];
@@ -721,10 +716,8 @@ class CHost extends CHostGeneral {
 			}
 			DB::insert('hosts_groups', $groupsToAdd);
 
-			if (array_key_exists('tags', $host)) {
-				foreach ($host['tags'] as $tag) {
-					$ins_tags[] = ['hostid' => $hostid] + $tag;
-				}
+			if (array_key_exists('tags', $host) && $host['tags']) {
+				$this->createTags([$hostid => zbx_toArray($host['tags'])]);
 			}
 
 			$options = [
@@ -767,10 +760,9 @@ class CHost extends CHostGeneral {
 				DB::insert('host_inventory', [$hostInventory], false);
 			}
 		}
+		unset($host);
 
-		if ($ins_tags) {
-			DB::insert('host_tag', $ins_tags);
-		}
+		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_HOST, $hosts);
 
 		return ['hostids' => $hostids];
 	}
@@ -854,7 +846,7 @@ class CHost extends CHostGeneral {
 		$macros = [];
 		foreach ($hosts as &$host) {
 			if (isset($host['macros'])) {
-				$macros[$host['hostid']] = $host['macros'];
+				$macros[$host['hostid']] = zbx_toArray($host['macros']);
 
 				unset($host['macros']);
 			}
@@ -889,7 +881,7 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		$this->updateTags($hosts, 'hostid');
+		$this->updateTags(array_column($hosts, 'tags', 'hostid'));
 
 		return ['hostids' => $hostids];
 	}
@@ -966,7 +958,10 @@ class CHost extends CHostGeneral {
 		sort($hostids);
 
 		$db_hosts = $this->get([
-			'output' => ['hostid', 'host'],
+			'output' => ['hostid', 'proxy_hostid', 'host', 'status', 'ipmi_authtype', 'ipmi_privilege', 'ipmi_username',
+				'ipmi_password', 'name', 'description', 'tls_connect', 'tls_accept', 'tls_issuer', 'tls_subject',
+				'tls_psk_identity', 'tls_psk', 'inventory_mode'
+			],
 			'hostids' => $hostids,
 			'editable' => true,
 			'preservekeys' => true
@@ -1112,12 +1107,8 @@ class CHost extends CHostGeneral {
 			$updateInventory['inventory_mode'] = $data['inventory_mode'];
 		}
 
-		if (isset($data['status'])) {
-			$updateStatus = $data['status'];
-		}
-
 		unset($data['hosts'], $data['groups'], $data['interfaces'], $data['templates_clear'], $data['templates'],
-			$data['macros'], $data['inventory'], $data['inventory_mode'], $data['status']);
+			$data['macros'], $data['inventory'], $data['inventory_mode']);
 
 		if (!zbx_empty($data)) {
 			DB::update('hosts', [
@@ -1126,10 +1117,6 @@ class CHost extends CHostGeneral {
 			]);
 		}
 
-		if (isset($updateStatus)) {
-			updateHostStatus($hostids, $updateStatus);
-		}
-
 		/*
 		 * Update template linkage
 		 */
@@ -1321,6 +1308,18 @@ class CHost extends CHostGeneral {
 			}
 		}
 
+		$new_hosts = [];
+		foreach ($db_hosts as $hostid => $db_host) {
+			$new_host = $data + $db_host;
+			if ($new_host['status'] != $db_host['status']) {
+				info(_s('Updated status of host "%1$s".', $new_host['host']));
+			}
+
+			$new_hosts[] = $new_host;
+		}
+
+		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST, $new_hosts, $db_hosts);
+
 		return ['hostids' => $inputHostIds];
 	}
 
@@ -1332,18 +1331,24 @@ class CHost extends CHostGeneral {
 	 * Additional supported $data parameters are:
 	 * - interfaces  - an array of interfaces to delete from the hosts
 	 *
+	 * @throws APIException if the input is invalid.
+	 *
 	 * @param array $data
 	 *
 	 * @return array
 	 */
 	public function massRemove(array $data) {
-		$hostids = zbx_toArray($data['hostids']);
+		if (!array_key_exists('hostids', $data) || $data['hostids'] === null) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+		}
 
-		$this->checkPermissions($hostids, _('No permissions to referred object or it does not exist!'));
+		$data['hostids'] = zbx_toArray($data['hostids']);
+
+		$this->checkPermissions($data['hostids'], _('No permissions to referred object or it does not exist!'));
 
 		if (isset($data['interfaces'])) {
 			$options = [
-				'hostids' => $hostids,
+				'hostids' => $data['hostids'],
 				'interfaces' => zbx_toArray($data['interfaces'])
 			];
 			API::HostInterface()->massRemove($options);
@@ -1520,7 +1525,7 @@ class CHost extends CHostGeneral {
 		}
 
 		DB::delete('opcommand_hst', [
-			'hostid' => $hostIds,
+			'hostid' => $hostIds
 		]);
 
 		// delete empty operations
@@ -1535,7 +1540,7 @@ class CHost extends CHostGeneral {
 		}
 
 		DB::delete('operations', [
-			'operationid' => $delOperationids,
+			'operationid' => $delOperationids
 		]);
 
 		$hosts = API::Host()->get([
@@ -1631,43 +1636,48 @@ class CHost extends CHostGeneral {
 			}
 		}
 
-		// adding screens
-		if ($options['selectScreens'] !== null) {
-			if ($options['selectScreens'] != API_OUTPUT_COUNT) {
-				$screens = API::TemplateScreen()->get([
-					'output' => $this->outputExtend($options['selectScreens'], ['hostid']),
-					'hostids' => $hostids,
-					'nopermissions' => true
+		// Adding dashboards.
+		if ($options['selectDashboards'] !== null) {
+			[$hosts_templates, $templateids] = CApiHostHelper::getParentTemplates($hostids);
+
+			if ($options['selectDashboards'] != API_OUTPUT_COUNT) {
+				$dashboards = API::TemplateDashboard()->get([
+					'output' => $this->outputExtend($options['selectDashboards'], ['templateid']),
+					'templateids' => $templateids
 				]);
+
 				if (!is_null($options['limitSelects'])) {
-					order_result($screens, 'name');
+					order_result($dashboards, 'name');
 				}
 
-				// inherited screens do not have a unique screenid, so we're building a map using array keys
-				$relationMap = new CRelationMap();
-				foreach ($screens as $key => $screen) {
-					$relationMap->addRelation($screen['hostid'], $key);
+				foreach ($result as &$host) {
+					foreach ($hosts_templates[$host['hostid']] as $templateid) {
+						foreach ($dashboards as $dashboard) {
+							if ($dashboard['templateid'] == $templateid) {
+								$host['dashboards'][] = $dashboard;
+							}
+						}
+					}
 				}
-
-				$screens = $this->unsetExtraFields($screens, ['hostid'], $options['selectScreens']);
-				$result = $relationMap->mapMany($result, $screens, 'screens', $options['limitSelects']);
+				unset($host);
 			}
 			else {
-				$screens = API::TemplateScreen()->get([
-					'hostids' => $hostids,
-					'nopermissions' => true,
+				$dashboards = API::TemplateDashboard()->get([
+					'templateids' => $templateids,
 					'countOutput' => true,
 					'groupCount' => true
 				]);
 
 				foreach ($result as $hostid => $host) {
-					$result[$hostid]['screens'] = 0;
+					$result[$hostid]['dashboards'] = 0;
 
-					foreach ($screens as $screen) {
-						if (bccomp($screen['hostid'], $hostid) == 0) {
-							$result[$hostid]['screens'] += $screen['rowscount'];
+					foreach ($dashboards as $dashboard) {
+						if (in_array($dashboard['templateid'], $hosts_templates[$hostid])) {
+							$result[$hostid]['dashboards'] += $dashboard['rowscount'];
 						}
 					}
+
+					$result[$hostid]['dashboards'] = (string) $result[$hostid]['dashboards'];
 				}
 			}
 		}
@@ -1945,7 +1955,19 @@ class CHost extends CHostGeneral {
 				);
 			}
 
-			$groupids = array_merge($groupids, zbx_objectValues($host['groups'], 'groupid'));
+			$host['groups'] = zbx_toArray($host['groups']);
+
+			foreach ($host['groups'] as $group) {
+				if (!is_array($group) || (is_array($group) && !array_key_exists('groupid', $group))) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect value for field "%1$s": %2$s.', 'groups',
+							_s('the parameter "%1$s" is missing', 'groupid')
+						)
+					);
+				}
+
+				$groupids[$group['groupid']] = true;
+			}
 
 			// Validate tags.
 			if (array_key_exists('tags', $host)) {
@@ -1970,14 +1992,14 @@ class CHost extends CHostGeneral {
 		}
 
 		// Validate permissions to host groups.
-		if ($groupids) {
-			$db_groups = API::HostGroup()->get([
+		$db_groups = $groupids
+			? API::HostGroup()->get([
 				'output' => ['groupid'],
-				'groupids' => $groupids,
+				'groupids' => array_keys($groupids),
 				'editable' => true,
 				'preservekeys' => true
-			]);
-		}
+			])
+			: [];
 
 		foreach ($hosts as $host) {
 			foreach ($host['groups'] as $group) {
@@ -2006,8 +2028,9 @@ class CHost extends CHostGeneral {
 		$host_names = [];
 
 		foreach ($hosts as $host) {
-			if (!array_key_exists('interfaces', $host) || !is_array($host['interfaces']) || !$host['interfaces']) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _s('No interfaces for host "%1$s".', $host['host']));
+			if (array_key_exists('interfaces', $host) && $host['interfaces'] !== null
+					&& !is_array($host['interfaces'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
 			if (array_key_exists('status', $host)) {
@@ -2099,7 +2122,7 @@ class CHost extends CHostGeneral {
 	protected function validateUpdate(array $hosts, array $db_hosts) {
 		$host_db_fields = ['hostid' => null];
 
-		foreach ($hosts as $host) {
+		foreach ($hosts as &$host) {
 			// Validate mandatory fields.
 			if (!check_db_fields($host_db_fields, $host)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -2120,14 +2143,29 @@ class CHost extends CHostGeneral {
 			}
 
 			// Validate "groups" field.
-			if (array_key_exists('groups', $host) && (!is_array($host['groups']) || !$host['groups'])) {
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host "%1$s" cannot be without host group.', $db_hosts[$host['hostid']]['host'])
-				);
+			if (array_key_exists('groups', $host)) {
+				if (!is_array($host['groups']) || !$host['groups']) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Host "%1$s" cannot be without host group.', $db_hosts[$host['hostid']]['host'])
+					);
+				}
+
+				$host['groups'] = zbx_toArray($host['groups']);
+
+				foreach ($host['groups'] as $group) {
+					if (!is_array($group) || (is_array($group) && !array_key_exists('groupid', $group))) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Incorrect value for field "%1$s": %2$s.', 'groups',
+								_s('the parameter "%1$s" is missing', 'groupid')
+							)
+						);
+					}
+				}
 			}
 
 			// Permissions to host groups is validated in massUpdate().
 		}
+		unset($host);
 
 		$inventory_fields = zbx_objectValues(getHostInventories(), 'db_field');
 
@@ -2183,10 +2221,9 @@ class CHost extends CHostGeneral {
 			$update_discovered_validator->setObjectName($host_name);
 			$this->checkPartialValidator($host, $update_discovered_validator, $db_host);
 
-			if (array_key_exists('interfaces', $host)) {
-				if (!is_array($host['interfaces']) || !$host['interfaces']) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('No interfaces for host "%1$s".', $host['host']));
-				}
+			if (array_key_exists('interfaces', $host) && $host['interfaces'] !== null
+					&& !is_array($host['interfaces'])) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
 			}
 
 			if (array_key_exists('host', $host)) {
