@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CScreenItem extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_USER, 'action' => CRoleHelper::ACTIONS_EDIT_DASHBOARDS],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_USER, 'action' => CRoleHelper::ACTIONS_EDIT_DASHBOARDS],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_USER, 'action' => CRoleHelper::ACTIONS_EDIT_DASHBOARDS]
+	];
+
 	protected $tableName = 'screens_items';
 	protected $tableAlias = 'si';
 
@@ -41,7 +48,6 @@ class CScreenItem extends CApiService {
 		SCREEN_RESOURCE_TRIGGER_INFO,
 		SCREEN_RESOURCE_SERVER_INFO,
 		SCREEN_RESOURCE_CLOCK,
-		SCREEN_RESOURCE_SCREEN,
 		SCREEN_RESOURCE_TRIGGER_OVERVIEW,
 		SCREEN_RESOURCE_DATA_OVERVIEW,
 		SCREEN_RESOURCE_URL,
@@ -65,7 +71,7 @@ class CScreenItem extends CApiService {
 		$this->getOptions = zbx_array_merge($this->getOptions, [
 			'screenitemids'	=> null,
 			'screenids'		=> null,
-			'editable'		=> null,
+			'editable'		=> false,
 			'sortfield'		=> '',
 			'sortorder'		=> '',
 			'preservekeys'	=> false,
@@ -170,19 +176,6 @@ class CScreenItem extends CApiService {
 			'preservekeys' => true
 		]);
 
-		if (count($dbScreens) < count($screenIds)) {
-			$dbTemplateScreens = API::TemplateScreen()->get([
-				'output' => ['screenid', 'hsize', 'vsize', 'name'],
-				'screenids' => $screenIds,
-				'editable' => true,
-				'preservekeys' => true
-			]);
-
-			if ($dbTemplateScreens) {
-				$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
-			}
-		}
-
 		$dbScreenItems = API::getApiService()->select($this->tableName(), [
 			'output' => ['screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan'],
 			'filter' => ['screenid' => array_keys($dbScreens)],
@@ -284,17 +277,6 @@ class CScreenItem extends CApiService {
 			'preservekeys' => true
 		]);
 
-		$dbTemplateScreens = API::TemplateScreen()->get([
-			'output' => ['screenid', 'hsize', 'vsize', 'name'],
-			'screenitemids' => $screenItemIds,
-			'editable' => true,
-			'preservekeys' => true
-		]);
-
-		if ($dbTemplateScreens) {
-			$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
-		}
-
 		$dbScreenItems = API::getApiService()->select($this->tableName(), [
 			'output' => ['screenitemid', 'screenid', 'x', 'y', 'rowspan', 'colspan', 'resourcetype', 'resourceid',
 				'style'
@@ -538,14 +520,6 @@ class CScreenItem extends CApiService {
 					$mapIds[$screenItem['resourceid']] = $screenItem['resourceid'];
 					break;
 
-				case SCREEN_RESOURCE_SCREEN:
-					if (!$screenItem['resourceid']) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('No screen ID provided for screen element.'));
-					}
-
-					$screenIds[$screenItem['resourceid']] = $screenItem['resourceid'];
-					break;
-
 				case SCREEN_RESOURCE_ACTIONS:
 				case SCREEN_RESOURCE_SERVER_INFO:
 				case SCREEN_RESOURCE_SYSTEM_STATUS:
@@ -730,18 +704,6 @@ class CScreenItem extends CApiService {
 				'preservekeys' => true
 			]);
 
-			if (count($dbScreens) < count($screenIds)) {
-				$dbTemplateScreens = API::TemplateScreen()->get([
-					'output' => ['screenid'],
-					'screenids' => $screenIds,
-					'preservekeys' => true
-				]);
-
-				if ($dbTemplateScreens) {
-					$dbScreens = zbx_array_merge($dbScreens, $dbTemplateScreens);
-				}
-			}
-
 			foreach ($screenIds as $screenId) {
 				if (!isset($dbScreens[$screenId])) {
 					self::exception(ZBX_API_ERROR_PERMISSIONS, _s(
