@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CUserGroup extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
+	];
+
 	protected $tableName = 'usrgrp';
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['usrgrpid', 'name'];
@@ -72,6 +79,7 @@ class CUserGroup extends CApiService {
 			'output'					=> API_OUTPUT_EXTEND,
 			'selectUsers'				=> null,
 			'selectRights'				=> null,
+			'selectTagFilters'			=> null,
 			'countOutput'				=> false,
 			'preservekeys'				=> false,
 			'sortfield'					=> '',
@@ -138,7 +146,7 @@ class CUserGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($usrgrp = DBfetch($res)) {
 			if ($options['countOutput']) {
 				$result = $usrgrp['rowscount'];
@@ -186,6 +194,7 @@ class CUserGroup extends CApiService {
 		unset($usrgrp);
 
 		$this->updateRights($usrgrps, __FUNCTION__);
+		$this->updateTagFilters($usrgrps, __FUNCTION__);
 		$this->updateUsersGroups($usrgrps, __FUNCTION__);
 
 		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_USER_GROUP, $usrgrps);
@@ -206,12 +215,17 @@ class CUserGroup extends CApiService {
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
 			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
 			'debug_mode' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
-			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_DISABLED])],
+			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED])],
 			'users_status' =>	['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
 			'rights' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
 				'id' =>				['type' => API_ID, 'flags' => API_REQUIRED],
 				'permission' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
 			]],
+			'tag_filters' =>	['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
+				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
+				'tag' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
+				'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
+			]],
 			'userids' =>		['type' => API_IDS, 'flags' => API_NORMALIZE]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $usrgrps, '/', $error)) {
@@ -222,6 +236,7 @@ class CUserGroup extends CApiService {
 		$this->checkUsers($usrgrps);
 		$this->checkHimself($usrgrps, __FUNCTION__);
 		$this->checkHostGroups($usrgrps);
+		$this->checkTagFilters($usrgrps);
 	}
 
 	/**
@@ -265,6 +280,7 @@ class CUserGroup extends CApiService {
 		}
 
 		$this->updateRights($usrgrps, __FUNCTION__);
+		$this->updateTagFilters($usrgrps, __FUNCTION__);
 		$this->updateUsersGroups($usrgrps, __FUNCTION__);
 
 		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_USER_GROUP, $usrgrps, $db_usrgrps);
@@ -287,12 +303,17 @@ class CUserGroup extends CApiService {
 			'usrgrpid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
 			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('usrgrp', 'name')],
 			'debug_mode' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_DEBUG_MODE_DISABLED, GROUP_DEBUG_MODE_ENABLED])],
-			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_DISABLED])],
+			'gui_access' =>		['type' => API_INT32, 'in' => implode(',', [GROUP_GUI_ACCESS_SYSTEM, GROUP_GUI_ACCESS_INTERNAL, GROUP_GUI_ACCESS_LDAP, GROUP_GUI_ACCESS_DISABLED])],
 			'users_status' =>	['type' => API_INT32, 'in' => implode(',', [GROUP_STATUS_ENABLED, GROUP_STATUS_DISABLED])],
 			'rights' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['id']], 'fields' => [
 				'id' =>				['type' => API_ID, 'flags' => API_REQUIRED],
 				'permission' =>		['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', [PERM_DENY, PERM_READ, PERM_READ_WRITE])]
 			]],
+			'tag_filters' =>	['type' => API_OBJECTS, 'uniq' => [['groupid', 'tag', 'value']], 'fields' => [
+				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED],
+				'tag' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'tag'), 'default' => DB::getDefault('tag_filter', 'tag')],
+				'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('tag_filter', 'value'), 'default' => DB::getDefault('tag_filter', 'value')]
+			]],
 			'userids' =>		['type' => API_IDS, 'flags' => API_NORMALIZE]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $usrgrps, '/', $error)) {
@@ -330,6 +351,7 @@ class CUserGroup extends CApiService {
 		$this->checkHimself($usrgrps, __FUNCTION__, $db_usrgrps);
 		$this->checkUsersWithoutGroups($usrgrps);
 		$this->checkHostGroups($usrgrps);
+		$this->checkTagFilters($usrgrps);
 	}
 
 	/**
@@ -406,6 +428,11 @@ class CUserGroup extends CApiService {
 					$groupids[$right['id']] = true;
 				}
 			}
+			if (array_key_exists('tag_filters', $usrgrp)) {
+				foreach ($usrgrp['tag_filters'] as $tag_filter) {
+					$groupids[$tag_filter['groupid']] = true;
+				}
+			}
 		}
 
 		if (!$groupids) {
@@ -414,7 +441,7 @@ class CUserGroup extends CApiService {
 
 		$groupids = array_keys($groupids);
 
-		$db_groups = DB::select('groups', [
+		$db_groups = DB::select('hstgrp', [
 			'output' => [],
 			'groupids' => $groupids,
 			'preservekeys' => true
@@ -427,6 +454,27 @@ class CUserGroup extends CApiService {
 		}
 	}
 
+	/**
+	 * Tag filter validation.
+	 *
+	 * @param array  $usrgrps
+	 *
+	 * @throws APIException
+	 */
+	private function checkTagFilters(array $usrgrps) {
+		foreach ($usrgrps as $usrgrp) {
+			if (array_key_exists('tag_filters', $usrgrp)) {
+				foreach ($usrgrp['tag_filters'] as $tag_filter) {
+					if ($tag_filter['tag'] === '' && $tag_filter['value'] !== '') {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Incorrect value for field "%1$s": %2$s.', _('tag'), _('cannot be empty'))
+						);
+					}
+				}
+			}
+		}
+	}
+
 	/**
 	 * Auxiliary function for checkHimself().
 	 * Returns true if user group has GROUP_GUI_ACCESS_DISABLED or GROUP_STATUS_DISABLED states.
@@ -612,7 +660,7 @@ class CUserGroup extends CApiService {
 				if ($db_right['permission'] != $rights[$db_right['groupid']][$db_right['id']]) {
 					$upd_rights[] = [
 						'values' => ['permission' => $rights[$db_right['groupid']][$db_right['id']]],
-						'where' => ['rightid' => $db_right['rightid']],
+						'where' => ['rightid' => $db_right['rightid']]
 					];
 				}
 				unset($rights[$db_right['groupid']][$db_right['id']]);
@@ -645,6 +693,88 @@ class CUserGroup extends CApiService {
 		}
 	}
 
+	/**
+	 * Update table "tag_filter".
+	 *
+	 * @param array  $usrgrps
+	 * @param string $method
+	 */
+	private function updateTagFilters(array $usrgrps, $method) {
+		$tag_filters = [];
+
+		foreach ($usrgrps as $usrgrp) {
+			if (array_key_exists('tag_filters', $usrgrp)) {
+				$tag_filters[$usrgrp['usrgrpid']] = [];
+
+				foreach ($usrgrp['tag_filters'] as $tag_filter) {
+					$tag_filter['usrgrpid'] = $usrgrp['usrgrpid'];
+					$tag_filters[$usrgrp['usrgrpid']][] = $tag_filter;
+				}
+				CArrayHelper::sort($tag_filters[$usrgrp['usrgrpid']], ['groupid', 'tag', 'value']);
+			}
+		}
+
+		if (!$tag_filters) {
+			return;
+		}
+
+		$db_tag_filters = ($method === 'update')
+			? DB::select('tag_filter', [
+				'output' => ['tag_filterid', 'usrgrpid', 'groupid', 'tag', 'value'],
+				'filter' => ['usrgrpid' => array_keys($tag_filters)]
+			])
+			: [];
+		CArrayHelper::sort($db_tag_filters, ['usrgrpid', 'groupid', 'tag', 'value']);
+
+		$ins_tag_filters = [];
+		$upd_tag_filters = [];
+		$del_tag_filterids = [];
+
+		foreach ($db_tag_filters as $db_tag_filter) {
+			if ($tag_filters[$db_tag_filter['usrgrpid']]) {
+				$tag_filter = array_shift($tag_filters[$db_tag_filter['usrgrpid']]);
+
+				$upd_tag_filter = [];
+
+				if (bccomp($tag_filter['groupid'], $db_tag_filter['groupid']) != 0) {
+					$upd_tag_filter['groupid'] = $tag_filter['groupid'];
+				}
+				if ($tag_filter['tag'] !== $db_tag_filter['tag']) {
+					$upd_tag_filter['tag'] = $tag_filter['tag'];
+				}
+				if ($tag_filter['value'] !== $db_tag_filter['value']) {
+					$upd_tag_filter['value'] = $tag_filter['value'];
+				}
+
+				if ($upd_tag_filter) {
+					$upd_tag_filters[] = [
+						'values' => $upd_tag_filter,
+						'where' => ['tag_filterid' => $db_tag_filter['tag_filterid']]
+					];
+				}
+			}
+			else {
+				$del_tag_filterids[] = $db_tag_filter['tag_filterid'];
+			}
+		}
+
+		foreach ($usrgrps as $usrgrp) {
+			$ins_tag_filters = array_merge($ins_tag_filters, $tag_filters[$usrgrp['usrgrpid']]);
+		}
+
+		if ($ins_tag_filters) {
+			DB::insertBatch('tag_filter', array_values($ins_tag_filters));
+		}
+
+		if ($upd_tag_filters) {
+			DB::update('tag_filter', array_values($upd_tag_filters));
+		}
+
+		if ($del_tag_filterids) {
+			DB::delete('tag_filter', ['tag_filterid' => $del_tag_filterids]);
+		}
+	}
+
 	/**
 	 * Update table "users_groups".
 	 *
@@ -705,304 +835,6 @@ class CUserGroup extends CApiService {
 		}
 	}
 
-	/**
-	 * @deprecated	As of version 3.4, use create() method instead.
-	 */
-	public function massAdd($data) {
-		$this->deprecated('usergroup.massadd method is deprecated.');
-
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can delete user groups.'));
-		}
-
-		$usrgrpids = array_keys(array_flip(zbx_toArray($data['usrgrpids'])));
-		$userids = (isset($data['userids']) && !is_null($data['userids'])) ? zbx_toArray($data['userids']) : null;
-		$rights = (isset($data['rights']) && !is_null($data['rights'])) ? zbx_toArray($data['rights']) : null;
-
-		if (!is_null($userids)) {
-			$options = [
-				'usrgrpids' => $usrgrpids,
-				'output' => API_OUTPUT_EXTEND,
-			];
-			$usrgrps = $this->get($options);
-			foreach ($usrgrps as $usrgrp) {
-				if ((($usrgrp['gui_access'] == GROUP_GUI_ACCESS_DISABLED)
-					|| ($usrgrp['users_status'] == GROUP_STATUS_DISABLED))
-					&& uint_in_array(self::$userData['userid'], $userids)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _('User cannot change status of himself'));
-				}
-			}
-
-			$linkedUsers = [];
-			$sql = 'SELECT usrgrpid, userid'.
-				' FROM users_groups'.
-				' WHERE '.dbConditionInt('usrgrpid', $usrgrpids).
-				' AND '.dbConditionInt('userid', $userids);
-			$linkedUsersDb = DBselect($sql);
-			while ($link = DBfetch($linkedUsersDb)) {
-				if (!isset($linkedUsers[$link['usrgrpid']])) $linkedUsers[$link['usrgrpid']] = [];
-				$linkedUsers[$link['usrgrpid']][$link['userid']] = 1;
-			}
-
-			$usersInsert = [];
-			foreach ($usrgrpids as $usrgrpid) {
-				foreach ($userids as $userid) {
-					if (!isset($linkedUsers[$usrgrpid][$userid])) {
-						$usersInsert[] = [
-							'usrgrpid' => $usrgrpid,
-							'userid' => $userid,
-						];
-					}
-				}
-			}
-			DB::insert('users_groups', $usersInsert);
-		}
-
-		if (!is_null($rights)) {
-			$linkedRights = [];
-			$sql = 'SELECT groupid,id'.
-				' FROM rights'.
-				' WHERE '.dbConditionInt('groupid', $usrgrpids).
-					' AND '.dbConditionInt('id', zbx_objectValues($rights, 'id'));
-			$linkedRightsDb = DBselect($sql);
-			while ($link = DBfetch($linkedRightsDb)) {
-				if (!isset($linkedRights[$link['groupid']])) $linkedRights[$link['groupid']] = [];
-				$linkedRights[$link['groupid']][$link['id']] = 1;
-			}
-
-			$rightInsert = [];
-			foreach ($usrgrpids as $usrgrpid) {
-				foreach ($rights as $right) {
-					if (!isset($linkedRights[$usrgrpid][$right['id']])) {
-						$rightInsert[] = [
-							'groupid' => $usrgrpid,
-							'permission' => $right['permission'],
-							'id' => $right['id']
-						];
-					}
-				}
-			}
-			DB::insert('rights', $rightInsert);
-		}
-
-		return ['usrgrpids' => $usrgrpids];
-	}
-
-	/**
-	 * Mass update user group.
-	 * Checks for permissions - only super admins can change user groups.
-	 * Changes name to a group if name and one user group id is provided.
-	 * Links/unlinks users to user groups.
-	 * Links/unlinks rights to user groups.
-	 *
-	 * @deprecated	As of version 3.4, use update() method instead.
-	 *
-	 * @param array $data
-	 * @param int|int[] $data['usrgrpids'] id or ids of user groups to be updated.
-	 * @param string $data['name'] name to be set to a user group. Only one host group id can be passed at a time!
-	 * @param null|int|int[] $data['userids'] user ids to link to given user groups. Missing user ids will be unlinked from user groups.
-	 * @param null|array $data['rights'] rights to link to given user groups. Missing rights will be unlinked from user groups.
-	 * @param int $data['rights']['id'] id of right.
-	 * @param int $data['rights']['permission'] permission level of right.
-	 *
-	 * @return int[] array['usrgrpids'] returns passed user group ids
-	 */
-	public function massUpdate($data) {
-		$this->deprecated('usergroup.massupdate method is deprecated.');
-
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can delete user groups.'));
-		}
-
-		$usrgrpids = zbx_toArray($data['usrgrpids']);
-
-		if (count($usrgrpids) == 0) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Missing parameter: usrgrpids.'));
-		}
-
-		// $data['name'] parameter restrictions
-		if (isset($data['name'])) {
-			// same name can be set only to one hostgroup
-			if (count($usrgrpids) > 1) {
-				self::exception(ZBX_API_ERROR_PARAMETERS, _('Only one user group name can be changed at a time.'));
-			}
-			else {
-				// check if there already is hostgroup with this name, except current hostgroup
-				$groupExists = $this->get([
-					'filter' => ['name' => $data['name']],
-					'output' => ['usrgrpid'],
-					'limit' => 1
-				]);
-				$groupExists = reset($groupExists);
-				if ($groupExists && (bccomp($groupExists['usrgrpid'], $usrgrpids[0]) != 0) ) {
-					self::exception(ZBX_API_ERROR_PARAMETERS, _s('User group "%s" already exists.', $data['name']));
-				}
-			}
-		}
-
-		// update usrgrp (user group) table if there is something to update
-		$usrgrpTableUpdateData = $data;
-		unset($usrgrpTableUpdateData['usrgrpids'], $usrgrpTableUpdateData['userids'], $usrgrpTableUpdateData['rights']);
-		if (!empty($usrgrpTableUpdateData)) {
-			foreach ($usrgrpids as $usrgrpid) {
-				DB::update('usrgrp', [
-					'values' => $usrgrpTableUpdateData,
-					'where' => ['usrgrpid' => $usrgrpid],
-				]);
-			}
-		}
-
-		// check that user do not add himself to a disabled user group
-		// insert and delete user-userGroup links
-		if (isset($data['userids'])) {
-			$userids = zbx_toArray($data['userids']);
-
-			// check whether user tries to add himself to a disabled user group
-			$usrgrps = $this->get([
-				'output' => ['usrgrpid', 'gui_access', 'users_status'],
-				'usrgrpids' => $usrgrpids,
-				'selectUsers' => ['userid'],
-				'preservekeys' => true
-			]);
-			if (uint_in_array(self::$userData['userid'], $userids)) {
-				foreach ($usrgrps as $usrgrp) {
-					if (($usrgrp['gui_access'] == GROUP_GUI_ACCESS_DISABLED)
-						|| ($usrgrp['users_status'] == GROUP_STATUS_DISABLED)) {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _('User cannot add himself to a disabled group or a group with disabled GUI access.'));
-					}
-				}
-			}
-
-			$userUsergroupLinksToInsert = [];
-			$amount_of_usrgrps_to_unlink = [];
-
-			foreach ($usrgrps as $usrgrpid => $usrgrp) {
-				$usrgrp_userids = [];
-
-				foreach ($usrgrp['users'] as $user) {
-					$usrgrp_userids[$user['userid']] = true;
-				}
-
-				foreach ($userids as $userid) {
-					if (!array_key_exists($userid, $usrgrp_userids)) {
-						$userUsergroupLinksToInsert[] = [
-							'usrgrpid' => $usrgrpid,
-							'userid' => $userid
-						];
-					}
-					else {
-						unset($usrgrp_userids[$userid]);
-					}
-				}
-
-				foreach ($usrgrp_userids as $userid => $value) {
-					$amount_of_usrgrps_to_unlink[$userid] = array_key_exists($userid, $amount_of_usrgrps_to_unlink)
-						? $amount_of_usrgrps_to_unlink[$userid] + 1
-						: 1;
-				}
-			}
-
-			// link users to user groups
-			if (!empty($userUsergroupLinksToInsert)) {
-				DB::insert('users_groups', $userUsergroupLinksToInsert);
-			}
-
-			if ($amount_of_usrgrps_to_unlink) {
-				// Unlink users from user groups.
-
-				$userids_to_unlink = array_keys($amount_of_usrgrps_to_unlink);
-
-				$db_users = API::User()->get([
-					'output' => ['userid', 'alias'],
-					'userids' => $userids_to_unlink,
-					'selectUsrgrps' => ['usrgrpid']
-				]);
-
-				foreach ($db_users as $user) {
-					if (count($user['usrgrps']) <= $amount_of_usrgrps_to_unlink[$user['userid']]) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('User "%1$s" cannot be without user group.', $user['alias'])
-						);
-					}
-				}
-
-				DB::delete('users_groups', [
-					'userid' => $userids_to_unlink,
-					'usrgrpid' => $usrgrpids
-				]);
-			}
-		}
-
-		// link rights to user groups
-		// update permissions to right-userGroup links
-		// unlink rights from user groups (permissions)
-		if (isset($data['rights'])) {
-			$rights = zbx_toArray($data['rights']);
-
-			// get already linked rights
-			$linkedRights = [];
-			$sql = 'SELECT groupid,permission,id'.
-				' FROM rights'.
-				' WHERE '.dbConditionInt('groupid', $usrgrpids);
-			$linkedRightsDb = DBselect($sql);
-			while ($link = DBfetch($linkedRightsDb)) {
-				if (!isset($linkedRights[$link['groupid']])) {
-					$linkedRights[$link['groupid']] = [];
-				}
-				$linkedRights[$link['groupid']][$link['id']] = $link['permission'];
-			}
-
-			// get right-userGroup links to insert
-			// get right-userGroup links to update permissions
-			// get rightIds to unlink rights from user groups
-			$rightUsergroupLinksToInsert = [];
-			$rightUsergroupLinksToUpdate = [];
-			$rightIdsToUnlink = [];
-			foreach ($usrgrpids as $usrgrpid) {
-				foreach ($rights as $right) {
-					if (!isset($linkedRights[$usrgrpid][$right['id']])) {
-						$rightUsergroupLinksToInsert[] = [
-							'groupid' => $usrgrpid,
-							'id' => $right['id'],
-							'permission' => $right['permission'],
-						];
-					}
-					elseif ($linkedRights[$usrgrpid][$right['id']] != $right['permission']) {
-						$rightUsergroupLinksToUpdate[] = [
-							'values' => ['permission' => $right['permission']],
-							'where' => ['groupid' => $usrgrpid, 'id' => $right['id']],
-						];
-					}
-					unset($linkedRights[$usrgrpid][$right['id']]);
-				}
-
-				if (isset($linkedRights[$usrgrpid]) && !empty($linkedRights[$usrgrpid])) {
-					$rightIdsToUnlink = array_merge($rightIdsToUnlink, array_keys($linkedRights[$usrgrpid]));
-				}
-			}
-
-			// link rights to user groups
-			if (!empty($rightUsergroupLinksToInsert)) {
-				DB::insert('rights', $rightUsergroupLinksToInsert);
-			}
-
-			// unlink rights from user groups
-			if (!empty($rightIdsToUnlink)) {
-				DB::delete('rights', [
-					'id' => $rightIdsToUnlink,
-					'groupid' => $usrgrpids,
-				]);
-			}
-
-			// update right-userGroup permissions
-			if (!empty($rightUsergroupLinksToUpdate)) {
-				DB::update('rights', $rightUsergroupLinksToUpdate);
-			}
-		}
-
-		return ['usrgrpids' => $usrgrpids];
-	}
-
 	/**
 	 * @param array $usrgrpids
 	 *
@@ -1088,13 +920,10 @@ class CUserGroup extends CApiService {
 		}
 
 		// Check if user group are used in config.
-		$config = select_config();
-
-		if (array_key_exists($config['alert_usrgrpid'], $db_usrgrps)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-				'User group "%1$s" is used in configuration for database down messages.',
-				$db_usrgrps[$config['alert_usrgrpid']]['name']
-			));
+		if (array_key_exists(CSettingsHelper::get(CSettingsHelper::ALERT_USRGRPID), $db_usrgrps)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('User group "%1$s" is used in configuration for database down messages.', $db_usrgrps[CSettingsHelper::get(CSettingsHelper::ALERT_USRGRPID)]['name'])
+			);
 		}
 
 		$this->checkUsersWithoutGroups($usrgrps);
@@ -1107,10 +936,14 @@ class CUserGroup extends CApiService {
 		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'usrgrpid', 'userid', 'users_groups');
 
+			$get_access = ($this->outputIsRequested('gui_access', $options['selectUsers'])
+				|| $this->outputIsRequested('debug_mode', $options['selectUsers'])
+				|| $this->outputIsRequested('users_status', $options['selectUsers'])) ? true : null;
+
 			$dbUsers = API::User()->get([
 				'output' => $options['selectUsers'],
 				'userids' => $relationMap->getRelatedIds(),
-				'getAccess' => ($options['selectUsers'] == API_OUTPUT_EXTEND) ? true : null,
+				'getAccess' => $get_access,
 				'preservekeys' => true
 			]);
 
@@ -1156,6 +989,42 @@ class CUserGroup extends CApiService {
 			$result = $relationMap->mapMany($result, $db_rights, 'rights');
 		}
 
+		// Adding usergroup tag filters.
+		if ($options['selectTagFilters'] !== null && $options['selectTagFilters'] != API_OUTPUT_COUNT) {
+			foreach ($result as &$usrgrp) {
+				$usrgrp['tag_filters'] = [];
+			}
+			unset($usrgrp);
+
+			if (is_array($options['selectTagFilters'])) {
+				$output_fields = [];
+
+				foreach ($this->outputExtend($options['selectTagFilters'], ['usrgrpid']) as $field) {
+					if ($this->hasField($field, 'tag_filter')) {
+						$output_fields[$field] = $this->fieldId($field, 't');
+					}
+				}
+
+				$output_fields = implode(',', $output_fields);
+			}
+			else {
+				$output_fields = 't.*';
+			}
+
+			$db_tag_filters = DBselect(
+				'SELECT '.$output_fields.
+				' FROM tag_filter t'.
+				' WHERE '.dbConditionInt('t.usrgrpid', array_keys($result))
+			);
+
+			while ($db_tag_filter = DBfetch($db_tag_filters)) {
+				$usrgrpid = $db_tag_filter['usrgrpid'];
+				unset($db_tag_filter['tag_filterid'], $db_tag_filter['usrgrpid']);
+
+				$result[$usrgrpid]['tag_filters'][] = $db_tag_filter;
+			}
+		}
+
 		return $result;
 	}
 }
