@@ -28,7 +28,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 */
 	protected $configs = [
 		'scriptConfirmation' => [
-			'types' => ['host', 'interfaceWithoutPort', 'user'],
+			'types' => ['host', 'interfaceWithoutPort', 'user', 'user_data'],
 			'method' => 'resolveTexts'
 		],
 		'httpTestName' => [
@@ -43,6 +43,26 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'types' => ['host', 'user'],
 			'method' => 'resolveTexts'
 		],
+		'hostInterfaceDetailsSecurityname' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsAuthPassphrase' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsPrivPassphrase' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsContextName' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
+		'hostInterfaceDetailsCommunity' => [
+			'types' => ['user'],
+			'method' => 'resolveTexts'
+		],
 		'hostInterfacePort' => [
 			'types' => ['user'],
 			'method' => 'resolveTexts'
@@ -115,6 +135,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$types['macros']['interface'] = ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}'];
 		}
 
+		if ($this->isTypeAvailable('user_data')) {
+			$types['macros']['user_data'] = ['{USER.ALIAS}', '{USER.FULLNAME}', '{USER.NAME}', '{USER.SURNAME}'];
+		}
+
 		if ($this->isTypeAvailable('user')) {
 			$types['usermacros'] = true;
 		}
@@ -125,7 +149,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$interface_hostids = [];
 
 		foreach ($data as $hostid => $texts) {
-			$matched_macros = $this->extractMacros($texts, $types);
+			$matched_macros = self::extractMacros($texts, $types);
 
 			if (array_key_exists('macros', $matched_macros)) {
 				if (array_key_exists('host', $matched_macros['macros']) && $matched_macros['macros']['host']) {
@@ -148,6 +172,39 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					}
 					$interface_hostids[$hostid] = true;
 				}
+
+				if (array_key_exists('user_data', $matched_macros['macros'])
+						&& $matched_macros['macros']['user_data']) {
+					foreach ($matched_macros['macros']['user_data'] as $macro) {
+						switch ($macro) {
+							case '{USER.ALIAS}':
+								$macros[$hostid][$macro] = CWebUser::$data['alias'];
+								break;
+
+							case '{USER.FULLNAME}':
+								$fullname = [];
+
+								foreach (['name', 'surname'] as $field) {
+									if (CWebUser::$data[$field] !== '') {
+										$fullname[] = CWebUser::$data[$field];
+									}
+								}
+
+								$macros[$hostid][$macro] = $fullname
+									? implode(' ', array_merge($fullname, ['('.CWebUser::$data['alias'].')']))
+									: CWebUser::$data['alias'];
+								break;
+
+							case '{USER.NAME}':
+								$macros[$hostid][$macro] = CWebUser::$data['name'];
+								break;
+
+							case '{USER.SURNAME}':
+								$macros[$hostid][$macro] = CWebUser::$data['surname'];
+								break;
+						}
+					}
+				}
 			}
 
 			if ($this->isTypeAvailable('user') && $matched_macros['usermacros']) {
@@ -323,7 +380,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'host' => [],
 			'interface' => [],
 			'item' => [],
-			'references' => []
+			'references' => [],
+			'log' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -332,7 +390,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -342,11 +403,13 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		];
 
 		$original_triggers = $triggers;
-		$triggers = $this->resolveTriggerExpressionUserMacro($triggers);
+		$triggers = $this->resolveTriggerExpressions($triggers,
+			['resolve_usermacros' => true, 'resolve_functionids' => false]
+		);
 
 		// Find macros.
 		foreach ($triggers as $triggerid => $trigger) {
-			$matched_macros = $this->extractMacros([$trigger['description']], $types);
+			$matched_macros = self::extractMacros([$trigger['description']], $types);
 
 			if (!$options['references_only']) {
 				$functionids = $this->findFunctions($trigger['expression']);
@@ -375,6 +438,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					}
 				}
 
+				foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+					$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+					if (array_key_exists($data['f_num'], $functionids)) {
+						$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+					}
+				}
+
 				foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 					$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -408,6 +479,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 			$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 			$macro_values = $this->getItemMacros($macros['item'], $macro_values);
+			$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 			if ($usermacros) {
 				// Get hosts for triggers.
@@ -474,7 +546,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macros = [
 			'host' => [],
 			'interface' => [],
-			'item' => []
+			'item' => [],
+			'log' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -483,7 +556,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{IPADDRESS}', '{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -500,7 +576,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				$texts[] = $trigger[$source];
 			}
 
-			$matched_macros = $this->extractMacros($texts, $types);
+			$matched_macros = self::extractMacros($texts, $types);
 
 			foreach ($matched_macros['macros_n']['host'] as $token => $data) {
 				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
@@ -526,6 +602,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				}
 			}
 
+			foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+				if (array_key_exists($data['f_num'], $functionids)) {
+					$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+				}
+			}
+
 			foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 				$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -547,6 +631,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 		$macro_values = $this->getItemMacros($macros['item'], $macro_values, $triggers, $options);
+		$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -630,7 +715,8 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'host' => [],
 			'interface' => [],
 			'item' => [],
-			'event' => []
+			'event' => [],
+			'log' => []
 		];
 		$usermacros = [];
 		$macro_values = [];
@@ -643,7 +729,10 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			'macros_n' => [
 				'host' => ['{HOST.ID}', '{HOST.HOST}', '{HOST.NAME}'],
 				'interface' => ['{HOST.IP}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
-				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
+				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}'],
+				'log' => ['{ITEM.LOG.DATE}', '{ITEM.LOG.TIME}', '{ITEM.LOG.AGE}', '{ITEM.LOG.SOURCE}',
+					'{ITEM.LOG.SEVERITY}', '{ITEM.LOG.NSEVERITY}', '{ITEM.LOG.EVENTID}'
+				]
 			],
 			'macro_funcs_n' => [
 				'item' => ['{ITEM.LASTVALUE}', '{ITEM.VALUE}']
@@ -655,7 +744,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 		// Find macros.
 		$functionids = $this->findFunctions($trigger['expression']);
-		$matched_macros = $this->extractMacros([$trigger['url']], $types);
+		$matched_macros = self::extractMacros([$trigger['url']], $types);
 
 		foreach ($matched_macros['macros']['trigger'] as $macro) {
 			$macro_values[$triggerid][$macro] = $triggerid;
@@ -692,6 +781,14 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 		}
 
+		foreach ($matched_macros['macros_n']['log'] as $token => $data) {
+			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
+
+			if (array_key_exists($data['f_num'], $functionids)) {
+				$macros['log'][$functionids[$data['f_num']]][$data['macro']][] = ['token' => $token];
+			}
+		}
+
 		foreach ($matched_macros['macro_funcs_n']['item'] as $token => $data) {
 			$macro_values[$triggerid][$token] = UNRESOLVED_MACRO_STRING;
 
@@ -712,6 +809,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$macro_values = $this->getHostMacros($macros['host'], $macro_values);
 		$macro_values = $this->getIpMacros($macros['interface'], $macro_values);
 		$macro_values = $this->getItemMacros($macros['item'], $macro_values);
+		$macro_values = $this->getItemLogMacros($macros['log'], $macro_values);
 
 		if ($usermacros) {
 			// Get hosts for triggers.
@@ -753,41 +851,70 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	 * Purpose: Translate {10}>10 to something like {localhost:system.cpu.load.last()}>10
 	 *
 	 * @param array  $triggers
-	 * @param string $triggers[][<sources>]			see options['source']
+	 * @param string $triggers[][<sources>]			  See options['source']
 	 * @param array  $options
-	 * @param bool   $options['html']				returns formatted trigger expression
-	 * @param bool   $options['resolve_usermacros']	resolve user macros
-	 * @param bool   $options['resolve_macros']		resolve macros in item keys and functions
-	 * @param array  $options['sources']			an array of the field names
+	 * @param bool   $options['html']				  Returns formatted trigger expression. Default: false.
+	 * @param bool   $options['resolve_usermacros']	  Resolve user macros. Default: false.
+	 * @param bool   $options['resolve_macros']		  Resolve macros in item keys and functions. Default: false.
+	 * @param bool   $options['resolve_functionids']  Resolve finctionid macros. Default: true.
+	 * @param array  $options['sources']			  An array of the field names. Default: ['expression'].
 	 *
 	 * @return string|array
 	 */
 	public function resolveTriggerExpressions(array $triggers, array $options) {
+		$options += [
+			'html' => false,
+			'resolve_usermacros' => false,
+			'resolve_macros' => false,
+			'resolve_functionids' => true,
+			'sources' => ['expression']
+		];
+
 		$functionids = [];
 		$usermacros = [];
 		$macro_values = [];
+		$usermacro_values = [];
 
 		$types = [
 			'macros' => [
 				'trigger' => ['{TRIGGER.VALUE}']
 			],
-			'functionids' => true,
 			'lldmacros' => true,
 			'usermacros' => true
 		];
 
+		$expression_data = new CTriggerExpression(['collapsed_expression' => true]);
+
 		// Find macros.
 		foreach ($triggers as $key => $trigger) {
+			$functionid_macros = [];
 			$texts = [];
 			foreach ($options['sources'] as $source) {
-				$texts[] = $trigger[$source];
+				if ($trigger[$source] !== '' && ($result = $expression_data->parse($trigger[$source])) !== false) {
+					foreach ($result->getTokens() as $token) {
+						switch ($token['type']) {
+							case CTriggerExprParserResult::TOKEN_TYPE_FUNCTIONID_MACRO:
+								$functionid_macros[$token['value']] = null;
+								break;
+
+							case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+								$texts[] = $token['value'];
+								break;
+
+							case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+								$texts[] = $token['data']['string'];
+								break;
+						}
+					}
+				}
 			}
 
-			$matched_macros = $this->extractMacros($texts, $types);
+			$matched_macros = self::extractMacros($texts, $types);
 
-			$macro_values[$key] = $matched_macros['functionids'];
+			$macro_values[$key] = $functionid_macros;
+			$usermacro_values[$key] = [];
 
-			foreach (array_keys($matched_macros['functionids']) as $macro) {
+			foreach (array_keys($functionid_macros) as $macro) {
 				$functionids[] = substr($macro, 1, -1); // strip curly braces
 			}
 
@@ -800,154 +927,175 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		if ($functionids) {
 			$functions = [];
 
-			// Selecting functions.
-			$result = DBselect(
-				'SELECT f.functionid,f.itemid,f.name,f.parameter'.
-				' FROM functions f'.
-				' WHERE '.dbConditionInt('f.functionid', $functionids)
-			);
+			if ($options['resolve_functionids']) {
+				// Selecting functions.
+				$result = DBselect(
+					'SELECT f.functionid,f.itemid,f.name,f.parameter'.
+					' FROM functions f'.
+					' WHERE '.dbConditionInt('f.functionid', $functionids)
+				);
 
-			$hostids = [];
-			$itemids = [];
-			$hosts = [];
-			$items = [];
-
-			while ($row = DBfetch($result)) {
-				$itemids[$row['itemid']] = true;
-				$row['function'] = $row['name'];
-				unset($row['name']);
-
-				$functions['{'.$row['functionid'].'}'] = $row;
-				unset($functions['{'.$row['functionid'].'}']['functionid']);
-			}
-
-			// Selecting items.
-			if ($itemids) {
-				if ($options['html']) {
-					$sql = 'SELECT i.itemid,i.hostid,i.key_,i.type,i.flags,i.status,ir.state,id.parent_itemid'.
-						' FROM items i'.
-							' LEFT JOIN item_rtdata ir ON i.itemid=ir.itemid'.
-							' LEFT JOIN item_discovery id ON i.itemid=id.itemid'.
-						' WHERE '.dbConditionInt('i.itemid', array_keys($itemids));
-				}
-				else {
-					$sql = 'SELECT i.itemid,i.hostid,i.key_'.
-						' FROM items i'.
-						' WHERE '.dbConditionInt('i.itemid', array_keys($itemids));
-				}
-				$result = DBselect($sql);
+				$hostids = [];
+				$itemids = [];
+				$hosts = [];
+				$items = [];
 
 				while ($row = DBfetch($result)) {
-					$hostids[$row['hostid']] = true;
-					$items[$row['itemid']] = $row;
-				}
-			}
+					$itemids[$row['itemid']] = true;
+					$row['function'] = $row['name'];
+					unset($row['name']);
 
-			// Selecting hosts.
-			if ($hostids) {
+					$functions['{'.$row['functionid'].'}'] = $row;
+					unset($functions['{'.$row['functionid'].'}']['functionid']);
+				}
+
+				// Selecting items.
+				if ($itemids) {
+					if ($options['html']) {
+						$sql = 'SELECT i.itemid,i.hostid,i.key_,i.type,i.flags,i.status,ir.state,id.parent_itemid'.
+							' FROM items i'.
+								' LEFT JOIN item_rtdata ir ON i.itemid=ir.itemid'.
+								' LEFT JOIN item_discovery id ON i.itemid=id.itemid'.
+							' WHERE '.dbConditionInt('i.itemid', array_keys($itemids));
+					}
+					else {
+						$sql = 'SELECT i.itemid,i.hostid,i.key_'.
+							' FROM items i'.
+							' WHERE '.dbConditionInt('i.itemid', array_keys($itemids));
+					}
+					$result = DBselect($sql);
+
+					while ($row = DBfetch($result)) {
+						$hostids[$row['hostid']] = true;
+						$items[$row['itemid']] = $row;
+					}
+				}
+
+				// Selecting hosts.
+				if ($hostids) {
+					$result = DBselect(
+						'SELECT h.hostid,h.host FROM hosts h WHERE '.dbConditionInt('h.hostid', array_keys($hostids))
+					);
+
+					while ($row = DBfetch($result)) {
+						$hosts[$row['hostid']] = $row;
+					}
+				}
+
+				if ($options['resolve_macros']) {
+					$items = $this->resolveItemKeys($items);
+					foreach ($items as &$item) {
+						$item['key_'] = $item['key_expanded'];
+						unset($item['key_expanded']);
+					}
+					unset($item);
+				}
+
+				foreach ($functions as $macro => &$function) {
+					if (!array_key_exists($function['itemid'], $items)) {
+						unset($functions[$macro]);
+						continue;
+					}
+					$item = $items[$function['itemid']];
+
+					if (!array_key_exists($item['hostid'], $hosts)) {
+						unset($functions[$macro]);
+						continue;
+					}
+					$host = $hosts[$item['hostid']];
+
+					$function['hostid'] = $item['hostid'];
+					$function['host'] = $host['host'];
+					$function['key_'] = $item['key_'];
+					if ($options['html']) {
+						$function['type'] = $item['type'];
+						$function['flags'] = $item['flags'];
+						$function['status'] = $item['status'];
+						$function['state'] = $item['state'];
+						$function['parent_itemid'] = $item['parent_itemid'];
+					}
+				}
+				unset($function);
+
+				if ($options['resolve_macros']) {
+					$functions = $this->resolveFunctionParameters($functions);
+					foreach ($functions as &$function) {
+						$function['parameter'] = $function['parameter_expanded'];
+						unset($function['parameter_expanded']);
+					}
+					unset($function);
+				}
+
+				foreach ($macro_values as &$macros) {
+					foreach ($macros as $macro => &$value) {
+						if (array_key_exists($macro, $functions)) {
+							$function = $functions[$macro];
+
+							if ($options['html']) {
+								$style = ($function['status'] == ITEM_STATUS_ACTIVE)
+									? ($function['state'] == ITEM_STATE_NORMAL) ? ZBX_STYLE_GREEN : ZBX_STYLE_GREY
+									: $style = ZBX_STYLE_RED;
+
+								if ($function['type'] == ITEM_TYPE_HTTPTEST) {
+									$link = (new CSpan($function['host'].':'.$function['key_']))->addClass($style);
+								}
+								elseif ($function['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
+									$link = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+										? (new CLink($function['host'].':'.$function['key_'],
+											'disc_prototypes.php?form=update&itemid='.$function['itemid'].
+											'&parent_discoveryid='.$function['parent_itemid']
+										))
+											->addClass(ZBX_STYLE_LINK_ALT)
+											->addClass($style)
+										: (new CSpan($function['host'].':'.$function['key_']))
+											->addClass($style);
+								}
+								else {
+									$link = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_HOSTS)
+									? (new CLink($function['host'].':'.$function['key_'],
+										'items.php?form=update&itemid='.$function['itemid']
+									))
+										->addClass(ZBX_STYLE_LINK_ALT)
+										->setAttribute('data-itemid', $function['itemid'])
+										->addClass($style)
+									: (new CSpan($function['host'].':'.$function['key_']))
+										->addClass($style);
+								}
+
+								$value = [
+									'{', $link, '.', bold($function['function'].'('), $function['parameter'], bold(')'), '}'
+								];
+							}
+							else {
+								$value = '{'.
+									$function['host'].':'.
+									$function['key_'].'.'.
+									$function['function'].'('.$function['parameter'].')'.
+								'}';
+							}
+						}
+						else {
+							$value = $options['html'] ? (new CSpan('*ERROR*'))->addClass(ZBX_STYLE_RED) : '*ERROR*';
+						}
+					}
+					unset($value);
+				}
+				unset($macros);
+			}
+			else {
+				// Selecting functions.
 				$result = DBselect(
-					'SELECT h.hostid,h.host FROM hosts h WHERE '.dbConditionInt('h.hostid', array_keys($hostids))
+					'SELECT f.functionid,i.hostid'.
+					' FROM functions f,items i'.
+					' WHERE f.itemid=i.itemid'.
+						' AND '.dbConditionInt('f.functionid', $functionids)
 				);
 
 				while ($row = DBfetch($result)) {
-					$hosts[$row['hostid']] = $row;
+					$functions['{'.$row['functionid'].'}'] = ['hostid' => $row['hostid']];
 				}
 			}
 
-			if ($options['resolve_macros']) {
-				$items = $this->resolveItemKeys($items);
-				foreach ($items as &$item) {
-					$item['key_'] = $item['key_expanded'];
-					unset($item['key_expanded']);
-				}
-				unset($item);
-			}
-
-			foreach ($functions as $macro => &$function) {
-				if (!array_key_exists($function['itemid'], $items)) {
-					unset($functions[$macro]);
-					continue;
-				}
-				$item = $items[$function['itemid']];
-
-				if (!array_key_exists($item['hostid'], $hosts)) {
-					unset($functions[$macro]);
-					continue;
-				}
-				$host = $hosts[$item['hostid']];
-
-				$function['hostid'] = $item['hostid'];
-				$function['host'] = $host['host'];
-				$function['key_'] = $item['key_'];
-				if ($options['html']) {
-					$function['type'] = $item['type'];
-					$function['flags'] = $item['flags'];
-					$function['status'] = $item['status'];
-					$function['state'] = $item['state'];
-					$function['parent_itemid'] = $item['parent_itemid'];
-				}
-			}
-			unset($function);
-
-			if ($options['resolve_macros']) {
-				$functions = $this->resolveFunctionParameters($functions);
-				foreach ($functions as &$function) {
-					$function['parameter'] = $function['parameter_expanded'];
-					unset($function['parameter_expanded']);
-				}
-				unset($function);
-			}
-
-			foreach ($macro_values as &$macros) {
-				foreach ($macros as $macro => &$value) {
-					if (array_key_exists($macro, $functions)) {
-						$function = $functions[$macro];
-
-						if ($options['html']) {
-							$style = ($function['status'] == ITEM_STATUS_ACTIVE)
-								? ($function['state'] == ITEM_STATE_NORMAL) ? ZBX_STYLE_GREEN : ZBX_STYLE_GREY
-								: $style = ZBX_STYLE_RED;
-
-							if ($function['type'] == ITEM_TYPE_HTTPTEST) {
-								$link = (new CSpan($function['host'].':'.$function['key_']))->addClass($style);
-							}
-							elseif ($function['flags'] == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
-								$link = (new CLink($function['host'].':'.$function['key_'],
-									'disc_prototypes.php?form=update&itemid='.$function['itemid'].
-									'&parent_discoveryid='.$function['parent_itemid']
-								))
-									->addClass(ZBX_STYLE_LINK_ALT)
-									->addClass($style);
-							}
-							else {
-								$link = (new CLink($function['host'].':'.$function['key_'],
-									'items.php?form=update&itemid='.$function['itemid']
-								))
-									->addClass(ZBX_STYLE_LINK_ALT)
-									->setAttribute('data-itemid', $function['itemid'])
-									->addClass($style);
-							}
-
-							$value = [
-								'{', $link, '.', bold($function['function'].'('), $function['parameter'], bold(')'), '}'
-							];
-						}
-						else {
-							$value = '{'.
-								$function['host'].':'.
-								$function['key_'].'.'.
-								$function['function'].'('.$function['parameter'].')'.
-							'}';
-						}
-					}
-					else {
-						$value = $options['html'] ? (new CSpan('*ERROR*'))->addClass(ZBX_STYLE_RED) : '*ERROR*';
-					}
-				}
-				unset($value);
-			}
-			unset($macros);
-
 			foreach ($usermacros as $key => &$usermacros_data) {
 				foreach (array_keys($macro_values[$key]) as $macro) {
 					if (array_key_exists($macro, $functions)) {
@@ -959,112 +1107,70 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			unset($usermacros_data);
 
 			// Get user macros values.
-			foreach ($this->getUserMacros($usermacros) as $key => $usermacros_data) {
-				$macro_values[$key] = array_key_exists($key, $macro_values)
-					? array_merge($macro_values[$key], $usermacros_data['macros'])
-					: $usermacros_data['macros'];
+			foreach ($this->getUserMacros($usermacros, true) as $key => $usermacros_data) {
+				$usermacro_values[$key] = $usermacros_data['macros'];
 			}
 		}
 
-		$types = $this->transformToPositionTypes($types);
-
 		// Replace macros to value.
-		foreach ($triggers as $key => &$trigger) {
+		foreach ($triggers as $key => $trigger) {
 			foreach ($options['sources'] as $source) {
-				$matched_macros = $this->getMacroPositions($trigger[$source], $types);
+				if ($trigger[$source] === '' || ($result = $expression_data->parse($trigger[$source])) === false) {
+					continue;
+				}
 
-				if ($options['html']) {
-					$expression = [];
-					$pos_left = 0;
+				$expression = [];
+				$pos_left = 0;
 
-					foreach ($matched_macros as $pos => $macro) {
-						if (array_key_exists($macro, $macro_values[$key])) {
-							if ($pos_left != $pos) {
-								$expression[] = substr($trigger[$source], $pos_left, $pos - $pos_left);
+				foreach ($result->getTokens() as $token) {
+					switch ($token['type']) {
+						case CTriggerExprParserResult::TOKEN_TYPE_FUNCTIONID_MACRO:
+							if (!$options['resolve_functionids']) {
+								continue 2;
 							}
+							// break; is not missing here
 
-							$expression[] = $macro_values[$key][$macro];
-
-							$pos_left = $pos + strlen($macro);
-						}
+						case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+						case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+							if ($pos_left != $token['pos']) {
+								$expression[] = substr($trigger[$source], $pos_left, $token['pos'] - $pos_left);
+							}
+							$pos_left = $token['pos'] + $token['length'];
+							break;
 					}
+
+					switch ($token['type']) {
+						case CTriggerExprParserResult::TOKEN_TYPE_FUNCTIONID_MACRO:
+							$expression[] = $macro_values[$key][$token['value']];
+							break;
+
+						case CTriggerExprParserResult::TOKEN_TYPE_USER_MACRO:
+							if (array_key_exists($token['value'], $usermacro_values[$key])) {
+								$expression[] =
+									CTriggerExpression::quoteString($usermacro_values[$key][$token['value']], false);
+							}
+							else {
+								$expression[] = ($options['resolve_usermacros'] && $options['html'])
+									? (new CSpan('*ERROR*'))->addClass(ZBX_STYLE_RED)
+									: $token['value'];
+							}
+							break;
+
+						case CTriggerExprParserResult::TOKEN_TYPE_STRING:
+							$string = strtr($token['data']['string'], $usermacro_values[$key]);
+							$expression[] = CTriggerExpression::quoteString($string, false, true);
+							break;
+
+					}
+				}
+
+				if ($pos_left != strlen($trigger[$source])) {
 					$expression[] = substr($trigger[$source], $pos_left);
-
-					$trigger[$source] = $expression;
-				}
-				else {
-					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-						if (array_key_exists($macro, $macro_values[$key])) {
-							$trigger[$source] =
-								substr_replace($trigger[$source], $macro_values[$key][$macro], $pos, strlen($macro));
-						}
-					}
 				}
+
+				$triggers[$key][$source] = $options['html'] ? $expression : implode('', $expression);
 			}
 		}
-		unset($trigger);
-
-		return $triggers;
-	}
-
-	/**
-	 * Resolve user macros in trigger expression.
-	 *
-	 * @param string $triggers[$triggerid]['expression']
-	 *
-	 * @return array
-	 */
-	public function resolveTriggerExpressionUserMacro(array $triggers) {
-		$usermacros = [];
-		$macro_values = [];
-
-		$types = ['usermacros' => true];
-
-		// Find macros.
-		foreach ($triggers as $triggerid => $trigger) {
-			$matched_macros = $this->extractMacros([$trigger['expression']], $types);
-
-			if ($matched_macros['usermacros']) {
-				$usermacros[$triggerid] = ['hostids' => [], 'macros' => $matched_macros['usermacros']];
-			}
-		}
-
-		if ($usermacros) {
-			// Get hosts for triggers.
-			$db_triggers = API::Trigger()->get([
-				'output' => [],
-				'selectHosts' => ['hostid'],
-				'triggerids' => array_keys($usermacros),
-				'preservekeys' => true
-			]);
-
-			foreach ($usermacros as $triggerid => &$usermacros_data) {
-				if (array_key_exists($triggerid, $db_triggers)) {
-					$usermacros_data['hostids'] = zbx_objectValues($db_triggers[$triggerid]['hosts'], 'hostid');
-				}
-			}
-			unset($usermacros_data);
-
-			// Get user macros values.
-			foreach ($this->getUserMacros($usermacros) as $triggerid => $usermacros_data) {
-				$macro_values[$triggerid] = array_key_exists($triggerid, $macro_values)
-					? array_merge($macro_values[$triggerid], $usermacros_data['macros'])
-					: $usermacros_data['macros'];
-			}
-		}
-
-		$types = $this->transformToPositionTypes($types);
-
-		// Replace macros to value.
-		foreach ($triggers as $triggerid => &$trigger) {
-			$matched_macros = $this->getMacroPositions($trigger['expression'], $types);
-
-			foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-				$trigger['expression'] =
-					substr_replace($trigger['expression'], $macro_values[$triggerid][$macro], $pos, strlen($macro));
-			}
-		}
-		unset($trigger);
 
 		return $triggers;
 	}
@@ -1339,7 +1445,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$usermacros = [];
 
 		foreach ($items as $key => $item) {
-			$matched_macros = $this->extractMacros([$item['name_expanded']], $types);
+			$matched_macros = self::extractMacros([$item['name_expanded']], $types);
 
 			if ($matched_macros['usermacros']) {
 				$usermacros[$key] = ['hostids' => [$item['hostid']], 'macros' => $matched_macros['usermacros']];
@@ -1590,6 +1696,41 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		return $items;
 	}
 
+	/**
+	 * Resolve macros in item description.
+	 *
+	 * @param array  $items
+	 * @param string $items[n]['hostid']
+	 * @param string $items[n]['description']
+	 *
+	 * @return array
+	 */
+	public function resolveItemDescriptions(array $items): array {
+		$types = ['usermacros' => true];
+		$macro_values = [];
+		$usermacros = [];
+
+		foreach ($items as $key => $item) {
+			$matched_macros = self::extractMacros([$item['description']], $types);
+
+			if ($matched_macros['usermacros']) {
+				$usermacros[$key] = ['hostids' => [$item['hostid']], 'macros' => $matched_macros['usermacros']];
+			}
+		}
+
+		foreach ($this->getUserMacros($usermacros) as $key => $usermacros_data) {
+			$macro_values[$key] = array_key_exists($key, $macro_values)
+				? array_merge($macro_values[$key], $usermacros_data['macros'])
+				: $usermacros_data['macros'];
+		}
+
+		foreach ($macro_values as $key => $macro_value) {
+			$items[$key]['description'] = strtr($items[$key]['description'], $macro_value);
+		}
+
+		return $items;
+	}
+
 	/**
 	 * Resolve item delay macros, item history and item trend macros.
 	 *
@@ -1616,7 +1757,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				$texts[] = $value[$source];
 			}
 
-			$matched_macros = $this->extractMacros($texts, $types);
+			$matched_macros = self::extractMacros($texts, $types);
 
 			if ($matched_macros['usermacros']) {
 				$usermacros[$key] = [
@@ -1779,7 +1920,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				}
 				else {
 					$value = getItemFunctionalValue($item, $function,
-							$expressionData->expressions[0]['functionParamList'][0]
+						$expressionData->expressions[0]['functionParamList'][0]
 					);
 				}
 
@@ -2005,7 +2146,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 
 			// Extract macros from collected strings.
-			$matched_macros = $this->extractMacros($texts, $types_by_selement_type[$selement_type]);
+			$matched_macros = self::extractMacros($texts, $types_by_selement_type[$selement_type]);
 
 			// Map extracted macros to map elements.
 			if (array_key_exists('macros', $matched_macros)) {
@@ -2205,7 +2346,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 						 * For example, having a trigger like: '{host:item.change()}=1 or {$MACRO: "{1234} ..."}' gives
 						 * incorrect result of 'functionids' without requesting to parse macros and user macros as well.
 						 */
-						$matched_functionids = $this->extractMacros([$trigger['expression']], [
+						$matched_functionids = self::extractMacros([$trigger['expression']], [
 							'macros' => [
 								'trigger' => ['{TRIGGER.VALUE}']
 							],
@@ -2298,19 +2439,19 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 					case 'HOST.IP':
 					case 'IPADDRESS': // deprecated
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['ip'];
 						}
 						break;
 
 					case 'HOST.DNS':
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['dns'];
 						}
 						break;
 
 					case 'HOST.CONN':
-						if ($host) {
+						if ($host && $host['interface']) {
 							$matched_macro['value'] = $host['interface']['conn'];
 						}
 						break;
@@ -2562,7 +2703,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 
 			$num = 0;
-			$matched_macros = $this->extractMacros([$trigger['expression'].$trigger['recovery_expression']], $types);
+			$matched_macros = self::extractMacros([$trigger['expression'].$trigger['recovery_expression']], $types);
 
 			foreach (array_keys($matched_macros['functionids']) as $macro) {
 				$functionid = substr($macro, 1, -1); // strip curly braces
@@ -2607,22 +2748,26 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 	}
 
 	/**
-	 * Extract macros from properties used for preprocessing step test and find effective values.
+	 * Extract macros from item property fields and apply effective value for each of extracted macros.
+	 * Each type of macros are extracted separately because there are fields that support only LLD macros and doesn't
+	 * support user macros.
 	 *
 	 * @param array  $data
 	 * @param string $data['steps']                              Preprocessing steps details.
 	 * @param string $data['steps'][]['params']                  Preprocessing step parameters.
 	 * @param string $data['steps'][]['error_handler_params]     Preprocessing steps error handle parameters.
 	 * @param string $data['delay']                              Update interval value.
-	 * @param string $data['hostids']                            Hostid for which tested item belongs to.
-	 * @param bool   $support_lldmacros                          Enable or disable LLD macro selection.
+	 * @param array  $data['supported_macros']                   Supported macros.
+	 * @param bool   $data['support_lldmacros']                  Either LLD macros need to be extracted.
+	 * @param array  $data['texts_support_macros']               List of texts potentially could contain macros.
+	 * @param array  $data['texts_support_user_macros']          List of texts potentially could contain user macros.
+	 * @param array  $data['texts_support_lld_macros']           List of texts potentially could contain LLD macros.
+	 * @param int    $data['hostids']                            Hostid for which tested item belongs to.
 	 *
 	 * @return array
 	 */
-	public function extractMacrosFromPreprocessingSteps(array $data, $support_lldmacros = false) {
-		$types = $support_lldmacros
-			? ['usermacros' => true, 'lldmacros' => true]
-			: ['usermacros' => true];
+	public function extractItemTestMacros(array $data) {
+		$macros = [];
 		$delay_macro = $data['delay'];
 
 		$texts = [];
@@ -2645,24 +2790,45 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			}
 		}
 
-		$matched_macros = $this->extractMacros($texts, $types);
-		$usermacros = [[
-			'macros' => $matched_macros['usermacros'],
-			'hostids' =>  $data['hostid']
-				? [$data['hostid']]
-				: []
-		]];
+		// Extract macros.
+		if ($data['supported_macros']) {
+			$matched_macros = self::extractMacros($data['texts_support_macros'],
+				['macros' => $data['supported_macros']]
+			);
 
-		$macros = [];
-		if ($support_lldmacros) {
-			foreach (array_keys($matched_macros['lldmacros']) as $lldmacro) {
-				$macros[$lldmacro] = $lldmacro;
+			foreach ($matched_macros['macros'] as $type => $matches) {
+				foreach ($matches as $macro) {
+					$macros[$macro] = $data['macros_values'][$type][$macro];
+				}
 			}
 		}
 
-		$usermacros = $this->getUserMacros($usermacros)[0]['macros'];
-		foreach ($usermacros as $macro => $value) {
-			$macros[$macro] = $value;
+		// Extract user macros.
+		$data['texts_support_user_macros'] = array_merge($texts, $data['texts_support_user_macros']);
+		if ($data['texts_support_user_macros']) {
+			$matched_macros = self::extractMacros($data['texts_support_user_macros'], ['usermacros' => true]);
+
+			$usermacros = [[
+				'macros' => $matched_macros['usermacros'],
+				'hostids' =>  ($data['hostid'] == 0) ? [] : [$data['hostid']]
+			]];
+
+			$usermacros = $this->getUserMacros($usermacros)[0]['macros'];
+			foreach ($usermacros as $macro => $value) {
+				$macros[$macro] = $value;
+			}
+		}
+
+		// Extract LLD macros.
+		$data['texts_support_lld_macros'] = $data['support_lldmacros']
+			? array_merge($texts, $data['texts_support_lld_macros'])
+			: [];
+		if ($data['texts_support_lld_macros']) {
+			$matched_macros = self::extractMacros($data['texts_support_lld_macros'], ['lldmacros' => true]);
+
+			foreach (array_keys($matched_macros['lldmacros']) as $lldmacro) {
+				$macros[$lldmacro] = $lldmacro;
+			}
 		}
 
 		if (array_key_exists($delay_macro, $macros)) {
@@ -2706,7 +2872,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		$url_macros = [];
 
 		foreach ($urls as $index => $url) {
-			$matched_macros = $this->extractMacros([$url['url'], $url['name']], $types);
+			$matched_macros = self::extractMacros([$url['url'], $url['name']], $types);
 			$url_macros[$index] = [];
 
 			foreach ($matched_macros['macros_an']['event'] as $token => $data) {
