@@ -152,23 +152,28 @@ class CMacrosResolverGeneral {
 		}
 
 		if ($extract_macros) {
-			$macro_parser = new CMacroParser($types['macros']);
+			$macro_parser = new CMacroParser(['macros' => $types['macros']]);
 		}
 
 		if ($extract_macros_n) {
-			$macro_n_parser = new CMacroParser($types['macros_n'], ['ref_type' => CMacroParser::REFERENCE_NUMERIC]);
+			$macro_n_parser = new CMacroParser([
+				'macros' => $types['macros_n'],
+				'ref_type' => CMacroParser::REFERENCE_NUMERIC
+			]);
 		}
 
 		if ($extract_macros_an) {
-			$macro_an_parser = new CMacroParser($types['macros_an'],
-				['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
-			);
+			$macro_an_parser = new CMacroParser([
+				'macros' => $types['macros_an'],
+				'ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC
+			]);
 		}
 
 		if ($extract_macro_funcs_n) {
-			$macro_func_n_parser = new CMacroFunctionParser($types['macro_funcs_n'],
-				['ref_type' => CMacroParser::REFERENCE_NUMERIC]
-			);
+			$macro_func_n_parser = new CMacroFunctionParser([
+				'macros' => $types['macro_funcs_n'],
+				'ref_type' => CMacroParser::REFERENCE_NUMERIC
+			]);
 		}
 
 		if ($extract_references) {
@@ -271,7 +276,7 @@ class CMacrosResolverGeneral {
 			$macros['macros'] = [];
 
 			foreach ($types['macros'] as $key => $macro_patterns) {
-				$types['macros'][$key] = new CMacroParser($macro_patterns);
+				$types['macros'][$key] = new CMacroParser(['macros' => $macro_patterns]);
 				$macros['macros'][$key] = [];
 			}
 		}
@@ -280,9 +285,10 @@ class CMacrosResolverGeneral {
 			$macros['macros_n'] = [];
 
 			foreach ($types['macros_n'] as $key => $macro_patterns) {
-				$types['macros_n'][$key] = new CMacroParser($macro_patterns,
-					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
-				);
+				$types['macros_n'][$key] = new CMacroParser([
+					'macros' => $macro_patterns,
+					'ref_type' => CMacroParser::REFERENCE_NUMERIC
+				]);
 				$macros['macros_n'][$key] = [];
 			}
 		}
@@ -291,9 +297,10 @@ class CMacrosResolverGeneral {
 			$macros['macros_an'] = [];
 
 			foreach ($types['macros_an'] as $key => $macro_patterns) {
-				$types['macros_an'][$key] = new CMacroParser($macro_patterns,
-					['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
-				);
+				$types['macros_an'][$key] = new CMacroParser([
+					'macros' => $macro_patterns,
+					'ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC
+				]);
 				$macros['macros_an'][$key] = [];
 			}
 		}
@@ -302,9 +309,10 @@ class CMacrosResolverGeneral {
 			$macros['macro_funcs_n'] = [];
 
 			foreach ($types['macro_funcs_n'] as $key => $macro_patterns) {
-				$types['macro_funcs_n'][$key] = new CMacroFunctionParser($macro_patterns,
-					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
-				);
+				$types['macro_funcs_n'][$key] = new CMacroFunctionParser([
+					'macros' => $macro_patterns,
+					'ref_type' => CMacroParser::REFERENCE_NUMERIC
+				]);
 				$macros['macro_funcs_n'][$key] = [];
 			}
 		}
@@ -442,7 +450,7 @@ class CMacrosResolverGeneral {
 	/**
 	 * Returns the list of the item key parameters.
 	 *
-	 * @param string $params_raw
+	 * @param array $params_raw
 	 *
 	 * @return array
 	 */
@@ -636,7 +644,7 @@ class CMacrosResolverGeneral {
 		$functionids = [];
 
 		$functionid_parser = new CFunctionIdParser();
-		$macro_parser = new CMacroParser(['{TRIGGER.VALUE}']);
+		$macro_parser = new CMacroParser(['macros' => ['{TRIGGER.VALUE}']]);
 		$user_macro_parser = new CUserMacroParser();
 
 		for ($pos = 0, $i = 1; isset($expression[$pos]); $pos++) {
@@ -782,7 +790,9 @@ class CMacrosResolverGeneral {
 						// break; is not missing here
 
 					case 'ITEM.LASTVALUE':
-						$history = Manager::History()->getLastValues([$function], 1, ZBX_HISTORY_PERIOD);
+						$history = Manager::History()->getLastValues([$function], 1, timeUnitToSeconds(
+							CSettingsHelper::get(CSettingsHelper::HISTORY_PERIOD)
+						));
 
 						if (array_key_exists($function['itemid'], $history)) {
 							$clock = $history[$function['itemid']][0]['clock'];
@@ -874,6 +884,76 @@ class CMacrosResolverGeneral {
 		return $macro_values;
 	}
 
+	protected function getItemLogMacros(array $macros, array $macro_values) {
+		if (!$macros) {
+			return $macro_values;
+		}
+
+		$functions = DBfetchArray(DBselect(
+			'SELECT f.triggerid,f.functionid,i.itemid,i.hostid,i.name,i.key_,i.value_type,i.units,i.valuemapid'.
+			' FROM functions f'.
+				' JOIN items i ON f.itemid=i.itemid'.
+				' JOIN hosts h ON i.hostid=h.hostid'.
+			' WHERE '.dbConditionInt('f.functionid', array_keys($macros)).
+			' AND i.value_type='.ITEM_VALUE_TYPE_LOG
+		));
+
+		if (!$functions) {
+			return $macro_values;
+		}
+
+		$functions = CMacrosResolverHelper::resolveItemNames($functions);
+
+		foreach ($functions as $function) {
+			foreach ($macros[$function['functionid']] as $m => $tokens) {
+				$value = UNRESOLVED_MACRO_STRING;
+
+				$history = Manager::History()->getLastValues([$function], 1,
+					CSettingsHelper::get(CSettingsHelper::HISTORY_PERIOD)
+				);
+				if (!array_key_exists($function['itemid'], $history)) {
+					continue;
+				}
+
+				switch ($m) {
+					case 'ITEM.LOG.DATE':
+						$value = date('Y.m.d', $history[$function['itemid']][0]['timestamp']);
+						break;
+					case 'ITEM.LOG.TIME':
+						$value = date('H:i:s', $history[$function['itemid']][0]['timestamp']);;
+						break;
+					case 'ITEM.LOG.AGE':
+						$value = zbx_date2age($history[$function['itemid']][0]['timestamp']);
+						break;
+					case 'ITEM.LOG.SOURCE':
+						$value = $history[$function['itemid']][0]['source'];
+						break;
+					case 'ITEM.LOG.SEVERITY':
+						$value = getSeverityName($history[$function['itemid']][0]['severity']);
+						break;
+					case 'ITEM.LOG.NSEVERITY':
+						$value = $history[$function['itemid']][0]['severity'];
+						break;
+					case 'ITEM.LOG.EVENTID':
+						$value = $history[$function['itemid']][0]['logeventid'];
+						break;
+				}
+
+				foreach ($tokens as $token) {
+					$macro_value = UNRESOLVED_MACRO_STRING;
+
+					if ($value !== null) {
+						$macro_value = formatHistoryValue($value, $function);
+					}
+
+					$macro_values[$function['triggerid']][$token['token']] = $macro_value;
+				}
+			}
+		}
+
+		return $macro_values;
+	}
+
 	/**
 	 * Get host macros.
 	 *
@@ -978,8 +1058,10 @@ class CMacrosResolverGeneral {
 		 * @var array  $host_macros
 		 * @var array  $host_macros[<hostid>]
 		 * @var array  $host_macros[<hostid>][<macro>]				macro base without curly braces
-		 * @var string $host_macros[<hostid>][<macro>]['value']		base macro value (without context); can be null
-		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context>' => '<value>', ...]
+		 * @var string $host_macros[<hostid>][<macro>]['value']		base macro value (without context and regex);
+		 * 															can be null
+		 * @var array  $host_macros[<hostid>][<macro>]['contexts']	context values; ['<context1>' => '<value1>', ...]
+		 * @var array  $host_macros[<hostid>][<macro>]['regex']		regex values; ['<regex1>' => '<value1>', ...]
 		 */
 		$host_macros = [];
 
@@ -1000,6 +1082,7 @@ class CMacrosResolverGeneral {
 					$hostid = $db_host_macro['hostid'];
 					$macro = $user_macro_parser->getMacro();
 					$context = $user_macro_parser->getContext();
+					$regex = $user_macro_parser->getRegex();
 					$value = self::getMacroValue($db_host_macro);
 
 					if (!array_key_exists($hostid, $host_macros)) {
@@ -1007,12 +1090,15 @@ class CMacrosResolverGeneral {
 					}
 
 					if (!array_key_exists($macro, $host_macros[$hostid])) {
-						$host_macros[$hostid][$macro] = ['value' => null, 'contexts' => []];
+						$host_macros[$hostid][$macro] = ['value' => null, 'contexts' => [], 'regex' => []];
 					}
 
-					if ($context === null) {
+					if ($context === null && $regex === null) {
 						$host_macros[$hostid][$macro]['value'] = $value;
 					}
+					elseif ($regex !== null) {
+						$host_macros[$hostid][$macro]['regex'][$regex] = $value;
+					}
 					else {
 						$host_macros[$hostid][$macro]['contexts'][$context] = $value;
 					}
@@ -1043,6 +1129,9 @@ class CMacrosResolverGeneral {
 			} while ($hostids);
 		}
 
+		// Reordering only regex array.
+		$host_macros = self::sortRegexHostMacros($host_macros);
+
 		$all_macros_resolved = true;
 
 		foreach ($data as &$element) {
@@ -1083,8 +1172,10 @@ class CMacrosResolverGeneral {
 			/*
 			 * @var array  $global_macros
 			 * @var array  $global_macros[<macro>]				macro base without curly braces
-			 * @var string $global_macros[<macro>]['value']		base macro value (without context); can be null
+			 * @var string $global_macros[<macro>]['value']		base macro value (without context and regex);
+			 * 													can be null
 			 * @var array  $global_macros[<macro>]['contexts']	context values; ['<context1>' => '<value1>', ...]
+			 * @var array  $global_macros[<macro>]['regex']		regex values; ['<regex1>' => '<value1>', ...]
 			 */
 			$global_macros = [];
 
@@ -1092,20 +1183,28 @@ class CMacrosResolverGeneral {
 				if ($user_macro_parser->parse($db_global_macro['macro']) == CParser::PARSE_SUCCESS) {
 					$macro = $user_macro_parser->getMacro();
 					$context = $user_macro_parser->getContext();
+					$regex = $user_macro_parser->getRegex();
+					$value = self::getMacroValue($db_global_macro);
 
 					if (!array_key_exists($macro, $global_macros)) {
-						$global_macros[$macro] = ['value' => null, 'contexts' => []];
+						$global_macros[$macro] = ['value' => null, 'contexts' => [], 'regex' => []];
 					}
 
-					if ($context === null) {
-						$global_macros[$macro]['value'] = self::getMacroValue($db_global_macro);
+					if ($context === null && $regex === null) {
+						$global_macros[$macro]['value'] = $value;
+					}
+					elseif ($regex !== null) {
+						$global_macros[$macro]['regex'][$regex] = $value;
 					}
 					else {
-						$global_macros[$macro]['contexts'][$context] = $db_global_macro['value'];
+						$global_macros[$macro]['contexts'][$context] = $value;
 					}
 				}
 			}
 
+			// Reordering only regex array.
+			$global_macros = self::sortRegexGlobalMacros($global_macros);
+
 			foreach ($data as &$element) {
 				foreach ($element['macros'] as $usermacro => &$value) {
 					if ($value['value'] === null && $user_macro_parser->parse($usermacro) == CParser::PARSE_SUCCESS) {
@@ -1116,7 +1215,16 @@ class CMacrosResolverGeneral {
 							if ($context !== null && array_key_exists($context, $global_macros[$macro]['contexts'])) {
 								$value['value'] = $global_macros[$macro]['contexts'][$context];
 							}
-							elseif ($global_macros[$macro]['value'] !== null) {
+							elseif ($context !== null && count($global_macros[$macro]['regex'])) {
+								foreach ($global_macros[$macro]['regex'] as $regex => $val) {
+									if (preg_match('/'.trim($regex, '/').'/', $context) === 1) {
+										$value['value'] = $val;
+										break;
+									}
+								}
+							}
+
+							if ($value['value'] === null && $global_macros[$macro]['value'] !== null) {
 								if ($context === null) {
 									$value['value'] = $global_macros[$macro]['value'];
 								}
@@ -1176,12 +1284,24 @@ class CMacrosResolverGeneral {
 			$value_default = null) {
 		foreach ($hostids as $hostid) {
 			if (array_key_exists($hostid, $host_macros) && array_key_exists($macro, $host_macros[$hostid])) {
+				// Searching context coincidence with macro contexts.
 				if ($context !== null && array_key_exists($context, $host_macros[$hostid][$macro]['contexts'])) {
 					return [
 						'value' => $host_macros[$hostid][$macro]['contexts'][$context],
 						'value_default' => $value_default
 					];
 				}
+				// Searching context coincidence, if regex array not empty.
+				elseif ($context !== null && count($host_macros[$hostid][$macro]['regex'])) {
+					foreach ($host_macros[$hostid][$macro]['regex'] as $regex => $val) {
+						if (preg_match('/'.trim($regex, '/').'/', $context) === 1) {
+							return [
+								'value' => $val,
+								'value_default' => $value_default
+							];
+						}
+					}
+				}
 
 				if ($host_macros[$hostid][$macro]['value'] !== null) {
 					if ($context === null) {
@@ -1223,11 +1343,75 @@ class CMacrosResolverGeneral {
 	/**
 	 * Get macro value refer by type.
 	 *
+	 * @static
+	 *
 	 * @param array $macro
 	 *
 	 * @return string
 	 */
-	static public function getMacroValue(array $macro): string {
-		return ($macro['type'] == ZBX_MACRO_TYPE_SECRET) ? ZBX_MACRO_SECRET_MASK : $macro['value'];
+	public static function getMacroValue(array $macro): string {
+		return ($macro['type'] == ZBX_MACRO_TYPE_SECRET || $macro['type'] == ZBX_MACRO_TYPE_VAULT)
+			? ZBX_SECRET_MASK
+			: $macro['value'];
+	}
+
+	/**
+	 * Sorting host macros.
+	 *
+	 * @param array $host_macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegexHostMacros(array $host_macros): array {
+		foreach ($host_macros as &$macros) {
+			foreach ($macros as &$value) {
+				$value['regex'] = self::sortRegex($value['regex']);
+			}
+			unset($value);
+		}
+		unset($macros);
+
+		return $host_macros;
+	}
+
+	/**
+	 * Sorting global macros.
+	 *
+	 * @static
+	 *
+	 * @param array $global_macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegexGlobalMacros(array $global_macros): array {
+		foreach ($global_macros as &$value) {
+			$value['regex'] = self::sortRegex($value['regex']);
+		}
+		unset($value);
+
+		return $global_macros;
+	}
+
+	/**
+	 * Sort regex.
+	 *
+	 * @static
+	 *
+	 * @param array $macros
+	 *
+	 * @return array
+	 */
+	private static function sortRegex(array $macros): array {
+		$keys = array_keys($macros);
+
+		usort($keys, 'strcmp');
+
+		$new_array = [];
+
+		foreach($keys as $key) {
+			$new_array[$key] = $macros[$key];
+		}
+
+		return $new_array;
 	}
 }
