@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -41,11 +41,17 @@ class CNewValidator {
 	 */
 	private $range_time_parser;
 
+	/**
+	 * A parser for a list of time periods separated by a semicolon.
+	 *
+	 * @var CTimePeriodsParser
+	 */
+	private $time_periods_parser;
+
 	public function __construct(array $input, array $rules) {
 		$this->input = $input;
 		$this->rules = $rules;
 		$this->validationRuleParser = new CValidationRule();
-		$this->range_time_parser = null;
 
 		$this->validate();
 	}
@@ -96,7 +102,7 @@ class CNewValidator {
 
 				case 'json':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || CJs::decodeJson($this->input[$field]) === null) {
+						if (!is_string($this->input[$field]) || json_decode($this->input[$field]) === null) {
 							$this->addError($fatal,
 								_s('Incorrect value for field "%1$s": %2$s.', $field, _('JSON string is expected'))
 							);
@@ -112,8 +118,9 @@ class CNewValidator {
 					if (array_key_exists($field, $this->input)) {
 						if (!is_string($this->input[$field]) || !in_array($this->input[$field], $params)) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -122,10 +129,11 @@ class CNewValidator {
 
 				case 'int32':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || !$this->is_int32($this->input[$field])) {
+						if (!is_string($this->input[$field]) || !self::is_int32($this->input[$field])) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -134,10 +142,11 @@ class CNewValidator {
 
 				case 'id':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || !$this->is_id($this->input[$field])) {
+						if (!is_string($this->input[$field]) || !self::is_id($this->input[$field])) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -151,8 +160,9 @@ class CNewValidator {
 					if (array_key_exists($field, $this->input)) {
 						if (!is_array($this->input[$field]) || !$this->is_array_id($this->input[$field])) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -180,11 +190,13 @@ class CNewValidator {
 				 */
 				case 'array_db':
 					if (array_key_exists($field, $this->input)) {
-						if (!$this->is_array_db($this->input[$field], $params['table'], $params['field'], $flags)
-								|| !is_array($this->input[$field])) {
+						if (!is_array($this->input[$field])
+								|| !$this->is_array_db($this->input[$field], $params['table'], $params['field'], $flags)
+						) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -196,10 +208,10 @@ class CNewValidator {
 				 */
 				case 'ge':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || !$this->is_int32($this->input[$field])
+						if (!is_string($this->input[$field]) || !self::is_int32($this->input[$field])
 								|| $this->input[$field] < $params) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+								_s('Incorrect value for field "%1$s": %2$s.', $field, _s('value must be no less than "%1$s"', $params))
 							);
 
 							return false;
@@ -212,10 +224,10 @@ class CNewValidator {
 				 */
 				case 'le':
 					if (array_key_exists($field, $this->input)) {
-						if (!is_string($this->input[$field]) || !$this->is_int32($this->input[$field])
+						if (!is_string($this->input[$field]) || !self::is_int32($this->input[$field])
 								|| $this->input[$field] > $params) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+								_s('Incorrect value for field "%1$s": %2$s.', $field, _s('value must be no greater than "%1$s"', $params))
 							);
 
 							return false;
@@ -233,8 +245,9 @@ class CNewValidator {
 					if (array_key_exists($field, $this->input)) {
 						if (!$this->is_db($this->input[$field], $params['table'], $params['field'], $flags)) {
 							$this->addError($fatal,
-								_s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
-								// TODO: stringify($this->input[$field]) ???
+								is_scalar($this->input[$field])
+									? _s('Incorrect value "%1$s" for "%2$s" field.', $this->input[$field], $field)
+									: _s('Incorrect value for "%1$s" field.', $field)
 							);
 							return false;
 						}
@@ -263,6 +276,47 @@ class CNewValidator {
 					}
 					break;
 
+				/*
+				 * 'time_periods' => true
+				 */
+				case 'time_periods':
+					if (array_key_exists($field, $this->input) && !$this->isTimePeriods($this->input[$field])) {
+						$this->addError($fatal,
+							_s('Incorrect value for field "%1$s": %2$s.', $field, _('a time period is expected'))
+						);
+						return false;
+					}
+					break;
+
+				/*
+				 * 'time_unit' => true
+				 */
+				case 'time_unit':
+					if (array_key_exists($field, $this->input)) {
+						$result = $this->isTimeUnit($this->input[$field], $params);
+						if (!$result['is_valid']) {
+							$this->addError($fatal,
+								_s('Incorrect value for field "%1$s": %2$s.', $field, $result['error'])
+							);
+							return false;
+						}
+					}
+					break;
+
+				/*
+				 * 'rgb' => true
+				 */
+				case 'rgb':
+					if (array_key_exists($field, $this->input) && !$this->isRgb($this->input[$field])) {
+						$this->addError($fatal,
+							_s('Incorrect value for field "%1$s": %2$s.', $field,
+								_('a hexadecimal colour code (6 symbols) is expected')
+							)
+						);
+						return false;
+					}
+					break;
+
 				/*
 				 * 'string' => true
 				 */
@@ -291,31 +345,28 @@ class CNewValidator {
 		return true;
 	}
 
-	private function is_id($value) {
-		if (1 != preg_match('/^[0-9]+$/', $value)) {
+	public static function is_id($value) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between 0 and _I64_MAX
-		return (bccomp($value, '0') >= 0 && bccomp($value, '9223372036854775807') <= 0);
+		return (bccomp($value, '0') >= 0 && bccomp($value, ZBX_DB_MAX_ID) <= 0);
 	}
 
 	public static function is_int32($value) {
-		if (1 != preg_match('/^\-?[0-9]+$/', $value)) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between INT_MIN and INT_MAX
-		return (bccomp($value, ZBX_MIN_INT32) >= 0 && bccomp($value, ZBX_MAX_INT32) <= 0);
+		return ($value >= ZBX_MIN_INT32 && $value <= ZBX_MAX_INT32);
 	}
 
 	public static function is_uint64($value) {
-		if (1 != preg_match('/^[0-9]+$/', $value)) {
+		if (!preg_match('/^'.ZBX_PREG_INT.'$/', $value)) {
 			return false;
 		}
 
-		// between 0 and _UI64_MAX
-		return (bccomp($value, '0') >= 0 && bccomp($value, '18446744073709551615') <= 0);
+		return ($value >= 0 && bccomp($value, ZBX_MAX_UINT64) <= 0);
 	}
 
 	/**
@@ -332,10 +383,10 @@ class CNewValidator {
 	private function check_db_value($field_schema, &$value, $flags) {
 		switch ($field_schema['type']) {
 			case DB::FIELD_TYPE_ID:
-				return $this->is_id($value);
+				return self::is_id($value);
 
 			case DB::FIELD_TYPE_INT:
-				return $this->is_int32($value);
+				return self::is_int32($value);
 
 			case DB::FIELD_TYPE_CHAR:
 				if ($flags & P_CRLF) {
@@ -344,6 +395,7 @@ class CNewValidator {
 
 				return (mb_strlen($value) <= $field_schema['length']);
 
+			case DB::FIELD_TYPE_NCLOB:
 			case DB::FIELD_TYPE_TEXT:
 				if ($flags & P_CRLF) {
 					$value = CRLFtoLF($value);
@@ -359,7 +411,7 @@ class CNewValidator {
 
 	private function is_array_id(array $values) {
 		foreach ($values as $value) {
-			if (!is_string($value) || !$this->is_id($value)) {
+			if (!is_string($value) || !self::is_id($value)) {
 				return false;
 			}
 		}
@@ -406,20 +458,73 @@ class CNewValidator {
 		return (is_string($value) && $this->check_db_value($table_schema['fields'][$field], $value, $flags));
 	}
 
-	private function isLeapYear($year) {
-		return (0 == $year % 4 && (0 != $year % 100 || 0 == $year % 400));
+	private function isTimePeriods($value) {
+		if ($this->time_periods_parser === null) {
+			$this->time_periods_parser = new CTimePeriodsParser(['usermacros' => true]);
+		}
+
+		return is_string($value) && $this->time_periods_parser->parse($value) == CParser::PARSE_SUCCESS;
 	}
 
-	private function getDaysInMonth($year, $month) {
-		if (in_array($month, [4, 6, 9, 11], true)) {
-			return 30;
+	/**
+	 * Validate a configuration value. Use simple interval parser to parse the string, convert to seconds and check
+	 * if the value is in between given min and max values. In some cases it's possible to enter 0, or even 0s or 0d.
+	 * If the value is incorrect, set an error.
+	 *
+	 * @param string $value                  Value to parse and validate.
+	 * @param bool   $options['with_year']   Set to "true" to allow month and year unit support.
+	 * @param bool   $options['allow_zero']  Set to "true" to allow value to be zero.
+	 * @param string $options['min']         Lower bound.
+	 * @param string $options['max']         Upper bound.
+	 *
+	 * @return array  An array with parameter 'is_valid' containing validation result. If validation fails, additionally
+	 *                returned parameter 'error' containing error message.
+	 */
+	private function isTimeUnit($value, $params) {
+		$simple_interval_parser = new CSimpleIntervalParser(
+			array_key_exists('with_year', $params) ? ['with_year' => true] : []
+		);
+		$value = (string) $value;
+
+		if ($simple_interval_parser->parse($value) == CParser::PARSE_SUCCESS) {
+			if (!$params) {
+				return ['is_valid' => true];
+			}
+
+			if ($value[0] !== '{') {
+				$value = timeUnitToSeconds($value,
+					array_key_exists('with_year', $params) ? $params['with_year'] : false
+				);
+
+				if (array_key_exists('ranges', $params)) {
+					$in_range = false;
+					$message_ranges = [];
+
+					foreach ($params['ranges'] as $range) {
+						if ($range['from'] <= $value && $value <= $range['to']) {
+							$in_range = true;
+							break;
+						}
+
+						$message_ranges[] = ($range['from'] == $range['to'])
+							?  $range['from']
+							:  $range['from'].'-'.$range['to'];
+					}
+
+					if (!$in_range) {
+						return [
+							'is_valid' => false,
+							'error' => _s('value must be one of %1$s', implode(', ', $message_ranges))
+						];
+					}
+				}
+			}
+		}
+		else {
+			return ['is_valid' => false, 'error' => _('a time unit is expected')];
 		}
 
-		if ($month == 2) {
-			return $this->isLeapYear($year) ? 29 : 28;
-		}
-
-		return 31;
+		return ['is_valid' => true];
 	}
 
 	private function isRangeTime($value) {
@@ -430,6 +535,10 @@ class CNewValidator {
 		return is_string($value) && $this->range_time_parser->parse($value) == CParser::PARSE_SUCCESS;
 	}
 
+	private function isRgb($value) {
+		return is_string($value) && preg_match('/^[A-F0-9]{6}$/', $value);
+	}
+
 	/**
 	 * Add validation error.
 	 *
