@@ -21,8 +21,8 @@
 
 class CWidgetFormSvgGraph extends CWidgetForm {
 
-	public function __construct($data) {
-		parent::__construct($data, WIDGET_SVG_GRAPH);
+	public function __construct($data, $templateid) {
+		parent::__construct($data, $templateid, WIDGET_SVG_GRAPH);
 
 		$this->data = self::convertDottedKeys($this->data);
 
@@ -311,13 +311,14 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		// Checkbox: Selected items only.
 		$field_show_problems = (new CWidgetFieldCheckBox('show_problems', _('Show problems')))
 			->setAction(
-				'var on = jQuery(this).is(":checked");'.
+				'var on = jQuery(this).is(":checked"),'.
+					'widget = jQuery(this).closest(".ui-widget");'.
 				'jQuery("#graph_item_problems, #problem_name, #problemhosts_select")'.
 					'.prop("disabled", !on);'.
 				'jQuery("#problemhosts_").multiSelect(on ? "enable" : "disable");'.
-				'jQuery("[name=\"severities[]\"]").prop("disabled", !on);'.
-				'jQuery("[name=\"evaltype\"]").prop("disabled", !on);'.
-				'jQuery("input, button", jQuery("#tags_table_tags")).prop("disabled", !on);'
+				'jQuery("[name^=\"severities[\"]", widget).prop("disabled", !on);'.
+				'jQuery("[name=\"evaltype\"]", widget).prop("disabled", !on);'.
+				'jQuery("input, button", jQuery("#tags_table_tags", widget)).prop("disabled", !on);'
 			);
 
 		if (array_key_exists('show_problems', $this->data)) {
@@ -353,8 +354,7 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		$this->fields[$field_problemhosts->getName()] = $field_problemhosts;
 
 		// Severity checkboxes list.
-		$field_severities = (new CWidgetFieldSeverities('severities', _('Severity')))
-			->setOrientation(CWidgetFieldSeverities::ORIENTATION_HORIZONTAL);
+		$field_severities = new CWidgetFieldSeverities('severities', _('Severity'));
 
 		if ($field_show_problems->getValue() != SVG_GRAPH_PROBLEMS_SHOW) {
 			$field_severities->setFlags(CWidgetField::FLAG_DISABLED);
@@ -378,7 +378,7 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 
 		$this->fields[$field_problem_name->getName()] = $field_problem_name;
 
-		// Problem tag evalype (And/Or).
+		// Problem tag evaltype (And/Or).
 		$field_evaltype = (new CWidgetFieldRadioButtonList('evaltype', _('Tags'), [
 			TAG_EVAL_TYPE_AND_OR => _('And/Or'),
 			TAG_EVAL_TYPE_OR => _('Or')
@@ -432,23 +432,30 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 	private static function validateTimeSelectorPeriod($from, $to) {
 		$errors = [];
 		$ts = [];
+		$ts['now'] = time();
 		$range_time_parser = new CRangeTimeParser();
 
 		foreach (['from' => $from, 'to' => $to] as $field => $value) {
 			$range_time_parser->parse($value);
-			$ts[$field] = $range_time_parser->getDateTime($field === 'from')->getTimestamp();
+			$ts[$field] = $range_time_parser
+				->getDateTime($field === 'from')
+				->getTimestamp();
 		}
 
 		$period = $ts['to'] - $ts['from'] + 1;
+		$range_time_parser->parse('now-'.CSettingsHelper::get(CSettingsHelper::MAX_PERIOD));
+		$max_period = 1 + $ts['now'] - $range_time_parser
+			->getDateTime(true)
+			->getTimestamp();
 
 		if ($period < ZBX_MIN_PERIOD) {
 			$errors[] = _n('Minimum time period to display is %1$s minute.',
 				'Minimum time period to display is %1$s minutes.', (int) (ZBX_MIN_PERIOD / SEC_PER_MIN)
 			);
 		}
-		elseif ($period > ZBX_MAX_PERIOD) {
+		elseif ($period > $max_period) {
 			$errors[] = _n('Maximum time period to display is %1$s day.',
-				'Maximum time period to display is %1$s days.', (int) (ZBX_MAX_PERIOD / SEC_PER_DAY)
+				'Maximum time period to display is %1$s days.', (int) ($max_period / SEC_PER_DAY)
 			);
 		}
 
@@ -473,29 +480,19 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 			));
 		}
 
+		$number_parser = new CNumberParser(['with_suffix' => true]);
+
 		// Validate Min/Max values in Axes tab.
 		if ($this->fields['lefty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
-			$lefty_min = $this->fields['lefty_min']->getValue();
-			$lefty_max = $this->fields['lefty_max']->getValue();
-			$lefty_min = ($lefty_min !== '') ? convertFunctionValue($lefty_min, ZBX_UNITS_ROUNDOFF_LOWER_LIMIT) : '';
-			$lefty_max = ($lefty_max !== '') ? convertFunctionValue($lefty_max, ZBX_UNITS_ROUNDOFF_LOWER_LIMIT) : '';
-			$compare = true;
+			$lefty_min = $number_parser->parse($this->fields['lefty_min']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser->calcValue()
+				: '';
 
-			if (strlen(substr(strrchr($lefty_min, '.'), 1)) > 4) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Left Y').'/'._('Min'),
-					_('too many decimal places')
-				);
-				$compare = false;
-			}
-			if (strlen(substr(strrchr($lefty_max, '.'), 1))  > 4) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Left Y').'/'._('Max'),
-					_('too many decimal places')
-				);
-				$compare = false;
-			}
+			$lefty_max = $number_parser->parse($this->fields['lefty_max']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser->calcValue()
+				: '';
 
-			if ($compare && $lefty_min !== '' && $lefty_max !== ''
-					&& bccomp($lefty_min, $lefty_max, 4) >= 0) {
+			if ($lefty_min !== '' && $lefty_max !== '' && $lefty_min >= $lefty_max) {
 				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Left Y').'/'._('Max'),
 					_('Y axis MAX value must be greater than Y axis MIN value')
 				);
@@ -503,27 +500,15 @@ class CWidgetFormSvgGraph extends CWidgetForm {
 		}
 
 		if ($this->fields['righty']->getValue() == SVG_GRAPH_AXIS_SHOW) {
-			$righty_min = $this->fields['righty_min']->getValue();
-			$righty_max = $this->fields['righty_max']->getValue();
-			$righty_min = ($righty_min != '') ? convertFunctionValue($righty_min, ZBX_UNITS_ROUNDOFF_LOWER_LIMIT) : '';
-			$righty_max = ($righty_max != '') ? convertFunctionValue($righty_max, ZBX_UNITS_ROUNDOFF_LOWER_LIMIT) : '';
-			$compare = true;
+			$righty_min = $number_parser->parse($this->fields['righty_min']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser->calcValue()
+				: '';
 
-			if (strlen(substr(strrchr($righty_min, '.'), 1)) > 4) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Right Y').'/'._('Min'),
-					_('too many decimal places')
-				);
-				$compare = false;
-			}
-			if (strlen(substr(strrchr($righty_max, '.'), 1))  > 4) {
-				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Right Y').'/'._('Max'),
-					_('too many decimal places')
-				);
-				$compare = false;
-			}
+			$righty_max = $number_parser->parse($this->fields['righty_max']->getValue()) == CParser::PARSE_SUCCESS
+				? $number_parser->calcValue()
+				: '';
 
-			if ($compare && $righty_min !== '' && $righty_max !== ''
-					&& bccomp($righty_min, $righty_max, 4) >= 0) {
+			if ($righty_min !== '' && $righty_max !== '' && $righty_min >= $righty_max) {
 				$errors[] = _s('Invalid parameter "%1$s": %2$s.', _('Right Y').'/'._('Max'),
 					_('Y axis MAX value must be greater than Y axis MIN value')
 				);
