@@ -286,10 +286,11 @@ function getGraphParentTemplates(array $graphs, $flag) {
  * @param string $graphid
  * @param array  $parent_templates  The list of the templates, prepared by getGraphParentTemplates() function.
  * @param int    $flag              Origin of the graph (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array|null
  */
-function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
+function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag, bool $provide_links) {
 	if (!array_key_exists($graphid, $parent_templates['links'])) {
 		return null;
 	}
@@ -300,7 +301,7 @@ function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
 
 	$template = $parent_templates['templates'][$parent_templates['links'][$graphid]['hostid']];
 
-	if ($template['permission'] == PERM_READ_WRITE) {
+	if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 		$url = (new CUrl('graphs.php'));
 
 		if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
@@ -328,16 +329,17 @@ function makeGraphTemplatePrefix($graphid, array $parent_templates, $flag) {
  * @param string $graphid
  * @param array  $parent_templates  The list of the templates, prepared by getGraphParentTemplates() function.
  * @param int    $flag              Origin of the item (ZBX_FLAG_DISCOVERY_NORMAL or ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array
  */
-function makeGraphTemplatesHtml($graphid, array $parent_templates, $flag) {
+function makeGraphTemplatesHtml($graphid, array $parent_templates, $flag, bool $provide_links) {
 	$list = [];
 
 	while (array_key_exists($graphid, $parent_templates['links'])) {
 		$template = $parent_templates['templates'][$parent_templates['links'][$graphid]['hostid']];
 
-		if ($template['permission'] == PERM_READ_WRITE) {
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 			$url = (new CUrl('graphs.php'))->setArgument('form', 'update');
 
 			if ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
@@ -849,7 +851,7 @@ function calculateGraphScaleExtremes(float $data_min, float $data_max, bool $is_
 				'max' => $max,
 				'interval' => $interval,
 				'rows' => $rows,
-				'power' => $power,
+				'power' => $power
 			];
 
 			$result_value = ($scale_min - $min) / $interval + ($max - $scale_max) / $interval;
@@ -916,7 +918,7 @@ function calculateGraphScaleValues(float $min, float $max, bool $min_calculated,
 		$rows[] = $value;
 	}
 
-	$ignore_milliseconds = $min <= 1 || $max >= 1;
+	$ignore_milliseconds = ($min <= -1 || $max >= 1);
 
 	$options = [
 		'units' => $units,
@@ -941,7 +943,7 @@ function calculateGraphScaleValues(float $min, float $max, bool $min_calculated,
 	$scale_values[] = [
 		'relative_pos' => 0,
 		'value' => convertUnits([
-			'value' => $min,
+			'value' => $min
 		] + ($min_calculated ? $options_calculated : $options_fixed))
 	];
 
@@ -951,7 +953,7 @@ function calculateGraphScaleValues(float $min, float $max, bool $min_calculated,
 				? ($value / 10 - $min / 10) / ($max / 10 - $min / 10)
 				: ($value - $min) / ($max - $min),
 			'value' => convertUnits([
-				'value' => $value,
+				'value' => $value
 			] + $options_calculated)
 		];
 	}
@@ -959,9 +961,37 @@ function calculateGraphScaleValues(float $min, float $max, bool $min_calculated,
 	$scale_values[] = [
 		'relative_pos' => 1,
 		'value' => convertUnits([
-			'value' => $max,
+			'value' => $max
 		] + ($max_calculated ? $options_calculated : $options_fixed))
 	];
 
 	return $scale_values;
 }
+
+/**
+ * @param string $short_item  Comma separated <short_field_name>:<value> pairs.
+ *
+ * @return array
+ */
+function expandShortGraphItem($short_item) {
+	$map = [
+		'gi' => 'gitemid',
+		'it' => 'itemid',
+		'so' => 'sortorder',
+		'fl' => 'flags',
+		'ty' => 'type',
+		'dr' => 'drawtype',
+		'ya' => 'yaxisside',
+		'ca' => 'calc_fnc',
+		'co' => 'color'
+	];
+
+	$item = [];
+
+	foreach (explode(',', $short_item) as $short_field) {
+		list($short_name, $value) = explode(':', $short_field);
+		$item[$map[$short_name]] = $value;
+	}
+
+	return $item;
+}
