@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -478,13 +478,13 @@ function hostInterfaceTypeNumToName($type) {
 }
 
 function get_hostgroup_by_groupid($groupid) {
-	$groups = DBfetch(DBselect('SELECT g.* FROM groups g WHERE g.groupid='.zbx_dbstr($groupid)));
+	$groups = DBfetch(DBselect('SELECT g.* FROM hstgrp g WHERE g.groupid='.zbx_dbstr($groupid)));
 
 	if ($groups) {
 		return $groups;
 	}
 
-	error(_s('No host groups with groupid "%s".', $groupid));
+	error(_s('No host groups with groupid "%1$s".', $groupid));
 
 	return false;
 }
@@ -527,140 +527,359 @@ function get_host_by_hostid($hostid, $no_error_message = 0) {
 	}
 
 	if ($no_error_message == 0) {
-		error(_s('No host with hostid "%s".', $hostid));
+		error(_s('No host with hostid "%1$s".', $hostid));
 	}
 
 	return false;
 }
 
-function updateHostStatus($hostids, $status) {
-	zbx_value2array($hostids);
+/**
+ * Get parent templates for each given application.
+ *
+ * @param array  $applications                     An array of applications.
+ * @param string $applications[]['applicationid']  ID of an application.
+ * @param array  $applications[]['templateids]     IDs of parent template applications.
+ *
+ * @return array
+ */
+function getApplicationParentTemplates(array $applications) {
+	$parent_applicationids = [];
+	$data = [
+		'links' => [],
+		'templates' => []
+	];
 
-	$hostIds = [];
-	$oldStatus = ($status == HOST_STATUS_MONITORED ? HOST_STATUS_NOT_MONITORED : HOST_STATUS_MONITORED);
-
-	$db_hosts = DBselect(
-		'SELECT h.hostid,h.host,h.status'.
-		' FROM hosts h'.
-		' WHERE '.dbConditionInt('h.hostid', $hostids).
-			' AND h.status='.zbx_dbstr($oldStatus)
-	);
-	while ($host = DBfetch($db_hosts)) {
-		$hostIds[] = $host['hostid'];
-
-		$host_new = $host;
-		$host_new['status'] = $status;
-		add_audit_ext(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_HOST, $host['hostid'], $host['host'], 'hosts', $host, $host_new);
-		info(_s('Updated status of host "%1$s".', $host['host']));
+	foreach ($applications as $application) {
+		foreach ($application['templateids'] as $parent_applicationid) {
+			$parent_applicationids[$parent_applicationid] = true;
+			$data['links'][$application['applicationid']][] = ['applicationid' => $parent_applicationid];
+		}
 	}
 
-	return DB::update('hosts', [
-		'values' => ['status' => $status],
-		'where' => ['hostid' => $hostIds]
-	]);
+	if (!$parent_applicationids) {
+		return $data;
+	}
+
+	$all_parent_applicationids = [];
+	$hostids = [];
+
+	while ($parent_applicationids) {
+		$db_applications = API::Application()->get([
+			'output' => ['applicationid', 'hostid', 'templateids'],
+			'applicationids' => array_keys($parent_applicationids)
+		]);
+
+		$all_parent_applicationids += $parent_applicationids;
+		$parent_applicationids = [];
+
+		foreach ($db_applications as $db_application) {
+			$data['templates'][$db_application['hostid']] = [];
+			$hostids[$db_application['applicationid']] = $db_application['hostid'];
+
+			foreach ($db_application['templateids'] as $parent_applicationid) {
+				if (!array_key_exists($parent_applicationid, $all_parent_applicationids)) {
+					$parent_applicationids[$parent_applicationid] = true;
+				}
+			}
+
+			if (!array_key_exists($db_application['applicationid'], $data['links'])) {
+				foreach ($db_application['templateids'] as $parent_applicationid) {
+					$data['links'][$db_application['applicationid']][] = ['applicationid' => $parent_applicationid];
+				}
+			}
+		}
+	}
+
+	foreach ($data['links'] as $applicationid => &$parent_applications) {
+		foreach ($parent_applications as &$parent_application) {
+			$parent_application['hostid'] = array_key_exists($parent_application['applicationid'], $hostids)
+				? $hostids[$parent_application['applicationid']]
+				: 0;
+		}
+		unset($parent_application);
+	}
+	unset($parent_applications);
+
+	$db_templates = $data['templates']
+		? API::Template()->get([
+			'output' => ['name'],
+			'templateids' => array_keys($data['templates']),
+			'preservekeys' => true
+		])
+		: [];
+
+	$rw_templates = $db_templates
+		? API::Template()->get([
+			'output' => [],
+			'templateids' => array_keys($db_templates),
+			'editable' => true,
+			'preservekeys' => true
+		])
+		: [];
+
+	$data['templates'][0] = [];
+
+	foreach ($data['templates'] as $hostid => &$template) {
+		$template = array_key_exists($hostid, $db_templates)
+			? [
+				'hostid' => $hostid,
+				'name' => $db_templates[$hostid]['name'],
+				'permission' => array_key_exists($hostid, $rw_templates) ? PERM_READ_WRITE : PERM_READ
+			]
+			: [
+				'hostid' => $hostid,
+				'name' => _('Inaccessible template'),
+				'permission' => PERM_DENY
+			];
+	}
+	unset($template);
+
+	return $data;
 }
 
 /**
- * Returns the farthest application ancestor for each given application.
+ * Auxiliary function for makeApplicationTemplatePrefix().
  *
- * @param array $applicationIds
- * @param array $templateApplicationIds		array with parent application IDs as keys and arrays of child application
- * 											IDs as values
+ * @param string $applicationid
+ * @param array  $parent_templates
  *
- * @return array	an array with child IDs as keys and arrays of ancestor IDs as values
+ * @return array
  */
-function getApplicationSourceParentIds(array $applicationIds, array $templateApplicationIds = []) {
-	$query = DBSelect(
-		'SELECT at.applicationid,at.templateid'.
-		' FROM application_template at'.
-		' WHERE '.dbConditionInt('at.applicationid', $applicationIds)
-	);
+function getTopLevelTemplates($applicationid, array $parent_templates) {
+	$templates = [];
 
-	$applicationIds = [];
-	$unsetApplicationIds = [];
-	while ($applicationTemplate = DBfetch($query)) {
-		// check if we already have an application inherited from the current application
-		// if we do - copy all of its child applications to the parent template
-		if (isset($templateApplicationIds[$applicationTemplate['applicationid']])) {
-			$templateApplicationIds[$applicationTemplate['templateid']] = $templateApplicationIds[$applicationTemplate['applicationid']];
-			$unsetApplicationIds[$applicationTemplate['applicationid']] = $applicationTemplate['applicationid'];
+	foreach ($parent_templates['links'][$applicationid] as $parent_application) {
+		if (!array_key_exists($parent_application['applicationid'], $parent_templates['links'])) {
+			$templates[] = $parent_templates['templates'][$parent_application['hostid']];
 		}
-		// if no - just add the application
 		else {
-			$templateApplicationIds[$applicationTemplate['templateid']][] = $applicationTemplate['applicationid'];
+			$templates = array_merge($templates,
+				getTopLevelTemplates($parent_application['applicationid'], $parent_templates)
+			);
 		}
-		$applicationIds[$applicationTemplate['applicationid']] = $applicationTemplate['templateid'];
 	}
 
-	// unset children of all applications that we found a new parent for
-	foreach ($unsetApplicationIds as $applicationId) {
-		unset($templateApplicationIds[$applicationId]);
-	}
-
-	// continue while we still have new applications to check
-	if ($applicationIds) {
-		return getApplicationSourceParentIds($applicationIds, $templateApplicationIds);
-	}
-	else {
-		// return an inverse hash with application IDs as keys and arrays of parent application IDs as values
-		$result = [];
-		foreach ($templateApplicationIds as $templateId => $applicationIds) {
-			foreach ($applicationIds as $applicationId) {
-				$result[$applicationId][] = $templateId;
-			}
-		}
-
-		return $result;
-	}
+	return $templates;
 }
 
 /**
- * Returns the farthest host prototype ancestor for each given host prototype.
+ * Returns a template prefix for selected application.
  *
- * @param array $hostPrototypeIds
- * @param array $templateHostPrototypeIds	array with parent host prototype IDs as keys and arrays of child host
- * 											prototype IDs as values
+ * @param string $applicationid
+ * @param array  $parent_templates  The list of the templates, prepared by getApplicationParentTemplates() function.
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
- * @return array	an array of child ID - ancestor ID pairs
+ * @return array|null
  */
-function getHostPrototypeSourceParentIds(array $hostPrototypeIds, array $templateHostPrototypeIds = []) {
-	$query = DBSelect(
-		'SELECT h.hostid,h.templateid'.
-		' FROM hosts h'.
-		' WHERE '.dbConditionInt('h.hostid', $hostPrototypeIds).
-			' AND h.templateid>0'
-	);
+function makeApplicationTemplatePrefix($applicationid, array $parent_templates, bool $provide_links) {
+	if (!array_key_exists($applicationid, $parent_templates['links'])) {
+		return null;
+	}
 
-	$hostPrototypeIds = [];
-	while ($hostPrototype = DBfetch($query)) {
-		// check if we already have host prototype inherited from the current host prototype
-		// if we do - move all of its child prototypes to the parent template
-		if (isset($templateHostPrototypeIds[$hostPrototype['hostid']])) {
-			$templateHostPrototypeIds[$hostPrototype['templateid']] = $templateHostPrototypeIds[$hostPrototype['hostid']];
-			unset($templateHostPrototypeIds[$hostPrototype['hostid']]);
+	$templates = getTopLevelTemplates($applicationid, $parent_templates);
+	CArrayHelper::sort($templates, ['name']);
+
+	$list = [];
+
+	foreach ($templates as $template) {
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
+			$name = (new CLink(CHtml::encode($template['name']),
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'application.list')
+					->setArgument('filter_set', '1')
+					->setArgument('filter_hostids', [$template['hostid']])
+			))->addClass(ZBX_STYLE_LINK_ALT);
 		}
-		// if no - just add the prototype
 		else {
-			$templateHostPrototypeIds[$hostPrototype['templateid']][] = $hostPrototype['hostid'];
-			$hostPrototypeIds[] = $hostPrototype['templateid'];
+			$name = new CSpan(CHtml::encode($template['name']));
+		}
+
+		$list[] = $name->addClass(ZBX_STYLE_GREY);
+		$list[] = ', ';
+	}
+
+	array_pop($list);
+	$list[] = NAME_DELIMITER;
+
+	return $list;
+}
+
+/**
+ * Get parent templates for each given host prototype.
+ *
+ * @param array  $host_prototypes                  An array of host prototypes.
+ * @param string $host_prototypes[]['hostid']      ID of host prototype.
+ * @param string $host_prototypes[]['templateid']  ID of parent template host prototype.
+ *
+ * @return array
+ */
+function getHostPrototypeParentTemplates(array $host_prototypes) {
+	$parent_host_prototypeids = [];
+	$data = [
+		'links' => [],
+		'templates' => []
+	];
+
+	foreach ($host_prototypes as $host_prototype) {
+		if ($host_prototype['templateid'] != 0) {
+			$parent_host_prototypeids[$host_prototype['templateid']] = true;
+			$data['links'][$host_prototype['hostid']] = ['hostid' => $host_prototype['templateid']];
 		}
 	}
 
-	// continue while we still have new host prototypes to check
-	if ($hostPrototypeIds) {
-		return getHostPrototypeSourceParentIds($hostPrototypeIds, $templateHostPrototypeIds);
+	if (!$parent_host_prototypeids) {
+		return $data;
 	}
-	else {
-		// return an inverse hash with prototype IDs as keys and parent prototype IDs as values
-		$result = [];
-		foreach ($templateHostPrototypeIds as $templateId => $hostIds) {
-			foreach ($hostIds as $hostId) {
-				$result[$hostId] = $templateId;
+
+	$all_parent_host_prototypeids = [];
+	$hostids = [];
+	$lld_ruleids = [];
+
+	do {
+		$db_host_prototypes = API::HostPrototype()->get([
+			'output' => ['hostid', 'templateid'],
+			'selectDiscoveryRule' => ['itemid'],
+			'selectParentHost' => ['hostid'],
+			'hostids' => array_keys($parent_host_prototypeids)
+		]);
+
+		$all_parent_host_prototypeids += $parent_host_prototypeids;
+		$parent_host_prototypeids = [];
+
+		foreach ($db_host_prototypes as $db_host_prototype) {
+			$data['templates'][$db_host_prototype['parentHost']['hostid']] = [];
+			$hostids[$db_host_prototype['hostid']] = $db_host_prototype['parentHost']['hostid'];
+			$lld_ruleids[$db_host_prototype['hostid']] = $db_host_prototype['discoveryRule']['itemid'];
+
+			if ($db_host_prototype['templateid'] != 0) {
+				if (!array_key_exists($db_host_prototype['templateid'], $all_parent_host_prototypeids)) {
+					$parent_host_prototypeids[$db_host_prototype['templateid']] = true;
+				}
+
+				$data['links'][$db_host_prototype['hostid']] = ['hostid' => $db_host_prototype['templateid']];
 			}
 		}
-
-		return $result;
 	}
+	while ($parent_host_prototypeids);
+
+	foreach ($data['links'] as &$parent_host_prototype) {
+		$parent_host_prototype['parent_hostid'] = array_key_exists($parent_host_prototype['hostid'], $hostids)
+			? $hostids[$parent_host_prototype['hostid']]
+			: 0;
+
+		$parent_host_prototype['lld_ruleid'] = array_key_exists($parent_host_prototype['hostid'], $lld_ruleids)
+			? $lld_ruleids[$parent_host_prototype['hostid']]
+			: 0;
+	}
+	unset($parent_host_prototype);
+
+	$db_templates = $data['templates']
+		? API::Template()->get([
+			'output' => ['name'],
+			'templateids' => array_keys($data['templates']),
+			'preservekeys' => true
+		])
+		: [];
+
+	$rw_templates = $db_templates
+		? API::Template()->get([
+			'output' => [],
+			'templateids' => array_keys($db_templates),
+			'editable' => true,
+			'preservekeys' => true
+		])
+		: [];
+
+	$data['templates'][0] = [];
+
+	foreach ($data['templates'] as $hostid => &$template) {
+		$template = array_key_exists($hostid, $db_templates)
+			? [
+				'hostid' => $hostid,
+				'name' => $db_templates[$hostid]['name'],
+				'permission' => array_key_exists($hostid, $rw_templates) ? PERM_READ_WRITE : PERM_READ
+			]
+			: [
+				'hostid' => $hostid,
+				'name' => _('Inaccessible template'),
+				'permission' => PERM_DENY
+			];
+	}
+	unset($template);
+
+	return $data;
+}
+
+/**
+ * Returns a template prefix for selected host prototype.
+ *
+ * @param string $host_prototypeid
+ * @param array  $parent_templates  The list of the templates, prepared by getHostPrototypeParentTemplates() function.
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
+ *
+ * @return array|null
+ */
+function makeHostPrototypeTemplatePrefix($host_prototypeid, array $parent_templates, bool $provide_links) {
+	if (!array_key_exists($host_prototypeid, $parent_templates['links'])) {
+		return null;
+	}
+
+	while (array_key_exists($parent_templates['links'][$host_prototypeid]['hostid'], $parent_templates['links'])) {
+		$host_prototypeid = $parent_templates['links'][$host_prototypeid]['hostid'];
+	}
+
+	$template = $parent_templates['templates'][$parent_templates['links'][$host_prototypeid]['parent_hostid']];
+
+	if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
+		$name = (new CLink(CHtml::encode($template['name']),
+			(new CUrl('host_prototypes.php'))
+				->setArgument('parent_discoveryid', $parent_templates['links'][$host_prototypeid]['lld_ruleid'])
+		))->addClass(ZBX_STYLE_LINK_ALT);
+	}
+	else {
+		$name = new CSpan(CHtml::encode($template['name']));
+	}
+
+	return [$name->addClass(ZBX_STYLE_GREY), NAME_DELIMITER];
+}
+
+/**
+ * Returns a list of host prototype templates.
+ *
+ * @param string $host_prototypeid
+ * @param array  $parent_templates  The list of the templates, prepared by getHostPrototypeParentTemplates() function.
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
+ *
+ * @return array
+ */
+function makeHostPrototypeTemplatesHtml($host_prototypeid, array $parent_templates, bool $provide_links) {
+	$list = [];
+
+	while (array_key_exists($host_prototypeid, $parent_templates['links'])) {
+		$template = $parent_templates['templates'][$parent_templates['links'][$host_prototypeid]['parent_hostid']];
+
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
+			$name = new CLink(CHtml::encode($template['name']),
+				(new CUrl('host_prototypes.php'))
+					->setArgument('form', 'update')
+					->setArgument('parent_discoveryid', $parent_templates['links'][$host_prototypeid]['lld_ruleid'])
+					->setArgument('hostid', $parent_templates['links'][$host_prototypeid]['hostid'])
+			);
+		}
+		else {
+			$name = (new CSpan(CHtml::encode($template['name'])))->addClass(ZBX_STYLE_GREY);
+		}
+
+		array_unshift($list, $name, '&nbsp;&rArr;&nbsp;');
+
+		$host_prototypeid = $parent_templates['links'][$host_prototypeid]['hostid'];
+	}
+
+	if ($list) {
+		array_pop($list);
+	}
+
+	return $list;
 }
 
 /**
@@ -711,7 +930,7 @@ function getDeletableHostGroupIds(array $groupIds) {
 
 	$dbResult = DBselect(
 		'SELECT g.groupid'.
-		' FROM groups g'.
+		' FROM hstgrp g'.
 		' WHERE g.internal='.ZBX_NOT_INTERNAL_GROUP.
 			' AND '.dbConditionInt('g.groupid', $groupIds).
 			' AND NOT EXISTS ('.
@@ -743,10 +962,15 @@ function isTemplate($hostId) {
  *   array(
  *       '{$MACRO}' => array(
  *           'macro' => '{$MACRO}',
+ *           'parent_host' => array(                <- optional
+ *               'value' => 'parent host level value',
+ *               'type' => 0,
+ *               'description' => ''
+ *           ),
  *           'template' => array(                   <- optional
  *               'value' => 'template-level value'
  *               'templateid' => 10001,
- *               'name' => 'Template OS Linux'
+ *               'name' => 'Template OS Linux by Zabbix agent'
  *           ),
  *           'global' => array(                     <- optional
  *               'value' => 'global-level value'
@@ -754,24 +978,29 @@ function isTemplate($hostId) {
  *       )
  *   )
  *
- * @param array $hostids
+ * @param array     $hostids        Host or template ids.
+ * @param int|null  $parent_hostid  Parent host id of host prototype.
  *
  * @return array
  */
-function getInheritedMacros(array $hostids) {
+function getInheritedMacros(array $hostids, ?int $parent_hostid = null): array {
 	$user_macro_parser = new CUserMacroParser();
 
 	$all_macros = [];
 	$global_macros = [];
 
 	$db_global_macros = API::UserMacro()->get([
-		'output' => ['macro', 'value'],
+		'output' => ['macro', 'value', 'description', 'type'],
 		'globalmacro' => true
 	]);
 
 	foreach ($db_global_macros as $db_global_macro) {
 		$all_macros[$db_global_macro['macro']] = true;
-		$global_macros[$db_global_macro['macro']] = $db_global_macro['value'];
+		$global_macros[$db_global_macro['macro']] = [
+			'value' => getMacroConfigValue($db_global_macro),
+			'description' => $db_global_macro['description'],
+			'type' => $db_global_macro['type']
+		];
 	}
 
 	// hostid => array('name' => name, 'macros' => array(macro => value), 'templateids' => array(templateid))
@@ -783,7 +1012,7 @@ function getInheritedMacros(array $hostids) {
 		$db_templates = API::Template()->get([
 			'output' => ['name'],
 			'selectParentTemplates' => ['templateid'],
-			'selectMacros' => ['macro', 'value'],
+			'selectMacros' => ['macro', 'value', 'description', 'type'],
 			'templateids' => $templateids,
 			'preservekeys' => true
 		]);
@@ -805,7 +1034,11 @@ function getInheritedMacros(array $hostids) {
 			 */
 			foreach ($db_template['macros'] as $dbMacro) {
 				if (array_key_exists($dbMacro['macro'], $all_macros)) {
-					$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+					$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+						'value' => getMacroConfigValue($dbMacro),
+						'description' => $dbMacro['description'],
+						'type' => $dbMacro['type']
+					];
 					$all_macros[$dbMacro['macro']] = true;
 				}
 				else {
@@ -814,7 +1047,11 @@ function getInheritedMacros(array $hostids) {
 					$tpl_context = $user_macro_parser->getContext();
 
 					if ($tpl_context === null) {
-						$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+						$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+							'value' => getMacroConfigValue($dbMacro),
+							'description' => $dbMacro['description'],
+							'type' => $dbMacro['type']
+						];
 						$all_macros[$dbMacro['macro']] = true;
 					}
 					else {
@@ -832,7 +1069,11 @@ function getInheritedMacros(array $hostids) {
 									$all_macros[$global_macro]
 								);
 
-								$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+								$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+									'value' => getMacroConfigValue($dbMacro),
+									'description' => $dbMacro['description'],
+									'type' => $dbMacro['type']
+								];
 								$all_macros[$dbMacro['macro']] = true;
 								$global_macros[$dbMacro['macro']] = $global_value;
 
@@ -841,7 +1082,11 @@ function getInheritedMacros(array $hostids) {
 						}
 
 						if (!$match_found) {
-							$hosts[$hostid]['macros'][$dbMacro['macro']] = $dbMacro['value'];
+							$hosts[$hostid]['macros'][$dbMacro['macro']] = [
+								'value' => getMacroConfigValue($dbMacro),
+								'description' => $dbMacro['description'],
+								'type' => $dbMacro['type']
+							];
 							$all_macros[$dbMacro['macro']] = true;
 						}
 					}
@@ -859,17 +1104,38 @@ function getInheritedMacros(array $hostids) {
 		}
 	} while ($templateids);
 
-	$all_macros = array_keys($all_macros);
 	$all_templates = [];
 	$inherited_macros = [];
+	$parent_host_macros = [];
+
+	if ($parent_hostid !== null) {
+		$parent_host_macros = API::UserMacro()->get([
+			'output' => ['macro', 'type', 'value', 'description'],
+			'hostids' => [$parent_hostid]
+		]);
+
+		$parent_host_macros = array_column($parent_host_macros, null, 'macro');
+		$all_macros += array_fill_keys(array_keys($parent_host_macros), true);
+	}
+
+	$all_macros = array_keys($all_macros);
 
 	// resolving
 	foreach ($all_macros as $macro) {
 		$inherited_macro = ['macro' => $macro];
 
-		if (array_key_exists($macro, $global_macros)) {
+		if (array_key_exists($macro, $parent_host_macros)) {
+			$inherited_macro['parent_host'] = [
+				'value' => getMacroConfigValue($parent_host_macros[$macro]),
+				'description' => $parent_host_macros[$macro]['description'],
+				'type' => $parent_host_macros[$macro]['type']
+			];
+		}
+		elseif (array_key_exists($macro, $global_macros)) {
 			$inherited_macro['global'] = [
-				'value' => $global_macros[$macro]
+				'value' => $global_macros[$macro]['value'],
+				'description' => $global_macros[$macro]['description'],
+				'type' => $global_macros[$macro]['type']
 			];
 		}
 
@@ -881,10 +1147,12 @@ function getInheritedMacros(array $hostids) {
 			foreach ($templateids as $templateid) {
 				if (array_key_exists($templateid, $hosts) && array_key_exists($macro, $hosts[$templateid]['macros'])) {
 					$inherited_macro['template'] = [
-						'value' => $hosts[$templateid]['macros'][$macro],
+						'value' => $hosts[$templateid]['macros'][$macro]['value'],
+						'description' => $hosts[$templateid]['macros'][$macro]['description'],
 						'templateid' => $hosts[$templateid]['templateid'],
 						'name' => $hosts[$templateid]['name'],
-						'rights' => PERM_READ
+						'rights' => PERM_READ,
+						'type' => $hosts[$templateid]['macros'][$macro]['type']
 					];
 
 					if (!array_key_exists($hosts[$templateid]['templateid'], $all_templates)) {
@@ -938,13 +1206,19 @@ function getInheritedMacros(array $hostids) {
  *   array(
  *       '{$MACRO}' => array(
  *           'macro' => '{$MACRO}',
- *           'type' => 0x03,						<- MACRO_TYPE_INHERITED, MACRO_TYPE_HOSTMACRO or MACRO_TYPE_BOTH
+ *           'type' => 0,                           <- ZBX_MACRO_TYPE_TEXT or ZBX_MACRO_TYPE_SECRET
+ *           'inherited_type' => 0x03,              <- ZBX_PROPERTY_INHERITED, ZBX_PROPERTY_OWN or ZBX_PROPERTY_BOTH
  *           'value' => 'effective value',
  *           'hostmacroid' => 7532,                 <- optional
+ *           'parent_host' => array(                <- optional
+ *               'value' => 'parent host value',
+ *               'type' => 0,
+ *               'description' => ''
+ *           ),
  *           'template' => array(                   <- optional
  *               'value' => 'template-level value'
  *               'templateid' => 10001,
- *               'name' => 'Template OS Linux'
+ *               'name' => 'Template OS Linux by Zabbix agent'
  *           ),
  *           'global' => array(                     <- optional
  *               'value' => 'global-level value'
@@ -952,19 +1226,27 @@ function getInheritedMacros(array $hostids) {
  *       )
  *   )
  *
- * @param array $host_macros		the list of host macros
- * @param array $inherited_macros	the list of inherited macros (the output of the getInheritedMacros() function)
+ * @param array $host_macros       The list of host macros.
+ * @param array $inherited_macros  The list of inherited macros (the output of the getInheritedMacros() function).
  *
  * @return array
  */
-function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
+function mergeInheritedMacros(array $host_macros, array $inherited_macros): array {
 	$user_macro_parser = new CUserMacroParser();
+	$inherit_order = ['parent_host', 'template', 'global'];
 
 	foreach ($inherited_macros as &$inherited_macro) {
-		$inherited_macro['type'] = MACRO_TYPE_INHERITED;
-		$inherited_macro['value'] = array_key_exists('template', $inherited_macro)
-			? $inherited_macro['template']['value']
-			: $inherited_macro['global']['value'];
+		[$inherited_level] = array_values(array_intersect($inherit_order, array_keys($inherited_macro)));
+		$inherited_macro['inherited_type'] = ZBX_PROPERTY_INHERITED;
+		$inherited_macro['inherited_level'] = $inherited_level;
+		$inherited_macro['value'] = $inherited_macro[$inherited_level]['value'];
+		$inherited_macro['type'] = $inherited_macro[$inherited_level]['type'];
+		$inherited_macro['description'] = $inherited_macro[$inherited_level]['description'];
+
+		// Secret macro value cannot be inherited.
+		if ($inherited_macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+			unset($inherited_macro['value']);
+		}
 	}
 	unset($inherited_macro);
 
@@ -973,6 +1255,11 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 	 * checking for contexts, since {$MACRO:} is the same as {$MACRO:""}.
 	 */
 	foreach ($host_macros as &$host_macro) {
+		// Secret macro value cannot be inherited.
+		if ($host_macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+			unset($inherited_macros[$host_macro['macro']]['value']);
+		}
+
 		if (array_key_exists($host_macro['macro'], $inherited_macros)) {
 			$host_macro = array_merge($inherited_macros[$host_macro['macro']], $host_macro);
 			unset($inherited_macros[$host_macro['macro']]);
@@ -987,7 +1274,7 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 				$hst_context = $user_macro_parser->getContext();
 
 				if ($hst_context === null) {
-					$host_macro['type'] = 0x00;
+					$host_macro['inherited_type'] = 0x00;
 				}
 				else {
 					$match_found = false;
@@ -1009,16 +1296,16 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
 					}
 
 					if (!$match_found) {
-						$host_macro['type'] = 0x00;
+						$host_macro['inherited_type'] = 0x00;
 					}
 				}
 			}
 			else {
-				$host_macro['type'] = 0x00;
+				$host_macro['inherited_type'] = 0x00;
 			}
 		}
 
-		$host_macro['type'] |= MACRO_TYPE_HOSTMACRO;
+		$host_macro['inherited_type'] |= ZBX_PROPERTY_OWN;
 	}
 	unset($host_macro);
 
@@ -1038,11 +1325,11 @@ function mergeInheritedMacros(array $host_macros, array $inherited_macros) {
  */
 function cleanInheritedMacros(array $macros) {
 	foreach ($macros as $idx => $macro) {
-		if (array_key_exists('type', $macro) && !($macro['type'] & MACRO_TYPE_HOSTMACRO)) {
+		if (array_key_exists('inherited_type', $macro) && !($macro['inherited_type'] & ZBX_PROPERTY_OWN)) {
 			unset($macros[$idx]);
 		}
 		else {
-			unset($macros[$idx]['type'], $macros[$idx]['inherited']);
+			unset($macros[$idx]['inherited_type'], $macros[$idx]['inherited']);
 		}
 	}
 
@@ -1091,32 +1378,7 @@ function isReadableTemplates(array $templateids) {
 }
 
 /**
- * Check if user has read permissions for hosts or templates.
- *
- * @param array $hostids
- *
- * @return bool
- */
-function isReadableHostTemplates(array $hostids) {
-	$count = API::Host()->get([
-		'countOutput' => true,
-		'hostids' => $hostids
-	]);
-
-	if ($count == count($hostids)) {
-		return true;
-	}
-
-	$count += API::Template()->get([
-		'countOutput' => true,
-		'templateids' => $hostids
-	]);
-
-	return ($count == count($hostids));
-}
-
-/**
- * Check if user has read permissions for hosts or templates.
+ * Check if user has write permissions for hosts or templates.
  *
  * @param array $hostids
  *
@@ -1141,3 +1403,82 @@ function isWritableHostTemplates(array $hostids) {
 
 	return ($count == count($hostids));
 }
+
+function getAddNewInterfaceSubmenu() {
+	return [
+		'main_section' => [
+			'items' => [
+				"javascript:hostInterfaceManager.addAgent();" => _('Agent'),
+				"javascript:hostInterfaceManager.addSnmp();" => _('SNMP'),
+				"javascript:hostInterfaceManager.addJmx();" => _('JMX'),
+				"javascript:hostInterfaceManager.addIpmi();" => _('IPMI')
+			]
+		]
+	];
+}
+
+function renderInterfaceHeaders() {
+	return (new CDiv())
+		->addClass(implode(' ', [ZBX_STYLE_HOST_INTERFACE_CONTAINER, ZBX_STYLE_HOST_INTERFACE_CONTAINER_HEADER]))
+		->addItem(
+			(new CDiv())
+				->addClass(implode(' ', [ZBX_STYLE_HOST_INTERFACE_ROW, ZBX_STYLE_HOST_INTERFACE_ROW_HEADER]))
+				->addItem([
+					(new CDiv())->addClass(ZBX_STYLE_HOST_INTERFACE_CELL),
+					(new CDiv(_('Type')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_TYPE
+						])
+					),
+					(new CDiv(_('IP address')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_IP
+						])
+					),
+					(new CDiv(_('DNS name')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_DNS
+						])
+					),
+					(new CDiv(_('Connect to')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_USEIP
+						])
+					),
+					(new CDiv(_('Port')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_PORT
+						])
+					),
+					(new CDiv(_('Default')))->addClass(
+						implode(' ', [ZBX_STYLE_HOST_INTERFACE_CELL, ZBX_STYLE_HOST_INTERFACE_CELL_HEADER,
+							ZBX_STYLE_HOST_INTERFACE_CELL_ACTION
+						])
+					)
+				])
+		);
+}
+
+function getHostDashboards(string $hostid, array $dashboard_fields = []): array {
+	$dashboard_fields = array_merge($dashboard_fields, ['dashboardid']);
+	$dashboard_fields = array_keys(array_flip($dashboard_fields));
+
+	$templateids = CApiHostHelper::getParentTemplates([$hostid])[1];
+
+	return API::TemplateDashboard()->get([
+		'output' => $dashboard_fields,
+		'templateids' => $templateids,
+		'preservekeys' => true
+	]);
+}
+
+/**
+ * Return macro value to display in the list of inherited macros.
+ *
+ * @param array $macro
+ *
+ * @return string
+ */
+function getMacroConfigValue(array $macro): string {
+	return ($macro['type'] == ZBX_MACRO_TYPE_SECRET) ? ZBX_SECRET_MASK : $macro['value'];
+}
