@@ -86,9 +86,7 @@ function item_type2str($type = null) {
 		ITEM_TYPE_ZABBIX => _('Zabbix agent'),
 		ITEM_TYPE_ZABBIX_ACTIVE => _('Zabbix agent (active)'),
 		ITEM_TYPE_SIMPLE => _('Simple check'),
-		ITEM_TYPE_SNMPV1 => _('SNMPv1 agent'),
-		ITEM_TYPE_SNMPV2C => _('SNMPv2 agent'),
-		ITEM_TYPE_SNMPV3 => _('SNMPv3 agent'),
+		ITEM_TYPE_SNMP => _('SNMP agent'),
 		ITEM_TYPE_SNMPTRAP => _('SNMP trap'),
 		ITEM_TYPE_INTERNAL => _('Zabbix internal'),
 		ITEM_TYPE_TRAPPER => _('Zabbix trapper'),
@@ -102,7 +100,8 @@ function item_type2str($type = null) {
 		ITEM_TYPE_JMX => _('JMX agent'),
 		ITEM_TYPE_CALCULATED => _('Calculated'),
 		ITEM_TYPE_HTTPTEST => _('Web monitoring'),
-		ITEM_TYPE_DEPENDENT => _('Dependent item')
+		ITEM_TYPE_DEPENDENT => _('Dependent item'),
+		ITEM_TYPE_SCRIPT => _('Script')
 	];
 
 	if ($type === null) {
@@ -268,6 +267,7 @@ function orderItemsByTrends(array &$items, $sortorder){
  * @param array  $items
  * @param int    $items['type']
  * @param string $items['delay']
+ * @param string $items['key_']
  * @param string $sortorder
  * @param array  $options
  * @param bool   $options['usermacros']
@@ -277,7 +277,8 @@ function orderItemsByDelay(array &$items, $sortorder, array $options){
 	$update_interval_parser = new CUpdateIntervalParser($options);
 
 	foreach ($items as &$item) {
-		if (in_array($item['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])) {
+		if (in_array($item['type'], [ITEM_TYPE_TRAPPER, ITEM_TYPE_SNMPTRAP, ITEM_TYPE_DEPENDENT])
+				|| ($item['type'] == ITEM_TYPE_ZABBIX_ACTIVE && strncmp($item['key_'], 'mqtt.get', 8) === 0)) {
 			$item['delay_sort'] = '';
 		}
 		elseif ($update_interval_parser->parse($item['delay']) == CParser::PARSE_SUCCESS) {
@@ -347,7 +348,7 @@ function interfaceType2str($type) {
 		INTERFACE_TYPE_AGENT => _('Agent'),
 		INTERFACE_TYPE_SNMP => _('SNMP'),
 		INTERFACE_TYPE_JMX => _('JMX'),
-		INTERFACE_TYPE_IPMI => _('IPMI'),
+		INTERFACE_TYPE_IPMI => _('IPMI')
 	];
 
 	return isset($interfaceGroupLabels[$type]) ? $interfaceGroupLabels[$type] : null;
@@ -355,9 +356,7 @@ function interfaceType2str($type) {
 
 function itemTypeInterface($type = null) {
 	$types = [
-		ITEM_TYPE_SNMPV1 => INTERFACE_TYPE_SNMP,
-		ITEM_TYPE_SNMPV2C => INTERFACE_TYPE_SNMP,
-		ITEM_TYPE_SNMPV3 => INTERFACE_TYPE_SNMP,
+		ITEM_TYPE_SNMP =>  INTERFACE_TYPE_SNMP,
 		ITEM_TYPE_SNMPTRAP => INTERFACE_TYPE_SNMP,
 		ITEM_TYPE_IPMI => INTERFACE_TYPE_IPMI,
 		ITEM_TYPE_ZABBIX => INTERFACE_TYPE_AGENT,
@@ -389,14 +388,12 @@ function itemTypeInterface($type = null) {
  */
 function copyItemsToHosts($src_itemids, $dst_hostids) {
 	$items = API::Item()->get([
-		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
-			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
-			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
-			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
-			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid', 'timeout',
-			'url', 'query_fields', 'posts', 'status_codes', 'follow_redirects', 'post_type', 'http_proxy', 'headers',
-			'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password',
-			'verify_peer', 'verify_host', 'allow_traps'
+		'output' => ['type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
+			'trapper_hosts', 'units', 'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+			'password', 'publickey', 'privatekey', 'flags', 'description', 'inventory_link', 'jmx_endpoint',
+			'master_itemid', 'timeout', 'url', 'query_fields', 'posts', 'status_codes', 'follow_redirects',
+			'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file',
+			'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps', 'parameters'
 		],
 		'selectApplications' => ['applicationid'],
 		'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
@@ -560,14 +557,13 @@ function copyItemsToHosts($src_itemids, $dst_hostids) {
 
 function copyItems($srcHostId, $dstHostId) {
 	$srcItems = API::Item()->get([
-		'output' => ['type', 'snmp_community', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status',
-			'value_type', 'trapper_hosts', 'units', 'snmpv3_contextname', 'snmpv3_securityname', 'snmpv3_securitylevel',
-			'snmpv3_authprotocol', 'snmpv3_authpassphrase', 'snmpv3_privprotocol', 'snmpv3_privpassphrase',
-			'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username', 'password', 'publickey',
-			'privatekey', 'flags', 'port', 'description', 'inventory_link', 'jmx_endpoint', 'master_itemid',
-			'templateid', 'url', 'query_fields', 'timeout', 'posts', 'status_codes', 'follow_redirects', 'post_type',
-			'http_proxy', 'headers', 'retrieve_mode', 'request_method', 'output_format', 'ssl_cert_file',
-			'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host', 'allow_traps'
+		'output' => ['type', 'snmp_oid', 'name', 'key_', 'delay', 'history', 'trends', 'status', 'value_type',
+			'trapper_hosts', 'units', 'logtimefmt', 'valuemapid', 'params', 'ipmi_sensor', 'authtype', 'username',
+			'password', 'publickey', 'privatekey', 'flags', 'description', 'inventory_link', 'jmx_endpoint',
+			'master_itemid', 'templateid', 'url', 'query_fields', 'timeout', 'posts', 'status_codes',
+			'follow_redirects', 'post_type', 'http_proxy', 'headers', 'retrieve_mode', 'request_method',
+			'output_format', 'ssl_cert_file', 'ssl_key_file', 'ssl_key_password', 'verify_peer', 'verify_host',
+			'allow_traps', 'parameters'
 		],
 		'selectApplications' => ['applicationid'],
 		'selectPreprocessing' => ['type', 'params', 'error_handler', 'error_handler_params'],
@@ -909,10 +905,11 @@ function getItemParentTemplates(array $items, $flag) {
  * @param array  $parent_templates  The list of the templates, prepared by getItemParentTemplates() function.
  * @param int    $flag              Origin of the item (ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_RULE,
  *                                  ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array|null
  */
-function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
+function makeItemTemplatePrefix($itemid, array $parent_templates, $flag, bool $provide_links) {
 	if (!array_key_exists($itemid, $parent_templates['links'])) {
 		return null;
 	}
@@ -923,9 +920,11 @@ function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
 
 	$template = $parent_templates['templates'][$parent_templates['links'][$itemid]['hostid']];
 
-	if ($template['permission'] == PERM_READ_WRITE) {
+	if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 		if ($flag == ZBX_FLAG_DISCOVERY_RULE) {
-			$url = (new CUrl('host_discovery.php'))->setArgument('hostid', $template['hostid']);
+			$url = (new CUrl('host_discovery.php'))
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$template['hostid']]);
 		}
 		elseif ($flag == ZBX_FLAG_DISCOVERY_PROTOTYPE) {
 			$url = (new CUrl('disc_prototypes.php'))
@@ -934,8 +933,8 @@ function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
 		// ZBX_FLAG_DISCOVERY_NORMAL
 		else {
 			$url = (new CUrl('items.php'))
-				->setArgument('hostid', $template['hostid'])
-				->setArgument('filter_set', 1);
+				->setArgument('filter_set', '1')
+				->setArgument('filter_hostids', [$template['hostid']]);
 		}
 
 		$name = (new CLink(CHtml::encode($template['name']), $url))->addClass(ZBX_STYLE_LINK_ALT);
@@ -954,16 +953,17 @@ function makeItemTemplatePrefix($itemid, array $parent_templates, $flag) {
  * @param array  $parent_templates  The list of the templates, prepared by getItemParentTemplates() function.
  * @param int    $flag              Origin of the item (ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_RULE,
  *                                  ZBX_FLAG_DISCOVERY_PROTOTYPE).
+ * @param bool   $provide_links     If this parameter is false, prefix will not contain links.
  *
  * @return array
  */
-function makeItemTemplatesHtml($itemid, array $parent_templates, $flag) {
+function makeItemTemplatesHtml($itemid, array $parent_templates, $flag, bool $provide_links) {
 	$list = [];
 
 	while (array_key_exists($itemid, $parent_templates['links'])) {
 		$template = $parent_templates['templates'][$parent_templates['links'][$itemid]['hostid']];
 
-		if ($template['permission'] == PERM_READ_WRITE) {
+		if ($provide_links && $template['permission'] == PERM_READ_WRITE) {
 			if ($flag == ZBX_FLAG_DISCOVERY_RULE) {
 				$url = (new CUrl('host_discovery.php'))
 					->setArgument('form', 'update')
@@ -1001,72 +1001,132 @@ function makeItemTemplatesHtml($itemid, array $parent_templates, $flag) {
 }
 
 /**
- * Retrieve overview table object for items.
+ * Collect latest value and actual severity value for each item of Data overview table.
  *
- * @param array  $groupids
- * @param string $application      IDs of applications to filter items by.
- * @param int    $viewMode
- * @param int    $show_suppressed  Whether to show suppressed problems.
+ * @param array $db_items
+ * @param array $data
+ * @param int   $show_suppressed
  *
- * @return CTableInfo
+ * @return array
  */
+function getDataOverviewCellData(array $db_items, array $data, int $show_suppressed): array {
+	$history = Manager::History()->getLastValues($db_items, 1,
+		timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::HISTORY_PERIOD))
+	);
 
-function getItemsDataOverview(array $groupids, $application, $viewMode,
-		$show_suppressed = ZBX_PROBLEM_SUPPRESSED_TRUE) {
-	// application filter
+	$db_triggers = getTriggersWithActualSeverity([
+		'output' => ['triggerid', 'priority', 'value'],
+		'selectItems' => ['itemid'],
+		'itemids' => array_keys($db_items),
+		'monitored' => true,
+		'preservekeys' => true
+	], ['show_suppressed' => $show_suppressed]);
+
+	$itemid_to_triggerids = [];
+	foreach ($db_triggers as $triggerid => $db_trigger) {
+		foreach ($db_trigger['items'] as $item) {
+			if (!array_key_exists($item['itemid'], $itemid_to_triggerids)) {
+				$itemid_to_triggerids[$item['itemid']] = [];
+			}
+			$itemid_to_triggerids[$item['itemid']][] = $triggerid;
+		}
+	}
+
+	// Apply values and trigger severity to each $data cell.
+	foreach ($data as &$data_clusters) {
+		foreach ($data_clusters as &$data_cluster) {
+			foreach ($data_cluster as &$item) {
+				$itemid = $item['itemid'];
+
+				if (array_key_exists($itemid, $itemid_to_triggerids)) {
+					$max_priority = -1;
+					$max_priority_triggerid = -1;
+					foreach ($itemid_to_triggerids[$itemid] as $triggerid) {
+						$trigger = $db_triggers[$triggerid];
+
+						// Bump lower priority triggers of value "true" ahead of triggers with value "false".
+						$multiplier = ($trigger['value'] == TRIGGER_VALUE_TRUE) ? TRIGGER_SEVERITY_COUNT : 0;
+						if ($trigger['priority'] + $multiplier > $max_priority) {
+							$max_priority_triggerid = $triggerid;
+							$max_priority = $trigger['priority'] + $multiplier;
+						}
+					}
+					$trigger = $db_triggers[$max_priority_triggerid];
+				}
+				else {
+					$trigger = null;
+				}
+
+				$item += [
+					'value' => array_key_exists($itemid, $history) ? $history[$itemid][0]['value'] : null,
+					'trigger' => $trigger
+				];
+			}
+		}
+	}
+	unset($data_clusters, $data_cluster, $item);
+
+	return $data;
+}
+
+/**
+ * @param array  $groupids
+ * @param array  $hostids
+ * @param string $application
+ *
+ * @return array
+ */
+function getDataOverviewItems(?array $groupids = null, ?array $hostids = null, ?string $application = ''): array {
 	if ($application !== '') {
-		$applicationids = array_keys(API::Application()->get([
-			'output' => [],
-			'groupids' => $groupids ? $groupids : null,
+		$db_applications = API::Application()->get([
+			'output' => ['hostid'],
+			'hostids' => $hostids,
+			'groupids' => $groupids,
 			'search' => ['name' => $application],
 			'preservekeys' => true
-		]));
-		$groupids = [];
+		]);
+
+		$applicationids = array_keys($db_applications);
+		$hostids = array_keys(array_flip(array_column($db_applications, 'hostid')));
 	}
 	else {
 		$applicationids = null;
 	}
 
-	$db_items = API::Item()->get([
-		'output' => ['itemid', 'hostid', 'key_', 'name', 'value_type', 'units', 'valuemapid'],
-		'selectHosts' => ['name'],
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
-		'monitored' => true,
-		'webitems' => true,
-		'preservekeys' => true
-	]);
+	if ($hostids === null) {
+		$limit = (int) CSettingsHelper::get(CSettingsHelper::MAX_OVERVIEW_TABLE_SIZE) + 1;
+		$db_hosts = API::Host()->get([
+			'output' => [],
+			'groupids' => $groupids,
+			'applicationids' => $applicationids,
+			'monitored_hosts' => true,
+			'with_monitored_items' => true,
+			'preservekeys' => true,
+			'limit' => $limit
+		]);
+		$hostids = array_keys($db_hosts);
+	}
 
-	$db_triggers = getTriggersWithActualSeverity([
-		'output' => ['triggerid', 'priority', 'value'],
-		'selectItems' => ['itemid'],
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
-		'monitored' => true,
-		'preservekeys' => true
-	], ['show_suppressed' => $show_suppressed]);
-
-	foreach ($db_triggers as $db_trigger) {
-		foreach ($db_trigger['items'] as $item) {
-			if (array_key_exists($item['itemid'], $db_items)) {
-				$db_item = &$db_items[$item['itemid']];
-
-				// a little tricky check for attempt to overwrite active trigger (value=1) with
-				// inactive or active trigger with lower priority.
-				if (!array_key_exists('triggerid', $db_item)
-						|| ($db_item['value'] == TRIGGER_VALUE_FALSE && $db_trigger['value'] == TRIGGER_VALUE_TRUE)
-						|| (($db_item['value'] == TRIGGER_VALUE_FALSE || $db_trigger['value'] == TRIGGER_VALUE_TRUE)
-							&& $db_item['priority'] < $db_trigger['priority'])) {
-					$db_item['triggerid'] = $db_trigger['triggerid'];
-					$db_item['priority'] = $db_trigger['priority'];
-					$db_item['value'] = $db_trigger['value'];
-				}
-
-				$db_item['acknowledged'] = $db_trigger['problem']['acknowledged'];
-
-				unset($db_item);
-			}
-		}
+	if ($application !== '') {
+		$db_items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'],
+			'selectHosts' => ['name'],
+			'applicationids' => $applicationids,
+			'monitored' => true,
+			'webitems' => true,
+			'preservekeys' => true
+		]);
+	}
+	else {
+		$db_items = API::Item()->get([
+			'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'],
+			'selectHosts' => ['name'],
+			'hostids' => $hostids,
+			'groupids' => $groupids,
+			'monitored' => true,
+			'webitems' => true,
+			'preservekeys' => true
+		]);
 	}
 
 	$db_items = CMacrosResolverHelper::resolveItemNames($db_items);
@@ -1076,28 +1136,29 @@ function getItemsDataOverview(array $groupids, $application, $viewMode,
 		['field' => 'itemid', 'order' => ZBX_SORT_UP]
 	]);
 
-	// fetch latest values
-	$history = Manager::History()->getLastValues(zbx_toHash($db_items, 'itemid'), 1, ZBX_HISTORY_PERIOD);
+	return [$db_items, $hostids];
+}
 
-	// fetch data for the host JS menu
-	$hosts = API::Host()->get([
-		'output' => ['name', 'hostid'],
-		'monitored_hosts' => true,
-		'groupids' => $groupids ? $groupids : null,
-		'applicationids' => $applicationids,
-		'with_monitored_items' => true,
-		'preservekeys' => true
-	]);
+/**
+ * @param array   $groupids
+ * @param array   $hostids
+ * @param array   $filter
+ * @param string  $filter['application']
+ * @param int     $filter['show_suppressed']
+ *
+ * @return array
+ */
+function getDataOverview(?array $groupids, ?array $hostids, array $filter): array {
+	[$db_items, $hostids] = getDataOverviewItems($groupids, $hostids, $filter['application']);
 
-	$items = [];
+	$data = [];
 	$item_counter = [];
-	$host_items = [];
-	$host_names = [];
+	$db_hosts = [];
 
 	foreach ($db_items as $db_item) {
 		$item_name = $db_item['name_expanded'];
 		$host_name = $db_item['hosts'][0]['name'];
-		$host_names[$db_item['hostid']] = $host_name;
+		$db_hosts[$db_item['hostid']] = $db_item['hosts'][0];
 
 		if (!array_key_exists($host_name, $item_counter)) {
 			$item_counter[$host_name] = [];
@@ -1107,140 +1168,154 @@ function getItemsDataOverview(array $groupids, $application, $viewMode,
 			$item_counter[$host_name][$item_name] = 0;
 		}
 
-		if (!array_key_exists($item_name, $host_items) || !array_key_exists($host_name, $host_items[$item_name])) {
-			$host_items[$item_name][$host_name] = [];
-		}
+		$item_place = $item_counter[$host_name][$item_name];
+		$item_counter[$host_name][$item_name]++;
 
-		if (!array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
-			if (array_key_exists($db_item['itemid'], $host_items[$item_name][$host_name])) {
-				$item_place = $host_items[$item_name][$host_name][$db_item['itemid']]['item_place'];
-			}
-			else {
-				$item_place = $item_counter[$host_name][$item_name];
-				$item_counter[$host_name][$item_name]++;
-			}
+		$item = [
+			'itemid' => $db_item['itemid'],
+			'value_type' => $db_item['value_type'],
+			'units' => $db_item['units'],
+			'valuemapid' => $db_item['valuemapid'],
+			'acknowledged' => array_key_exists('acknowledged', $db_item) ? $db_item['acknowledged'] : 0
+		];
 
-			$item = [
-				'itemid' => $db_item['itemid'],
-				'value_type' => $db_item['value_type'],
-				'value' => isset($history[$db_item['itemid']]) ? $history[$db_item['itemid']][0]['value'] : null,
-				'units' => $db_item['units'],
-				'valuemapid' => $db_item['valuemapid'],
-				'item_place' => $item_place,
-				'acknowledged' => array_key_exists('acknowledged', $db_item) ? $db_item['acknowledged'] : 0
+		if (array_key_exists('triggerid', $db_item)) {
+			$item += [
+				'triggerid' => $db_item['triggerid'],
+				'severity' => $db_item['priority'],
+				'tr_value' => $db_item['value']
 			];
-
-			if (array_key_exists('triggerid', $db_item)) {
-				$item += [
-					'triggerid' => $db_item['triggerid'],
-					'severity' => $db_item['priority'],
-					'tr_value' => $db_item['value']
-				];
-			}
-			else {
-				$item += [
-					'triggerid' => null,
-					'severity' => null,
-					'tr_value' => null
-				];
-			}
-
-			$items[$item_name][$item_place][$host_name] = $item;
-
-			$host_items[$item_name][$host_name][$db_item['itemid']] = $items[$item_name][$item_place][$host_name];
 		}
+		else {
+			$item += [
+				'triggerid' => null,
+				'severity' => null,
+				'tr_value' => null
+			];
+		}
+
+		$data[$item_name][$item_place][$host_name] = $item;
 	}
 
-	$table = (new CTableInfo())->setHeadingColumn(0);
-	if (!$host_names) {
-		return $table;
-	}
-	$table->makeVerticalRotation();
+	CArrayHelper::sort($db_hosts, [
+		['field' => 'name', 'order' => ZBX_SORT_UP],
+	]);
 
-	order_result($host_names);
+	$data_display_limit = (int) CSettingsHelper::get(CSettingsHelper::MAX_OVERVIEW_TABLE_SIZE);
+	$has_hidden_hosts = (count($db_hosts) > $data_display_limit);
+	$db_hosts = array_slice($db_hosts, 0, $data_display_limit, true);
 
-	if ($viewMode == STYLE_TOP) {
-		$header = [_('Items')];
-		foreach ($host_names as $host_name) {
-			$header[] = (new CColHeader($host_name))
-				->addClass('vertical_rotation')
-				->setTitle($host_name);
+	$data = array_slice($data, 0, $data_display_limit);
+	$items_left = $data_display_limit;
+	$itemids = [];
+	array_walk($data, function (array &$item_columns) use ($data_display_limit, &$itemids, &$items_left) {
+		if ($items_left != 0) {
+			$item_columns = array_slice($item_columns, 0, min($data_display_limit, $items_left));
+			$items_left -= count($item_columns);
 		}
-		$table->setHeader($header);
-
-		foreach ($items as $item_name => $item_data) {
-			foreach ($item_data as $ithosts) {
-				$tableRow = [(new CColHeader($item_name))->addClass(ZBX_STYLE_NOWRAP)];
-				foreach ($host_names as $host_name) {
-					$tableRow = getItemDataOverviewCells($tableRow, $ithosts, $host_name);
-				}
-				$table->addRow($tableRow);
-			}
+		else {
+			$item_columns = null;
+			return;
 		}
-	}
-	else {
-		$header = [_('Hosts')];
-		foreach ($items as $item_name => $item_data) {
-			foreach ($item_data as $ithosts) {
-				$header[] = (new CColHeader($item_name))
-					->addClass('vertical_rotation')
-					->setTitle($item_name);
-			}
-		}
-		$table->setHeader($header);
 
-		foreach ($host_names as $hostId => $host_name) {
-			$host = $hosts[$hostId];
+		array_walk($item_columns, function (array &$item_column) use ($data_display_limit, &$itemids) {
+			$item_column = array_slice($item_column, 0, $data_display_limit);
+			$itemids += array_column($item_column, 'itemid', 'itemid');
+		});
+	});
+	$data = array_filter($data);
 
-			$name = (new CLinkAction($host['name']))->setMenuPopup(CMenuPopupHelper::getHost($hostId));
+	$has_hidden_items = (count($db_items) != count($itemids));
 
-			$tableRow = [(new CColHeader($name))->addClass(ZBX_STYLE_NOWRAP)];
-			foreach ($items as $item_data) {
-				foreach ($item_data as $ithosts) {
-					$tableRow = getItemDataOverviewCells($tableRow, $ithosts, $host_name);
-				}
-			}
-			$table->addRow($tableRow);
-		}
-	}
+	$db_items = array_intersect_key($db_items, $itemids);
+	$data = getDataOverviewCellData($db_items, $data, $filter['show_suppressed']);
 
-	return $table;
+	return [$data, $db_hosts, ($has_hidden_items || $has_hidden_hosts)];
 }
 
-function getItemDataOverviewCells($tableRow, $ithosts, $hostName) {
+/**
+ * Prepares interfaces select element with options.
+ *
+ * @param array $interfaces
+ *
+ * @return CSelect
+ */
+function getInterfaceSelect(array $interfaces): CSelect {
+	$interface_select = new CSelect('interfaceid');
+
+	/** @var CSelectOption[] $options_by_type */
+	$options_by_type = [];
+
+	foreach ($interfaces as $interface) {
+		$label = $interface['useip']
+			? $interface['ip'].' : '.$interface['port']
+			: $interface['dns'].' : '.$interface['port'];
+
+		$option = new CSelectOption($interface['interfaceid'], $label);
+
+		if ($interface['type'] == INTERFACE_TYPE_SNMP) {
+			$version = $interface['details']['version'];
+			if ($version == SNMP_V3) {
+				$option->setExtra('description', sprintf('%s: %d, %s: %s', _('Version'), $version,
+					_('Context name'), $interface['details']['contextname']
+				));
+			} else {
+				$option->setExtra('description', sprintf('%s: %d, %s: %s', _('Version'), $version,
+					_x('Community', 'SNMP Community'), $interface['details']['community']
+				));
+			}
+		}
+
+		$options_by_type[$interface['type']][] = $option;
+	}
+
+	foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
+		if (array_key_exists($interface_type, $options_by_type)) {
+			$interface_group = new CSelectOptionGroup((string) interfaceType2str($interface_type));
+
+			if ($interface_type == INTERFACE_TYPE_SNMP) {
+				$interface_group->setOptionTemplate('#{label}'.(new CDiv('#{description}'))->addClass('description'));
+			}
+
+			$interface_group->addOptions($options_by_type[$interface_type]);
+
+			$interface_select->addOptionGroup($interface_group);
+		}
+	}
+
+	return $interface_select;
+}
+
+/**
+ * @param array $item
+ * @param array $trigger
+ *
+ * @return CCol
+ */
+function getItemDataOverviewCell(array $item, ?array $trigger = null): CCol {
 	$ack = null;
 	$css = '';
 	$value = UNKNOWN_VALUE;
 
-	if (isset($ithosts[$hostName])) {
-		$item = $ithosts[$hostName];
+	if ($trigger && $trigger['value'] == TRIGGER_VALUE_TRUE) {
+		$css = getSeverityStyle($trigger['priority']);
 
-		if ($item['tr_value'] == TRIGGER_VALUE_TRUE) {
-			$css = getSeverityStyle($item['severity']);
-			$ack = ($item['acknowledged'] == 1) ? [' ', (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)] : null;
-		}
-
-		if ($item['value'] !== null) {
-			$value = formatHistoryValue($item['value'], $item);
+		if ($trigger['problem']['acknowledged'] == 1) {
+			$ack = [' ', (new CSpan())->addClass(ZBX_STYLE_ICON_ACKN)];
 		}
 	}
 
-	if ($value != UNKNOWN_VALUE) {
-		$value = $value;
+	if ($item['value'] !== null) {
+		$value = formatHistoryValue($item['value'], $item);
 	}
 
-	$column = (new CCol([$value, $ack]))->addClass($css);
+	$col = (new CCol([$value, $ack]))
+		->addClass($css)
+		->addClass(ZBX_STYLE_NOWRAP)
+		->setMenuPopup(CMenuPopupHelper::getHistory($item['itemid']))
+		->addClass(ZBX_STYLE_CURSOR_POINTER);
 
-	if (isset($ithosts[$hostName])) {
-		$column
-			->setMenuPopup(CMenuPopupHelper::getHistory($item['itemid']))
-			->addClass(ZBX_STYLE_CURSOR_POINTER)
-			->addClass(ZBX_STYLE_NOWRAP);
-	}
-
-	$tableRow[] = $column;
-
-	return $tableRow;
+	return $col;
 }
 
 /**
@@ -1312,15 +1387,12 @@ function formatHistoryValue($value, array $item, $trim = true) {
 
 	// format value
 	if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-		$value = convert_units([
-				'value' => $value,
-				'units' => $item['units']
+		$value = convertUnits([
+			'value' => $value,
+			'units' => $item['units']
 		]);
 	}
-	elseif ($item['value_type'] != ITEM_VALUE_TYPE_STR
-		&& $item['value_type'] != ITEM_VALUE_TYPE_TEXT
-		&& $item['value_type'] != ITEM_VALUE_TYPE_LOG) {
-
+	elseif (!in_array($item['value_type'], [ITEM_VALUE_TYPE_STR, ITEM_VALUE_TYPE_TEXT, ITEM_VALUE_TYPE_LOG])) {
 		$value = _('Unknown value type');
 	}
 
@@ -1328,7 +1400,8 @@ function formatHistoryValue($value, array $item, $trim = true) {
 	switch ($item['value_type']) {
 		case ITEM_VALUE_TYPE_STR:
 			$mapping = getMappedValue($value, $item['valuemapid']);
-		// break; is not missing here
+			// break; is not missing here
+
 		case ITEM_VALUE_TYPE_TEXT:
 		case ITEM_VALUE_TYPE_LOG:
 			if ($trim && mb_strlen($value) > 20) {
@@ -1338,7 +1411,9 @@ function formatHistoryValue($value, array $item, $trim = true) {
 			if ($mapping !== false) {
 				$value = $mapping.' ('.$value.')';
 			}
+
 			break;
+
 		default:
 			$value = applyValueMap($value, $item['valuemapid']);
 	}
@@ -1360,33 +1435,37 @@ function formatHistoryValue($value, array $item, $trim = true) {
  * @return string item functional value from history
  */
 function getItemFunctionalValue($item, $function, $parameter) {
-	// check whether function is allowed
+	// Check whether function is allowed and parameter is specified.
 	if (!in_array($function, ['min', 'max', 'avg']) || $parameter === '') {
 		return UNRESOLVED_MACRO_STRING;
 	}
 
-	$parameter = convertFunctionValue($parameter);
-
-	if (bccomp($parameter, 0) == 0) {
+	// Check whether item type is allowed for min, max and avg functions.
+	if ($item['value_type'] != ITEM_VALUE_TYPE_FLOAT && $item['value_type'] != ITEM_VALUE_TYPE_UINT64) {
 		return UNRESOLVED_MACRO_STRING;
 	}
 
-	// allowed item types for min, max and avg function
-	$history_tables = [ITEM_VALUE_TYPE_FLOAT => 'history', ITEM_VALUE_TYPE_UINT64 => 'history_uint'];
+	$number_parser = new CNumberParser(['with_suffix' => true]);
 
-	if (!array_key_exists($item['value_type'], $history_tables)) {
+	if ($number_parser->parse($parameter) != CParser::PARSE_SUCCESS) {
 		return UNRESOLVED_MACRO_STRING;
 	}
+
+	$parameter = $number_parser->calcValue();
+
+	$time_from = time() - $parameter;
+
+	if ($time_from < 0 || $time_from > ZBX_MAX_DATE) {
+		return UNRESOLVED_MACRO_STRING;
+	}
+
+	$result = Manager::History()->getAggregatedValue($item, $function, $time_from);
+
+	if ($result !== null) {
+		return convertUnits(['value' => $result, 'units' => $item['units']]);
+	}
 	else {
-		$result = Manager::History()->getAggregatedValue($item, $function, (time() - $parameter));
-
-		if ($result !== null) {
-			return convert_units(['value' => $result, 'units' => $item['units']]);
-		}
-		// no data in history
-		else {
-			return UNRESOLVED_MACRO_STRING;
-		}
+		return UNRESOLVED_MACRO_STRING;
 	}
 }
 
@@ -1644,6 +1723,8 @@ function httpItemExists($items) {
 
 function getParamFieldNameByType($itemType) {
 	switch ($itemType) {
+		case ITEM_TYPE_SCRIPT:
+			return 'script';
 		case ITEM_TYPE_SSH:
 		case ITEM_TYPE_TELNET:
 		case ITEM_TYPE_JMX:
@@ -1659,6 +1740,8 @@ function getParamFieldNameByType($itemType) {
 
 function getParamFieldLabelByType($itemType) {
 	switch ($itemType) {
+		case ITEM_TYPE_SCRIPT:
+			return _('Script');
 		case ITEM_TYPE_SSH:
 		case ITEM_TYPE_TELNET:
 		case ITEM_TYPE_JMX:
@@ -1699,6 +1782,10 @@ function get_preprocessing_types($type = null, $grouped = true, array $supported
 			'group' => _('Text'),
 			'name' => _('Regular expression')
 		],
+		ZBX_PREPROC_STR_REPLACE => [
+			'group' => _('Text'),
+			'name' => _('Replace')
+		],
 		ZBX_PREPROC_TRIM => [
 			'group' => _('Text'),
 			'name' => _('Trim')
@@ -1775,6 +1862,10 @@ function get_preprocessing_types($type = null, $grouped = true, array $supported
 			'group' => _('Validation'),
 			'name' => _('Check for error using regular expression')
 		],
+		ZBX_PREPROC_VALIDATE_NOT_SUPPORTED => [
+			'group' => _('Validation'),
+			'name' => _('Check for not supported value')
+		],
 		ZBX_PREPROC_THROTTLE_VALUE => [
 			'group' => _('Throttling'),
 			'name' => _('Discard unchanged')
@@ -1916,7 +2007,7 @@ function expandItemNamesWithMasterItems($items, $data_source) {
 					: $items[$items_index]['type'],
 				'source' => ($items_index === false)
 					? $master_items[$master_itemid]['source']
-					: $items[$items_index]['source'],
+					: $items[$items_index]['source']
 			];
 		}
 	}
@@ -1933,11 +2024,8 @@ function expandItemNamesWithMasterItems($items, $data_source) {
 function checkNowAllowedTypes() {
 	return [
 		ITEM_TYPE_ZABBIX,
-		ITEM_TYPE_SNMPV1,
 		ITEM_TYPE_SIMPLE,
-		ITEM_TYPE_SNMPV2C,
 		ITEM_TYPE_INTERNAL,
-		ITEM_TYPE_SNMPV3,
 		ITEM_TYPE_AGGREGATE,
 		ITEM_TYPE_EXTERNAL,
 		ITEM_TYPE_DB_MONITOR,
@@ -1946,6 +2034,65 @@ function checkNowAllowedTypes() {
 		ITEM_TYPE_TELNET,
 		ITEM_TYPE_CALCULATED,
 		ITEM_TYPE_JMX,
-		ITEM_TYPE_HTTPAGENT
+		ITEM_TYPE_HTTPAGENT,
+		ITEM_TYPE_SCRIPT,
+		ITEM_TYPE_SNMP
 	];
 }
+
+/**
+ * Validates update interval for items, item prototypes and low-level discovery rules and their overrides.
+ *
+ * @param CUpdateIntervalParser $parser [IN]      Parser used for delay validation.
+ * @param string                $value  [IN]      Update interval to parse and validate.
+ * @param string                $field_name [IN]  Frontend or API field name in the error
+ * @param string                $error  [OUT]     Returned error string if delay validation fails.
+ *
+ * @return bool
+ */
+function validateDelay(CUpdateIntervalParser $parser, $field_name, $value, &$error) {
+	if ($parser->parse($value) != CParser::PARSE_SUCCESS) {
+		$error = _s('Incorrect value for field "%1$s": %2$s.', $field_name, _('invalid delay'));
+
+		return false;
+	}
+
+	$delay = $parser->getDelay();
+
+	if ($delay[0] !== '{') {
+		$delay_sec = timeUnitToSeconds($delay);
+		$intervals = $parser->getIntervals();
+		$flexible_intervals = $parser->getIntervals(ITEM_DELAY_FLEXIBLE);
+		$has_scheduling_intervals = (bool) $parser->getIntervals(ITEM_DELAY_SCHEDULING);
+		$has_macros = false;
+
+		foreach ($intervals as $interval) {
+			if (strpos($interval['interval'], '{') !== false) {
+				$has_macros = true;
+				break;
+			}
+		}
+
+		// If delay is 0, there must be at least one either flexible or scheduling interval.
+		if ($delay_sec == 0 && !$intervals) {
+			$error = _('Item will not be refreshed. Specified update interval requires having at least one either flexible or scheduling interval.');
+
+			return false;
+		}
+		elseif ($delay_sec < 0 || $delay_sec > SEC_PER_DAY) {
+			$error = _('Item will not be refreshed. Update interval should be between 1s and 1d. Also Scheduled/Flexible intervals can be used.');
+
+			return false;
+		}
+
+		// If there are scheduling intervals or intervals with macros, skip the next check calculation.
+		if (!$has_macros && !$has_scheduling_intervals && $flexible_intervals
+				&& calculateItemNextCheck(0, $delay_sec, $flexible_intervals, time()) == ZBX_JAN_2038) {
+			$error = _('Item will not be refreshed. Please enter a correct update interval.');
+
+			return false;
+		}
+	}
+
+	return true;
+}
