@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 $widget = (new CWidget())->setTitle(_('Items'));
 
 $host = $data['host'];
@@ -29,6 +33,7 @@ if (!empty($data['hostid'])) {
 
 // Create form.
 $form = (new CForm())
+	->setId('item-form')
 	->setName('itemForm')
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
@@ -84,7 +89,7 @@ else {
 }
 
 // Append key to form list.
-$key_controls = [(new CTextBox('key', $data['key'], $readonly))
+$key_controls = [(new CTextBox('key', $data['key'], $readonly, DB::getFieldLength('items', 'key_')))
 	->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	->setAriaRequired()
 ];
@@ -94,7 +99,7 @@ if (!$readonly) {
 	$key_controls[] = (new CButton('keyButton', _('Select')))
 		->addClass(ZBX_STYLE_BTN_GREY)
 		->onClick('return PopUp("popup.generic",jQuery.extend('.
-			CJs::encodeJson([
+			json_encode([
 				'srctbl' => 'help_items',
 				'srcfld1' => 'key',
 				'dstfrm' => $form->getName(),
@@ -123,7 +128,7 @@ $form_list
 		'url_row'
 	);
 
-// Append ITEM_TYPE_HTTPAGENT Query fields to form list.
+// Prepare ITEM_TYPE_HTTPAGENT query fields.
 $query_fields_data = [];
 
 if (is_array($data['query_fields']) && $data['query_fields']) {
@@ -136,9 +141,58 @@ elseif (!$readonly) {
 }
 
 $query_fields = (new CTag('script', true))->setAttribute('type', 'text/json');
-$query_fields->items = [CJs::encodeJson($query_fields_data)];
+$query_fields->items = [json_encode($query_fields_data)];
+
+// Prepare ITEM_TYPE_SCRIPT parameters.
+$parameters_data = [];
+if ($data['parameters']) {
+	$parameters_data = $data['parameters'];
+}
+elseif (!$readonly) {
+	$parameters_data[] = ['name' => '', 'value' => ''];
+}
+
+$parameters_table = (new CTable())
+	->setId('parameters_table')
+	->setHeader([
+		(new CColHeader(_('Name')))->setWidth('50%'),
+		(new CColHeader(_('Value')))->setWidth('50%'),
+		_('Action')
+	])
+	->setAttribute('style', 'width: 100%;');
+
+if ($parameters_data) {
+	foreach ($parameters_data as $parameter) {
+		$parameters_table->addRow([
+			(new CTextBox('parameters[name][]', $parameter['name'], $readonly,
+				DB::getFieldLength('item_parameter', 'name'))
+			)
+				->setAttribute('style', 'width: 100%;')
+				->removeId(),
+			(new CTextBox('parameters[value][]', $parameter['value'], $readonly,
+				DB::getFieldLength('item_parameter', 'value'))
+			)
+				->setAttribute('style', 'width: 100%;')
+				->removeId(),
+			(new CButton('', _('Remove')))
+				->removeId()
+				->onClick('jQuery(this).closest("tr").remove()')
+				->addClass(ZBX_STYLE_BTN_LINK)
+				->addClass('element-table-remove')
+				->setEnabled(!$readonly)
+		]);
+	}
+}
+
+$parameters_table->addRow([
+	(new CButton('parameter_add', _('Add')))
+		->addClass(ZBX_STYLE_BTN_LINK)
+		->addClass('element-table-add')
+		->setEnabled(!$readonly)
+]);
 
 $form_list
+	// Append ITEM_TYPE_HTTPAGENT Query fields to form list.
 	->addRow(
 		new CLabel(_('Query fields'), 'query_fields_pairs'),
 		(new CDiv([
@@ -178,6 +232,48 @@ $form_list
 			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH . 'px;'),
 		'query_fields_row'
 	)
+	// Append ITEM_TYPE_SCRIPT parameters to form list.
+	->addItem(
+		(new CTag('script', true))
+			->setId('parameters_table_row')
+			->setAttribute('type', 'text/x-jquery-tmpl')
+			->addItem(
+				(new CRow([
+					(new CTextBox('parameters[name][]', '', false, DB::getFieldLength('item_parameter', 'name')))
+						->setAttribute('style', 'width: 100%;')
+						->removeId(),
+					(new CTextBox('parameters[value][]', '', false, DB::getFieldLength('item_parameter', 'value')))
+						->setAttribute('style', 'width: 100%;')
+						->removeId(),
+					(new CButton('', _('Remove')))
+						->removeId()
+						->onClick('jQuery(this).closest("tr").remove()')
+						->addClass(ZBX_STYLE_BTN_LINK)
+						->addClass('element-table-remove')
+				]))
+			)
+	)
+	->addRow(
+		new CLabel(_('Parameters'), $parameters_table->getId()),
+		(new CDiv($parameters_table))
+			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
+			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px;'),
+		'parameters_row'
+	)
+	->addRow((new CLabel(_('Script'), 'script'))->setAsteriskMark(),
+		(new CMultilineInput('script', $data['params'], [
+			'title' => _('JavaScript'),
+			'placeholder' => _('script'),
+			'placeholder_textarea' => 'return value',
+			'grow' => 'auto',
+			'rows' => 0,
+			'maxlength' => DB::getFieldLength('items', 'params'),
+			'readonly' => $readonly
+		]))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->setAriaRequired(),
+		'script_row'
+	)
 	// Append ITEM_TYPE_HTTPAGENT Request type to form list.
 	->addRow(
 		new CLabel(_('Request type'), 'request_method'),
@@ -192,10 +288,12 @@ $form_list
 		],
 		'request_method_row'
 	)
-	// Append ITEM_TYPE_HTTPAGENT Timeout field to fomr list.
+	// Append ITEM_TYPE_HTTPAGENT and ITEM_TYPE_SCRIPT timeout field to form list.
 	->addRow(
-		new CLabel(_('Timeout'), 'timeout'),
-		(new CTextBox('timeout', $data['timeout'], $readonly))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
+		(new CLabel(_('Timeout'), 'timeout'))->setAsteriskMark(),
+		(new CTextBox('timeout', $data['timeout'], $readonly))
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+			->setAriaRequired(),
 		'timeout_row'
 	)
 	// Append ITEM_TYPE_HTTPAGENT Request body type to form list.
@@ -227,7 +325,7 @@ elseif (!$readonly) {
 	$headers_data[] = ['name' => '', 'value' => ''];
 }
 $headers = (new CTag('script', true))->setAttribute('type', 'text/json');
-$headers->items = [CJs::encodeJson($headers_data)];
+$headers->items = [json_encode($headers_data)];
 
 $form_list
 	// Append ITEM_TYPE_HTTPAGENT Headers fields to form list.
@@ -309,7 +407,8 @@ $form_list
 		new CLabel(_('HTTP proxy'), 'http_proxy'),
 		(new CTextBox('http_proxy', $data['http_proxy'], $readonly, DB::getFieldLength('items', 'http_proxy')))
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAttribute('placeholder', '[protocol://][user[:password]@]proxy.example.com[:port]'),
+			->setAttribute('placeholder', '[protocol://][user[:password]@]proxy.example.com[:port]')
+			->disableAutocomplete(),
 		'http_proxy_row'
 	)
 	// Append ITEM_TYPE_HTTPAGENT HTTP authentication to form list.
@@ -317,12 +416,8 @@ $form_list
 		new CLabel(_('HTTP authentication'), 'http_authtype'),
 		[
 			$readonly ? new CVar('http_authtype', $data['http_authtype']) : null,
-			(new CComboBox($readonly ? '' : 'http_authtype', $data['http_authtype'], null, [
-				HTTPTEST_AUTH_NONE => _('None'),
-				HTTPTEST_AUTH_BASIC => _('Basic'),
-				HTTPTEST_AUTH_NTLM => _('NTLM'),
-				HTTPTEST_AUTH_KERBEROS => _('Kerberos')
-			]))->setEnabled(!$readonly)
+			(new CComboBox($readonly ? '' : 'http_authtype', $data['http_authtype'], null, httptest_authentications()))
+				->setEnabled(!$readonly)
 		],
 		'http_authtype_row'
 	)
@@ -330,14 +425,16 @@ $form_list
 	->addRow(
 		new CLabel(_('User name'), 'http_username'),
 		(new CTextBox('http_username', $data['http_username'], $readonly, DB::getFieldLength('items', 'username')))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->disableAutocomplete(),
 		'http_username_row'
 	)
 	// Append ITEM_TYPE_HTTPAGENT Password to form list.
 	->addRow(
 		new CLabel(_('Password'), 'http_password'),
 		(new CTextBox('http_password', $data['http_password'], $readonly, DB::getFieldLength('items', 'password')))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->disableAutocomplete(),
 		'http_password_row'
 	)
 	// Append ITEM_TYPE_HTTPAGENT SSL verify peer to form list.
@@ -375,7 +472,9 @@ $form_list
 		new CLabel(_('SSL key password'), 'ssl_key_password'),
 		(new CTextBox('ssl_key_password', $data['ssl_key_password'], $readonly,
 			DB::getFieldLength('items', 'ssl_key_password')
-		))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+		))
+			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+			->disableAutocomplete(),
 		'ssl_key_password_row'
 	)
 	// Append master item select to form list.
@@ -414,12 +513,13 @@ $form_list
 	);
 
 // Append interface(s) to form list.
-if ($data['interfaces']) {
+if ($data['display_interfaces']) {
 	if ($discovered_item) {
 		if ($data['interfaceid'] != 0) {
 			$data['interfaces'] = zbx_toHash($data['interfaces'], 'interfaceid');
 			$interface = $data['interfaces'][$data['interfaceid']];
 
+			$form->addVar('selectedInterfaceId', $data['interfaceid']);
 			$form_list->addRow((new CLabel(_('Host interface'), 'interface'))->setAsteriskMark(),
 				(new CTextBox('interface',
 					$interface['useip']
@@ -432,41 +532,22 @@ if ($data['interfaces']) {
 		}
 	}
 	else {
-		$interfacesComboBox = (new CComboBox('interfaceid', $data['interfaceid']))
+		$select_interface = getInterfaceSelect($data['interfaces'])
+			->setId('interface-select')
+			->setValue($data['interfaceid'])
+			->addClass(ZBX_STYLE_ZSELECT_HOST_INTERFACE)
+			->setFocusableElementId('interfaceid')
 			->setAriaRequired();
 
-		// Set up interface groups sorted by priority.
-		$interface_types = zbx_objectValues($this->data['interfaces'], 'type');
-		$interface_groups = [];
-		foreach ([INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX, INTERFACE_TYPE_IPMI] as $interface_type) {
-			if (in_array($interface_type, $interface_types)) {
-				$interface_groups[$interface_type] = new COptGroup(interfaceType2str($interface_type));
-			}
-		}
-
-		// add interfaces to groups
-		foreach ($data['interfaces'] as $interface) {
-			$option = new CComboItem($interface['interfaceid'],
-				$interface['useip']
-					? $interface['ip'].' : '.$interface['port']
-					: $interface['dns'].' : '.$interface['port'],
-				($interface['interfaceid'] == $data['interfaceid'])
-			);
-			$option->setAttribute('data-interfacetype', $interface['type']);
-			$interface_groups[$interface['type']]->addItem($option);
-		}
-		foreach ($interface_groups as $interface_group) {
-			$interfacesComboBox->addItem($interface_group);
-		}
-
-		$span = (new CSpan(_('No interface found')))
-			->addClass(ZBX_STYLE_RED)
-			->setId('interface_not_defined')
-			->setAttribute('style', 'display: none;');
-
-		$form_list->addRow((new CLabel(_('Host interface'), 'interfaceid'))->setAsteriskMark(),
-			[$interfacesComboBox, $span], 'interface_row'
-		);
+		$form_list->addRow(
+			(new CLabel(_('Host interface'), $select_interface->getFocusableElementId()))->setAsteriskMark(),
+			[
+				$select_interface,
+				(new CSpan(_('No interface found')))
+					->setId('interface_not_defined')
+					->addClass(ZBX_STYLE_RED)
+					->setAttribute('style', 'display: none;')
+			], 'interface_row');
 		$form->addVar('selectedInterfaceId', $data['interfaceid']);
 	}
 }
@@ -480,109 +561,14 @@ $form_list
 			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 			->setAriaRequired(),
 		'row_snmp_oid'
-	)
-	->addRow(_('Context name'),
-		(new CTextBox('snmpv3_contextname', $data['snmpv3_contextname'], $discovered_item))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		'row_snmpv3_contextname'
-	)
-	->addRow(
-		(new CLabel(_('SNMP community'), 'snmp_community'))->setAsteriskMark(),
-		(new CTextBox('snmp_community', $data['snmp_community'], $discovered_item, 64))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired(),
-		'row_snmp_community'
-	)
-	->addRow(_('Security name'),
-		(new CTextBox('snmpv3_securityname', $data['snmpv3_securityname'], $discovered_item, 64))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		'row_snmpv3_securityname'
 	);
 
-// Append SNMPv3 security level to form list.
-$security_levels = [
-	ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV => 'noAuthNoPriv',
-	ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV => 'authNoPriv',
-	ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV => 'authPriv'
-];
-if ($discovered_item) {
-	$form->addVar('snmpv3_securitylevel', $data['snmpv3_securitylevel']);
-	$securityLevelComboBox = new CTextBox('snmpv3_securitylevel_name', $security_levels[$data['snmpv3_securitylevel']],
-		true
-	);
-}
-else {
-	$securityLevelComboBox = new CComboBox('snmpv3_securitylevel', $data['snmpv3_securitylevel'], null,
-		$security_levels
-	);
-}
-$form_list->addRow(_('Security level'), $securityLevelComboBox, 'row_snmpv3_securitylevel');
-
-// Append SNMPv3 authentication protocol to form list.
-if ($discovered_item) {
-	$form->addVar('snmpv3_authprotocol', (int) $data['snmpv3_authprotocol']);
-	$snmpv3_authprotocol = (new CRadioButtonList('snmpv3_authprotocol_names', (int) $data['snmpv3_authprotocol']))
-		->addValue(_('MD5'), ITEM_AUTHPROTOCOL_MD5)
-		->addValue(_('SHA'), ITEM_AUTHPROTOCOL_SHA)
-		->setModern(true)
-		->setEnabled(!$discovered_item);
-}
-else {
-	$snmpv3_authprotocol = (new CRadioButtonList('snmpv3_authprotocol', (int) $data['snmpv3_authprotocol']))
-		->addValue(_('MD5'), ITEM_AUTHPROTOCOL_MD5)
-		->addValue(_('SHA'), ITEM_AUTHPROTOCOL_SHA)
-		->setModern(true);
-}
-
 $form_list
-	->addRow((new CLabel(_('Authentication protocol'), 'snmpv3_authprotocol')),
-		$snmpv3_authprotocol,
-		'row_snmpv3_authprotocol'
-	)
-	// Append SNMPv3 authentication passphrase to form list.
-	->addRow(_('Authentication passphrase'),
-		(new CTextBox('snmpv3_authpassphrase', $data['snmpv3_authpassphrase'], $discovered_item, 64))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		'row_snmpv3_authpassphrase'
-	);
 
-// Append SNMPv3 privacy protocol to form list.
-if ($discovered_item) {
-	$form->addVar('snmpv3_privprotocol', (int) $data['snmpv3_privprotocol']);
-	$snmpv3_privprotocol = (new CRadioButtonList('snmpv3_privprotocol_names', (int) $data['snmpv3_privprotocol']))
-		->addValue(_('DES'), ITEM_PRIVPROTOCOL_DES)
-		->addValue(_('AES'), ITEM_PRIVPROTOCOL_AES)
-		->setModern(true)
-		->setEnabled(!$discovered_item);
-}
-else {
-	$snmpv3_privprotocol = (new CRadioButtonList('snmpv3_privprotocol', (int) $data['snmpv3_privprotocol']))
-		->addValue(_('DES'), ITEM_PRIVPROTOCOL_DES)
-		->addValue(_('AES'), ITEM_PRIVPROTOCOL_AES)
-		->setModern(true);
-}
-$form_list
-	->addRow((new CLabel(_('Privacy protocol'), 'snmpv3_privprotocol')),
-		$snmpv3_privprotocol,
-		'row_snmpv3_privprotocol'
-	)
-	// Append SNMPv3 privacy passphrase to form list.
-	->addRow(_('Privacy passphrase'),
-		(new CTextBox('snmpv3_privpassphrase', $data['snmpv3_privpassphrase'], $discovered_item, 64))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
-		'row_snmpv3_privpassphrase'
-	)
-	->addRow(_('Port'),
-		(new CTextBox('port', $data['port'], $discovered_item, 64))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
-		'row_port'
-	)
-	->addRow(
-		(new CLabel(_('IPMI sensor'), 'ipmi_sensor'))->setAsteriskMark(),
-		(new CTextBox('ipmi_sensor', $data['ipmi_sensor'], $readonly, 128))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired(),
-		'row_ipmi_sensor'
-	);
+->addRow(_('IPMI sensor'),
+	(new CTextBox('ipmi_sensor', $data['ipmi_sensor'], $readonly, 128))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
+	'row_ipmi_sensor'
+);
 
 // Append authentication method to form list.
 $auth_types = [
@@ -606,7 +592,9 @@ $form_list
 		'row_jmx_endpoint'
 	)
 	->addRow(_('User name'),
-		(new CTextBox('username', $data['username'], $discovered_item, 64))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
+		(new CTextBox('username', $data['username'], $discovered_item, 64))
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+			->disableAutocomplete(),
 		'row_username'
 	)
 	->addRow(
@@ -624,7 +612,9 @@ $form_list
 		'row_privatekey'
 	)
 	->addRow(_('Password'),
-		(new CTextBox('password', $data['password'], $discovered_item, 64))->setWidth(ZBX_TEXTAREA_SMALL_WIDTH),
+		(new CTextBox('password', $data['password'], $discovered_item, 64))
+			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
+			->disableAutocomplete(),
 		'row_password'
 	)
 	->addRow(
@@ -658,13 +648,13 @@ $form_list
 // Append value type to form list.
 if ($readonly) {
 	$form->addVar('value_type', $data['value_type']);
-	$form_list->addRow((new CLabel(_('Type of information'), 'value_type_name'))->setAsteriskMark(),
+	$form_list->addRow(new CLabel(_('Type of information'), 'value_type_name'),
 		(new CTextBox('value_type_name', itemValueTypeString($data['value_type']), true))
 			->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
 	);
 }
 else {
-	$form_list->addRow((new CLabel(_('Type of information'), 'value_type')),
+	$form_list->addRow(new CLabel(_('Type of information'), 'value_type'),
 		(new CComboBox('value_type', $data['value_type'], null, [
 			ITEM_VALUE_TYPE_UINT64 => _('Numeric (unsigned)'),
 			ITEM_VALUE_TYPE_FLOAT => _('Numeric (float)'),
@@ -753,10 +743,13 @@ $form_list->addRow(_('Custom intervals'),
 
 // Append history storage to form list.
 $keep_history_hint = null;
-if ($data['config']['hk_history_global']
-		&& ($host['status'] == HOST_STATUS_MONITORED || $host['status'] == HOST_STATUS_NOT_MONITORED)) {
+if ($data['config']['hk_history_global']  && ($host['status'] == HOST_STATUS_MONITORED
+			|| $host['status'] == HOST_STATUS_NOT_MONITORED)) {
 	$link = (CWebUser::getType() == USER_TYPE_SUPER_ADMIN)
-		? (new CLink(_x('global housekeeping settings', 'item_form'), 'adm.housekeeper.php'))
+		? (new CLink(_x('global housekeeping settings', 'item_form'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'housekeeping.edit')
+				->getUrl()
+			))
 				->setAttribute('target', '_blank')
 		: _x('global housekeeping settings', 'item_form');
 
@@ -786,10 +779,13 @@ $form_list->addRow((new CLabel(_('History storage period'), 'history'))->setAste
 
 // Append trend storage to form list.
 $keep_trend_hint = null;
-if ($data['config']['hk_trends_global']
-		&& ($host['status'] == HOST_STATUS_MONITORED || $host['status'] == HOST_STATUS_NOT_MONITORED)) {
+if ($data['config']['hk_trends_global'] && ($host['status'] == HOST_STATUS_MONITORED
+			|| $host['status'] == HOST_STATUS_NOT_MONITORED)) {
 	$link = (CWebUser::getType() == USER_TYPE_SUPER_ADMIN)
-		? (new CLink(_x('global housekeeping settings', 'item_form'), 'adm.housekeeper.php'))
+		? (new CLink(_x('global housekeeping settings', 'item_form'), (new CUrl('zabbix.php'))
+				->setArgument('action', 'housekeeping.edit')
+				->getUrl()
+			))
 				->setAttribute('target', '_blank')
 		: _x('global housekeeping settings', 'item_form');
 
@@ -841,7 +837,10 @@ else {
 
 if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
 	$valuemapComboBox = [$valuemapComboBox, '&nbsp;',
-		(new CLink(_('show value mappings'), 'adm.valuemapping.php'))->setAttribute('target', '_blank')
+		(new CLink(_('show value mappings'), (new CUrl('zabbix.php'))
+			->setArgument('action', 'valuemap.list')
+			->getUrl()
+		))->setAttribute('target', '_blank')
 	];
 }
 
@@ -943,7 +942,8 @@ $itemTab = (new CTabView())
 		(new CFormList('item_preproc_list'))
 			->addRow(_('Preprocessing steps'),
 				getItemPreprocessing($form, $data['preprocessing'], $readonly, $data['preprocessing_types'])
-			)
+			),
+		TAB_INDICATOR_PREPROCESSING
 	);
 
 if (!hasRequest('form_refresh')) {
@@ -955,19 +955,23 @@ if ($data['itemid'] != 0) {
 	$buttons = [new CSubmit('clone', _('Clone'))];
 
 	if ($data['host']['status'] != HOST_STATUS_TEMPLATE) {
-		$buttons[] = (new CSubmit('check_now', _('Check now')))
+		$buttons[] = (new CSubmit('check_now', _('Execute now')))
 			->setEnabled(in_array($data['item']['type'], checkNowAllowedTypes())
 					&& $data['item']['status'] == ITEM_STATUS_ACTIVE
 					&& $data['host']['status'] == HOST_STATUS_MONITORED
 			);
 	}
 
+	$buttons[] = (new CSimpleButton(_('Test')))->setId('test_item');
+
 	if ($host['status'] == HOST_STATUS_MONITORED || $host['status'] == HOST_STATUS_NOT_MONITORED) {
-		$buttons[] = new CButtonQMessage(
-			'del_history',
-			_('Clear history and trends'),
-			_('History clearing can take a long time. Continue?')
-		);
+		$buttons[] = ($data['config']['compression_status'])
+			? new CSubmit('del_history', _('Clear history and trends'))
+			: new CButtonQMessage(
+				'del_history',
+				_('Clear history and trends'),
+				_('History clearing can take a long time. Continue?')
+			);
 	}
 
 	$buttons[] = (new CButtonDelete(_('Delete item?'), url_params(['form', 'itemid', 'hostid'])))
@@ -979,7 +983,7 @@ if ($data['itemid'] != 0) {
 else {
 	$itemTab->setFooter(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel(url_param('hostid'))]
+		[(new CSimpleButton(_('Test')))->setId('test_item'), new CButtonCancel(url_param('hostid'))]
 	));
 }
 
@@ -988,4 +992,4 @@ $widget->addItem($form);
 
 require_once dirname(__FILE__).'/js/configuration.item.edit.js.php';
 
-return $widget;
+$widget->show();
