@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 zbx_add_post_js('jqBlink.blink();');
 
 // hint table
@@ -37,7 +41,7 @@ for ($severity = TRIGGER_SEVERITY_NOT_CLASSIFIED; $severity < TRIGGER_SEVERITY_C
 		(new CDiv())
 			->addClass(ZBX_STYLE_NOTIF_INDIC)
 			->addClass(getSeverityStyle($severity)),
-		new CTag('h4', true, getSeverityName($severity, $data['config'])),
+		new CTag('h4', true, getSeverityName($severity)),
 		(new CTag('p', true, _('PROBLEM')))->addClass(ZBX_STYLE_GREY)
 	]);
 }
@@ -64,36 +68,43 @@ if ($blink_period > 0) {
 			);
 	}
 	$indic_container->addItem(
-		(new CTag('p', true, _s('Age less than %s', convertUnitsS($blink_period))))->addClass(ZBX_STYLE_GREY)
+		(new CTag('p', true, _s('Age less than %1$s', convertUnitsS($blink_period))))->addClass(ZBX_STYLE_GREY)
 	);
 
 	$help_hint->addItem($indic_container);
 }
 
 // header right
-$web_layout_mode = CView::getLayoutMode();
+$web_layout_mode = CViewHelper::loadLayoutMode();
+
+$submenu_source = [
+	SHOW_TRIGGERS => _('Trigger overview'),
+	SHOW_DATA => _('Data overview')
+];
+
+$submenu = [];
+foreach ($submenu_source as $value => $label) {
+	$url = (new CUrl('overview.php'))
+		->setArgument('type', $value)
+		->getUrl();
+
+	$submenu[$url] = $label;
+}
 
 $widget = (new CWidget())
-	->setTitle(_('Overview'))
+	->setTitle(array_key_exists($this->data['type'], $submenu_source) ? $submenu_source[$this->data['type']] : null)
+	->setTitleSubmenu([
+		'main_section' => [
+			'items' => $submenu
+		]
+	])
 	->setWebLayoutMode($web_layout_mode)
 	->setControls(new CList([
 		(new CForm('get'))
 			->cleanItems()
 			->setAttribute('aria-label', _('Main filter'))
+			->addItem(new CInput('hidden', 'type', $this->data['type']))
 			->addItem((new CList())
-				->addItem([
-					new CLabel(_('Group'), 'groupid'),
-					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					$this->data['pageFilter']->getGroupsCB()
-				])
-				->addItem([
-					new CLabel(_('Type'), 'type'),
-					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-					new CComboBox('type', $this->data['type'], 'submit()', [
-						SHOW_TRIGGERS => _('Triggers'),
-						SHOW_DATA => _('Data')
-					])
-				])
 				->addItem([
 					new CLabel(_('Hosts location'), 'view_style'),
 					(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
@@ -104,48 +115,47 @@ $widget = (new CWidget())
 				])
 			),
 		(new CTag('nav', true, (new CList())
-			->addItem(get_icon('fullscreen'))
+			->addItem(get_icon('kioskmode', ['mode' => $web_layout_mode]))
 			->addItem(get_icon('overviewhelp')->setHint($help_hint))
 		))
 			->setAttribute('aria-label', _('Content controls'))
 	]));
 
-if (in_array($web_layout_mode, [ZBX_LAYOUT_NORMAL, ZBX_LAYOUT_FULLSCREEN])) {
+if ($web_layout_mode == ZBX_LAYOUT_NORMAL) {
 	// filter
-	$filter = $data['filter'];
-	$filterFormView = new CView('common.filter.trigger', [
+	$widget->addItem(new CPartial('common.filter.trigger', [
 		'filter' => [
-			'showTriggers' => $filter['showTriggers'],
-			'ackStatus' => $filter['ackStatus'],
-			'showSeverity' => $filter['showSeverity'],
-			'statusChange' => $filter['statusChange'],
-			'statusChangeDays' => $filter['statusChangeDays'],
-			'txtSelect' => $filter['txtSelect'],
-			'application' => $filter['application'],
-			'inventory' => $filter['inventory'],
-			'show_suppressed' => $filter['show_suppressed'],
-			'hostId' => $data['hostid'],
-			'groupId' => $data['groupid']
+			'showTriggers' => $data['filter']['showTriggers'],
+			'ackStatus' => $data['filter']['ackStatus'],
+			'showSeverity' => $data['filter']['showSeverity'],
+			'statusChange' => $data['filter']['statusChange'],
+			'statusChangeDays' => $data['filter']['statusChangeDays'],
+			'txtSelect' => $data['filter']['txtSelect'],
+			'application' => $data['filter']['application'],
+			'inventory' => $data['filter']['inventory'],
+			'show_suppressed' => $data['filter']['show_suppressed'],
+			'groups' => $data['filter']['groups'],
+			'hosts' => $data['filter']['hosts']
 		],
-		'config' => $data['config'],
 		'profileIdx' => $data['profileIdx'],
 		'active_tab' => $data['active_tab']
-	]);
-	$filterForm = $filterFormView->render();
-
-	$widget->addItem($filterForm);
+	]));
 }
 
-// data table
-if ($data['pageFilter']->groupsSelected) {
-	global $page;
-
-	$dataTable = getTriggersOverview($data['hosts'], $data['triggers'], $page['file'], $data['view_style']);
+if ($data['view_style'] == STYLE_TOP) {
+	$table = new CPartial('trigoverview.table.top', $data);
 }
 else {
-	$dataTable = new CTableInfo();
+	$table = new CPartial('trigoverview.table.left', $data);
 }
 
-$widget->addItem($dataTable);
+$widget->addItem($table);
 
-return $widget;
+$widget->addItem((new CScriptTag(
+	'$.subscribe("acknowledge.create", function(event, response, overlay) {'.
+		'postMessageOk(response.message);'.
+		'location.href = location.href;'.
+	'});'
+))->setOnDocumentReady());
+
+$widget->show();
