@@ -1,6 +1,6 @@
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -30,6 +30,10 @@ function getMenuPopupHistory(options) {
 	var items = [],
 		url = new Curl('history.php', false);
 
+	if (!options.allowed_ui_latest_data) {
+		return [];
+	}
+
 	url.setArgument('itemids[]', options.itemid);
 
 	// latest graphs
@@ -73,72 +77,69 @@ function getMenuPopupHistory(options) {
 /**
  * Get menu popup host section data.
  *
- * @param {string} options['hostid']             Host ID.
- * @param {array}  options['scripts']            Host scripts (optional).
- * @param {string} options[]['name']             Script name.
- * @param {string} options[]['scriptid']         Script ID.
- * @param {string} options[]['confirmation']     Confirmation text.
- * @param {bool}   options['showGraphs']         Link to host graphs page.
- * @param {bool}   options['showScreens']        Link to host screen page.
- * @param {bool}   options['showTriggers']       Link to Monitoring->Problems page.
- * @param {bool}   options['hasGoTo']            "Go to" block in popup.
- * @param {int}    options['severity_min']       (optional)
- * @param {bool}   options['show_suppressed']    (optional)
- * @param {array}  options['urls']               (optional)
+ * @param {string} options['hostid']                  Host ID.
+ * @param {array}  options['scripts']                 Host scripts (optional).
+ * @param {string} options[]['name']                  Script name.
+ * @param {string} options[]['scriptid']              Script ID.
+ * @param {string} options[]['confirmation']          Confirmation text.
+ * @param {bool}   options['showGraphs']              Link to Monitoring->Hosts->Graphs page.
+ * @param {bool}   options['showDashboards']          Link to Monitoring->Hosts->Dashboards page.
+ * @param {bool}   options['showWeb']		          Link to Monitoring->Hosts->Web page.
+ * @param {bool}   options['showTriggers']            Link to Monitoring->Problems page.
+ * @param {bool}   options['hasGoTo']                 "Go to" block in popup.
+ * @param {array}  options['severities']              (optional)
+ * @param {bool}   options['show_suppressed']         (optional)
+ * @param {array}  options['urls']                    (optional)
  * @param {string} options['url'][]['label']
  * @param {string} options['url'][]['url']
- * @param {string} options['filter_application'] (optional) Application name for filter by application.
- * @param {object} trigger_elmnt                 UI element which triggered opening of overlay dialogue.
+ * @param {string} options['filter_application']      (optional) Application name for filter by application.
+ * @param {bool}   options['allowed_ui_inventory']    Whether user has access to inventory hosts page.
+ * @param {bool}   options['allowed_ui_latest_data']  Whether user has access to latest data page.
+ * @param {bool}   options['allowed_ui_problems']     Whether user has access to problems page.
+ * @param {bool}   options['allowed_ui_hosts']        Whether user has access to monitoring hosts pages.
+ * @param {bool}   options['allowed_ui_conf_hosts']   Whether user has access to configuration hosts page.
+ * @param {object} trigger_elmnt                      UI element which triggered opening of overlay dialogue.
  *
  * @return array
  */
 function getMenuPopupHost(options, trigger_elmnt) {
 	var sections = [];
 
-	// scripts
-	if (typeof options.scripts !== 'undefined') {
-		sections.push({
-			label: t('Scripts'),
-			items: getMenuPopupScriptData(options.scripts, options.hostid, trigger_elmnt)
-		});
-	}
-
 	// go to section
 	if (options.hasGoTo) {
-		// inventory
 		var	host_inventory = {
-				label: t('Host inventory')
+				label: t('Inventory')
 			},
-			host_inventory_url = new Curl('hostinventories.php', false),
-			// latest
 			latest_data = {
 				label: t('Latest data')
 			},
-			latest_data_url = new Curl('latest.php', false),
-			// problems
 			problems = {
 				label: t('Problems')
 			},
-			// graphs
 			graphs = {
 				label: t('Graphs')
 			},
-			// screens
-			screens = {
-				label: t('Host screens')
+			dashboards = {
+				label: t('Dashboards')
+			},
+			web = {
+				label: t('Web')
 			};
 
 		// inventory link
-		host_inventory_url.setArgument('hostid', options.hostid);
-		host_inventory.url = host_inventory_url.getUrl();
+		var url = new Curl('hostinventories.php', false);
+		url.setArgument('hostid', options.hostid);
+		host_inventory.url = url.getUrl();
 
 		// latest data link
+		var url = new Curl('zabbix.php', false);
+		url.setArgument('action', 'latest.view');
 		if (typeof options.filter_application !== 'undefined') {
-			latest_data_url.setArgument('application', options.filter_application);
+			url.setArgument('filter_application', options.filter_application);
 		}
-		latest_data_url.setArgument('hostids[]', options.hostid);
-		latest_data_url.setArgument('filter_set', '1');
-		latest_data.url = latest_data_url.getUrl();
+		url.setArgument('filter_hostids[]', options.hostid);
+		url.setArgument('filter_set', '1');
+		latest_data.url = url.getUrl();
 
 		if (!options.showTriggers) {
 			problems.disabled = true;
@@ -146,17 +147,18 @@ function getMenuPopupHost(options, trigger_elmnt) {
 		else {
 			var url = new Curl('zabbix.php', false);
 			url.setArgument('action', 'problem.view');
-			url.setArgument('filter_hostids[]', options.hostid);
-			if (typeof options.severity_min !== 'undefined') {
-				url.setArgument('filter_severity', options.severity_min);
+			url.setArgument('filter_name', '');
+			url.setArgument('hostids[]', options.hostid);
+			if (typeof options.severities !== 'undefined') {
+				url.setArgument('severities[]', options.severities);
 			}
 			if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
-				url.setArgument('filter_show_suppressed', '1');
+				url.setArgument('show_suppressed', '1');
 			}
 			if (typeof options.filter_application !== 'undefined') {
-				url.setArgument('filter_application', options.filter_application);
+				url.setArgument('application', options.filter_application);
 			}
-			url.setArgument('filter_set', '1');
+
 			problems.url = url.getUrl();
 		}
 
@@ -164,32 +166,82 @@ function getMenuPopupHost(options, trigger_elmnt) {
 			graphs.disabled = true;
 		}
 		else {
-			var graphs_url = new Curl('charts.php', false);
+			var graphs_url = new Curl('zabbix.php', false);
 
-			graphs_url.setArgument('hostid', options.hostid);
+			graphs_url.setArgument('action', 'charts.view')
+			graphs_url.setArgument('view_as', 'showgraph'); // HISTORY_GRAPH
+			graphs_url.setArgument('filter_search_type', '0'); // ZBX_SEARCH_TYPE_STRICT
+			graphs_url.setArgument('filter_hostids[]', options.hostid);
+			graphs_url.setArgument('filter_set', '1');
 			graphs.url = graphs_url.getUrl();
 		}
 
-		if (!options.showScreens) {
-			screens.disabled = true;
+		if (!options.showDashboards) {
+			dashboards.disabled = true;
 		}
 		else {
-			var screens_url = new Curl('host_screen.php', false);
+			var dashboards_url = new Curl('zabbix.php', false);
 
-			screens_url.setArgument('hostid', options.hostid);
-			screens.url = screens_url.getUrl();
+			dashboards_url.setArgument('action', 'host.dashboard.view')
+			dashboards_url.setArgument('hostid', options.hostid)
+			dashboards.url = dashboards_url.getUrl();
 		}
 
-		sections.push({
-			label: t('Go to'),
-			items: [
-				host_inventory,
-				latest_data,
-				problems,
-				graphs,
-				screens
-			]
-		});
+		if (!options.showWeb) {
+			web.disabled = true;
+		}
+		else {
+			var web_url = new Curl('zabbix.php', false);
+			web_url.setArgument('action', 'web.view');
+			web_url.setArgument('filter_hostids[]', options.hostid);
+			web_url.setArgument('filter_set', '1');
+			web.url = web_url.getUrl();
+		}
+
+		var items = [];
+
+		if (options.allowed_ui_inventory) {
+			items.push(host_inventory);
+		}
+
+		if (options.allowed_ui_latest_data) {
+			items.push(latest_data);
+		}
+
+		if (options.allowed_ui_problems) {
+			items.push(problems);
+		}
+
+		if (options.allowed_ui_hosts) {
+			items.push(graphs);
+			items.push(dashboards);
+			items.push(web);
+		}
+
+		if (options.allowed_ui_conf_hosts) {
+			var config = {
+				label: t('Configuration')
+			};
+
+			if (options.isWriteable) {
+				var config_url = new Curl('hosts.php', false);
+				config_url.setArgument('form', 'update');
+				config_url.setArgument('hostid', options.hostid);
+				config.url = config_url.getUrl();
+			}
+			else {
+				config.disabled = true;
+			}
+
+			items.push(config);
+		}
+
+		if (items.length) {
+			sections.push({
+				label: t('Host'),
+				items: items
+			});
+		}
 	}
 
 	// urls
@@ -200,6 +252,14 @@ function getMenuPopupHost(options, trigger_elmnt) {
 		});
 	}
 
+	// scripts
+	if (typeof options.scripts !== 'undefined') {
+		sections.push({
+			label: t('Scripts'),
+			items: getMenuPopupScriptData(options.scripts, options.hostid, trigger_elmnt)
+		});
+	}
+
 	return sections;
 }
 
@@ -224,6 +284,10 @@ function getMenuPopupMapElementSubmap(options) {
 			', "' + options.widget_uniqueid + '");', false);
 	}
 	else {
+		if (!options.allowed_ui_maps) {
+			return [];
+		}
+
 		submap_url = new Curl('zabbix.php', false);
 		submap_url.setArgument('action', 'map.view');
 		submap_url.setArgument('sysmapid', options.sysmapid);
@@ -255,7 +319,7 @@ function getMenuPopupMapElementSubmap(options) {
  * Get menu popup host group map element section data.
  *
  * @param {string} options['groupid']
- * @param {int}    options['severity_min']       (optional)
+ * @param {array}  options['severities']         (optional)
  * @param {bool}   options['show_suppressed']    (optional)
  * @param {array}  options['urls']               (optional)
  * @param {string} options['url'][]['label']
@@ -265,21 +329,25 @@ function getMenuPopupMapElementSubmap(options) {
  * @return array
  */
 function getMenuPopupMapElementGroup(options) {
+	if (!options.allowed_ui_problems) {
+		return [];
+	}
+
 	var sections = [],
 		problems_url = new Curl('zabbix.php', false);
 
 	problems_url.setArgument('action', 'problem.view');
-	problems_url.setArgument('filter_groupids[]', options.groupid);
-	if (typeof options.severity_min !== 'undefined') {
-		problems_url.setArgument('severity_min', options.severity_min);
+	problems_url.setArgument('filter_name', '');
+	problems_url.setArgument('groupids[]', options.groupid);
+	if (typeof options.severities !== 'undefined') {
+		problems_url.setArgument('severities[]', options.severities);
 	}
 	if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
-		problems_url.setArgument('filter_show_suppressed', '1');
+		problems_url.setArgument('show_suppressed', '1');
 	}
 	if (typeof options.filter_application !== 'undefined') {
-		problems_url.setArgument('filter_application', options.filter_application);
+		problems_url.setArgument('application', options.filter_application);
 	}
-	problems_url.setArgument('filter_set', '1');
 
 	sections.push({
 		label: t('Go to'),
@@ -304,7 +372,7 @@ function getMenuPopupMapElementGroup(options) {
  * Get menu popup trigger map element section data.
  *
  * @param {array}  options['triggerids']
- * @param {int}    options['severity_min']     (optional)
+ * @param {array}  options['severities']       (optional)
  * @param {bool}   options['show_suppressed']  (optional)
  * @param {array}  options['urls']             (optional)
  * @param {string} options['url'][]['label']
@@ -313,18 +381,22 @@ function getMenuPopupMapElementGroup(options) {
  * @return array
  */
 function getMenuPopupMapElementTrigger(options) {
+	if (!options.allowed_ui_problems) {
+		return [];
+	}
+
 	var sections = [],
 		problems_url = new Curl('zabbix.php', false);
 
 	problems_url.setArgument('action', 'problem.view');
-	problems_url.setArgument('filter_triggerids[]', options.triggerids);
-	if (typeof options.severity_min !== 'undefined') {
-		problems_url.setArgument('filter_severity', options.severity_min);
+	problems_url.setArgument('filter_name', '');
+	problems_url.setArgument('triggerids[]', options.triggerids);
+	if (typeof options.severities !== 'undefined') {
+		problems_url.setArgument('severities[]', options.severities);
 	}
 	if (typeof options.show_suppressed !== 'undefined' && options.show_suppressed) {
-		problems_url.setArgument('filter_show_suppressed', '1');
+		problems_url.setArgument('show_suppressed', '1');
 	}
-	problems_url.setArgument('filter_set', '1');
 
 	sections.push({
 		label: t('Go to'),
@@ -369,6 +441,7 @@ function getMenuPopupMapElementImage(options) {
 /**
  * Get menu popup refresh section data.
  *
+ * @param {integer}  options['widgetid']	 Widget ID.
  * @param {string}   options['widgetName']   Widget name.
  * @param {string}   options['currentRate']  Current rate value.
  * @param {bool}     options['multiplier']   Multiplier or time mode.
@@ -409,8 +482,8 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 				value: value
 			},
 			clickCallback: function() {
-				var obj = jQuery(this),
-					currentRate = obj.data('value');
+				var $obj = jQuery(this),
+					currentRate = $obj.data('value');
 
 				// it is a quick solution for slide refresh multiplier, should be replaced with slide.refresh or similar
 				if (options.multiplier) {
@@ -420,10 +493,13 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 							widgetRefreshRate: currentRate
 						}),
 						dataType: 'script',
-						success: function(js) { js }
+						success: function() {
+							// Set new refresh rate as current in slideshow controls.
+							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
+						}
 					});
 
-					jQuery('a', obj.closest('.menu-popup')).each(function() {
+					jQuery('a', $obj.closest('.menu-popup')).each(function() {
 						var link = jQuery(this);
 
 						if (link.data('value') == currentRate) {
@@ -438,7 +514,7 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 						}
 					});
 
-					obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
+					$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 				}
 				else {
 					var url = new Curl('zabbix.php');
@@ -449,11 +525,12 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 						method: 'POST',
 						dataType: 'json',
 						data: {
-							'widgetid': options.widgetName,
+							'widgetid': options.widgetid,
 							'rf_rate': currentRate
-						},
-						success: function(resp) {
-							jQuery('a', obj.closest('.menu-popup')).each(function() {
+						}
+					})
+						.then(function() {
+							jQuery('a', $obj.closest('.menu-popup')).each(function() {
 								var link = jQuery(this);
 
 								if (link.data('value') == currentRate) {
@@ -468,16 +545,18 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 								}
 							});
 
-							obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
+							// Set new refresh rate as current in widget controls.
+							trigger_elmnt.data('menu-popup').data.currentRate = currentRate;
+
+							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 
 							jQuery('.dashbrd-grid-container')
-								.dashboardGrid('setWidgetRefreshRate', options.widgetName, parseInt(currentRate));
-						},
-						error: function() {
-							obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
+								.dashboardGrid('setWidgetRefreshRate', options.widgetid, parseInt(currentRate));
+						})
+						.fail(function() {
+							$obj.closest('.menu-popup').menuPopup('close', trigger_elmnt);
 							// TODO: gentle message about failed saving of widget refresh rate
-						}
-					});
+						});
 				}
 			}
 		};
@@ -495,6 +574,95 @@ function getMenuPopupRefresh(options, trigger_elmnt) {
 	}];
 }
 
+/**
+ * Get menu popup widget actions data.
+ *
+ * @param {string}   options['currentRate']      Current rate value.
+ * @param {bool}     options['multiplier']       Multiplier or time mode.
+ * @param {callback} options['callback']         Callback function on success (optional).
+ * @param {string}   options['widget_uniqueid']  Widget instance unique id.
+ * @param {object}   trigger_elmnt               UI element which triggered opening of overlay dialogue.
+ *
+ * @return array
+ */
+function getMenuPopupWidgetActions(options, trigger_elmnt) {
+	var $dashboard = jQuery('.dashbrd-grid-container'),
+		dashboard_data = $dashboard.dashboardGrid('getDashboardData'),
+		editMode = $dashboard.dashboardGrid('isEditMode'),
+		widget = $dashboard.dashboardGrid('getWidgetsBy', 'uniqueid', options.widget_uniqueid).pop(),
+		widgetid = widget.widgetid,
+		loading = (!widget['ready'] || widget['content_body'].find('.is-loading').length > 0),
+		widget_actions = [],
+		menu;
+
+	options.widgetid = widgetid;
+	menu = editMode ? [] : getMenuPopupRefresh(options, trigger_elmnt);
+
+	// Do not show "Copy" action for host dashboards.
+	if ($dashboard.data('dashboardGrid')['options']['allowed_edit']
+			&& (dashboard_data.templateid === null || dashboard_data.dynamic_hostid === null)) {
+		widget_actions.push({
+			label: t('S_COPY'),
+			clickCallback: function() {
+				jQuery('.dashbrd-grid-container').dashboardGrid('copyWidget', widget);
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			}
+		});
+	}
+
+	if (editMode) {
+		widget_actions.push({
+			label: t('S_PASTE'),
+			disabled: ($dashboard.dashboardGrid('getCopiedWidget') === null),
+			clickCallback: function() {
+				$dashboard.dashboardGrid('pasteWidget', widget, widget.pos);
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			}
+		});
+
+		widget_actions.push({
+			label: t('Delete'),
+			clickCallback: function() {
+				jQuery('.dashbrd-grid-container').dashboardGrid('deleteWidget', widget);
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			}
+		});
+	}
+
+	if ('download' in options && !editMode) {
+		widget_actions.push({
+			label: t('Download image'),
+			disabled: loading || !options.download,
+			clickCallback: function() {
+				var svg = widget['content_body'].find('svg').first();
+
+				if (svg.length) {
+					downloadSvgImage(svg, 'graph.png');
+				}
+				else {
+					downloadPngImage(widget['content_body'].find('img').first(), 'graph.png');
+				}
+
+				jQuery(this).closest('.menu-popup').menuPopup('close', trigger_elmnt);
+			},
+			refreshCallback: function(widget) {
+				if (widget['widgetid'] == widgetid && options.download) {
+					this.disabled = !widget['ready'];
+				}
+			}
+		});
+	}
+
+	if (widget_actions.length) {
+		menu.unshift({
+			label: t('Actions'),
+			items: widget_actions
+		});
+	}
+
+	return menu;
+}
+
 /**
  * Get menu popup trigger section data.
  *
@@ -524,10 +692,10 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
 			{
 				label: t('Sharing'),
 				clickCallback: function () {
+					jQuery(this).closest('.menu-popup').menuPopup('close', null);
+
 					var popup_options = {'dashboardid': options.dashboardid};
 					PopUp('dashboard.share.edit', popup_options, 'dashboard_share', trigger_elmnt);
-
-					jQuery(this).closest('.menu-popup').menuPopup('close', null);
 				},
 				disabled: !options.editable
 			},
@@ -541,12 +709,8 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
 			},
 			{
 				label: t('Delete'),
-				url: 'javascript:void(0)',
 				clickCallback: function () {
-					var	obj = jQuery(this);
-
-					// hide menu
-					obj.closest('.menu-popup').hide();
+					jQuery(this).closest('.menu-popup').menuPopup('close', null);
 
 					if (!confirm(t('Delete dashboard?'))) {
 						return false;
@@ -564,16 +728,13 @@ function getMenuPopupDashboard(options, trigger_elmnt) {
  * Get menu popup trigger section data.
  *
  * @param {string} options['triggerid']               Trigger ID.
- * @param {string} options['eventid']                 (optional) Required for Acknowledge and Description sections.
+ * @param {string} options['eventid']                 (optional) Required for Acknowledge section.
  * @param {object} options['items']                   Link to trigger item history page (optional).
  * @param {string} options['items'][]['name']         Item name.
  * @param {object} options['items'][]['params']       Item URL parameters ("name" => "value").
- * @param {object} options['acknowledge']             Link to acknowledge page (optional).
- * @param {string} options['acknowledge']['backurl']  Return URL.
+ * @param {bool}   options['acknowledge']             (optional) Whether to show Acknowledge section.
  * @param {object} options['configuration']           Link to trigger configuration page (optional).
  * @param {bool}   options['showEvents']              Show Problems item enabled. Default: false.
- * @param {bool}   options['show_description']        Show Description item in context menu. Default: true.
- * @param {bool}   options['description_enabled']     Show Description item enabled. Default: true.
  * @param {string} options['url']                     Trigger URL link (optional).
  * @param {object} trigger_elmnt                      UI element which triggered opening of overlay dialogue.
  *
@@ -583,66 +744,41 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 	var sections = [],
 		items = [];
 
-	// events
-	var events = {
-		label: t('Problems')
-	};
+	if (options.allowed_ui_problems) {
+		// events
+		var events = {
+			label: t('Problems')
+		};
 
 	if (typeof options.showEvents !== 'undefined' && options.showEvents) {
 		var url = new Curl('zabbix.php', false);
 		url.setArgument('action', 'problem.view');
-		url.setArgument('filter_triggerids[]', options.triggerid);
-		url.setArgument('filter_set', '1');
+		url.setArgument('filter_name', '');
+		url.setArgument('triggerids[]', options.triggerid);
 
-		events.url = url.getUrl();
-	}
-	else {
-		events.disabled = true;
-	}
-
-	items[items.length] = events;
-
-	// acknowledge
-	if (typeof options.acknowledge !== 'undefined' && objectSize(options.acknowledge) > 0) {
-		var url = new Curl('zabbix.php', false);
-
-		url.setArgument('action', 'acknowledge.edit');
-		url.setArgument('eventids[]', options.eventid);
-		url.setArgument('backurl', options.acknowledge.backurl);
-
-		items[items.length] = {
-			label: t('Acknowledge'),
-			url: url.getUrl()
-		};
-	}
-
-	// description
-	if (typeof options.show_description === 'undefined' || options.show_description !== false) {
-		var trigger_descr = {
-			label: t('Description')
-		};
-
-		if (typeof options.description_enabled === 'undefined' || options.description_enabled !== false) {
-			trigger_descr.clickCallback = function() {
-				var	popup_options = {triggerid: options.triggerid};
-
-				if (typeof options.eventid !== 'undefined') {
-					popup_options.eventid = options.eventid;
-				}
-
-				jQuery(this).closest('.menu-popup').menuPopup('close', null);
-
-				return PopUp('popup.trigdesc.view', popup_options, null, trigger_elmnt);
-			}
+			events.url = url.getUrl();
 		}
 		else {
-			trigger_descr.disabled = true;
+			events.disabled = true;
 		}
-		items[items.length] = trigger_descr;
+
+		items[items.length] = events;
+	}
+
+	// acknowledge
+	if (typeof options.acknowledge !== 'undefined' && options.acknowledge) {
+		items[items.length] = {
+			label: t('Acknowledge'),
+			clickCallback: function() {
+				jQuery(this).closest('.menu-popup-top').menuPopup('close', null);
+
+				acknowledgePopUp({eventids: [options.eventid]}, trigger_elmnt);
+			}
+		};
 	}
 
 	// configuration
-	if (typeof options.configuration !== 'undefined' && options.configuration) {
+	if (options.allowed_ui_conf_hosts) {
 		var url = new Curl('triggers.php', false);
 
 		url.setArgument('form', 'update');
@@ -654,21 +790,23 @@ function getMenuPopupTrigger(options, trigger_elmnt) {
 		};
 	}
 
-	// url
-	if (typeof options.url !== 'undefined' && options.url.length > 0) {
-		items[items.length] = {
-			label: t('URL'),
-			url: options.url
+	if (items.length) {
+		sections[sections.length] = {
+			label: t('S_TRIGGER'),
+			items: items
 		};
 	}
 
-	sections[sections.length] = {
-		label: t('S_TRIGGER'),
-		items: items
-	};
+	// urls
+	if ('urls' in options) {
+		sections[sections.length] = {
+			label: t('Links'),
+			items: options.urls
+		};
+	}
 
 	// items
-	if (typeof options.items !== 'undefined' && objectSize(options.items) > 0) {
+	if (options.allowed_ui_latest_data && typeof options.items !== 'undefined' && objectSize(options.items) > 0) {
 		var items = [];
 
 		jQuery.each(options.items, function(i, item) {
@@ -809,6 +947,106 @@ function getMenuPopupItemPrototype(options) {
 	}];
 }
 
+/**
+ * Get dropdown section data.
+ *
+ * @param {array}  options
+ * @param {object} trigger_elem  UI element that was clicked to open overlay dialogue.
+ *
+ * @returns array
+ */
+function getMenuPopupDropdown(options, trigger_elem) {
+	var items = [];
+
+	jQuery.each(options.items, function(i, item) {
+		var row = {
+			label: item.label,
+			url: item.url || 'javascript:void(0);'
+		};
+
+		if (item.class) {
+			row.class = item.class;
+		}
+
+		if (options.toggle_class) {
+			row.clickCallback = () => {
+				jQuery(trigger_elem)
+					.removeClass()
+					.addClass(['btn-alt', options.toggle_class, item.class].join(' '));
+
+				jQuery('input[type=hidden]', jQuery(trigger_elem).parent())
+					.val(item.value)
+					.trigger('change');
+			}
+		}
+		else if (options.submit_form) {
+			row.url = 'javascript:void(0);';
+			row.clickCallback = () => {
+				var $_form = trigger_elem.closest('form');
+
+				if (!$_form.data("action")) {
+					$_form.data("action", $_form.attr("action"));
+				}
+
+				$_form.attr("action", item.url);
+				$_form.submit();
+			}
+		}
+
+		items.push(row);
+	});
+
+	return [{
+		items: items
+	}];
+}
+
+/**
+ * Get menu popup submenu section data.
+ *
+ * @param object options['submenu']                                    List of menu sections.
+ * @param object options['submenu'][section]                           An individual section definition.
+ * @param string options['submenu'][section]['label']                  Non-clickable section label.
+ * @param object options['submenu'][section]['items']                  List of menu items of the section.
+ * @param string options['submenu'][section]['items'][url]             Menu item label for the given url.
+ * @param object options['submenu'][section]['items'][key]             Menu item with a submenu.
+ * @param object options['submenu'][section]['items'][key]['label']    Non-clickable subsection label.
+ * @param object options['submenu'][section]['items'][key]['items']    List of menu items of the subsection.
+ * @param object options['submenu'][section]['items'][key]['items'][]  More levels of submenu.
+ *
+ * @returns array
+ */
+function getMenuPopupSubmenu(options) {
+	var transform = function(sections) {
+			var result = [];
+
+			for (var key in sections) {
+				if (typeof sections[key] === 'object') {
+					var item = {};
+					for (var item_key in sections[key]) {
+						if (item_key === 'items') {
+							item[item_key] = transform(sections[key][item_key]);
+						}
+						else {
+							item[item_key] = sections[key][item_key];
+						}
+					}
+					result.push(item);
+				}
+				else {
+					result.push({
+						'label': sections[key],
+						'url': key
+					});
+				}
+			}
+
+			return result;
+		};
+
+	return transform(options.submenu);
+}
+
 /**
  * Get data for the "Insert expression" menu in the trigger expression constructor.
  *
@@ -926,6 +1164,37 @@ function getMenuPopupScriptData(scripts, hostId, trigger_elmnt) {
 	return getMenuPopupScriptItems(tree, trigger_elmnt);
 }
 
+/**
+ * Create menu for dashboard widget area selector.
+ *
+ * @param {object} area_selected  Area in which new widget will be created.
+ *
+ * @returns {array}
+ */
+function getDashboardWidgetActionMenu(area_selected) {
+	return [{
+		items: [{
+			label: t('Add widget'),
+			clickCallback: function() {
+				jQuery('.dashbrd-grid-container').dashboardGrid('addNewWidget', null, area_selected);
+			}
+		}, {
+			label: t('Paste widget'),
+			clickCallback: function() {
+				var widget_dims = {
+					x: area_selected.x,
+					y: area_selected.y,
+					width: area_selected.width,
+					height: area_selected.height
+				};
+
+				jQuery('.dashbrd-grid-container').dashboardGrid('pasteWidget', null, widget_dims);
+				jQuery('.dashbrd-grid-new-widget-placeholder').hide();
+			}
+		}]
+	}];
+}
+
 jQuery(function($) {
 
 	/**
@@ -935,6 +1204,9 @@ jQuery(function($) {
 	 * @param string sections[n]['label']  Section title (optional).
 	 * @param array  sections[n]['items']  Section menu data (see createMenuItem() for available options).
 	 * @param object event                 Menu popup call event.
+	 * @param object options               Menu popup options (optional).
+	 * @param object options['class']      Menu popup additional class name (optional).
+	 * @param object options['position']   Menu popup position object (optional).
 	 *
 	 * @see createMenuItem()
 	 */
@@ -947,157 +1219,163 @@ jQuery(function($) {
 		}
 	};
 
-	var methods = {
-		init: function(sections, event) {
-			var opener = $(this),
-				id = opener.data('menu-popup-id'),
-				menuPopup = $('#' + id),
-				mapContainer = null,
-				position_target = event.target;
-
-			if (event.type === 'contextmenu' || (IE && opener.closest('svg').length > 0)
-					|| event.originalEvent.detail !== 0) {
-				position_target = event;
+	/**
+	 * Created popup menu item nodes and append to $menu_popup.
+	 *
+	 * @param {object} $menu_popup    jQuery node of popup menu.
+	 * @param {array} sections        Array of menu popup sections.
+	 */
+	function addMenuPopupItems($menu_popup, sections) {
+		// Create menu sections.
+		$.each(sections, function(i, section) {
+			// Add a separator between menu item sections.
+			if (i > 0) {
+				$menu_popup.append($('<li>').append($('<div>')));
 			}
 
-			opener.attr('data-expanded', 'true');
+			var section_label = null;
 
-			// Close other action menus.
-			$('.menu-popup-top').not('#' + id).menuPopup('close');
+			if (typeof section.label === 'string' && section.label.length) {
+				section_label = section.label;
+			}
 
-			if (menuPopup.length > 0) {
-				var display = menuPopup.css('display');
+			// Add menu item section label, if provided.
+			if (section_label !== null) {
+				$menu_popup.append($('<li>').append($('<h3>').text(section_label)));
+			}
 
-				// Hide current action menu sub-levels.
-				$('.menu-popup', menuPopup).css('display', 'none');
-
-				if (display === 'block') {
-					menuPopup.fadeOut(0);
-					$(opener).removeAttr('data-expanded');
-				}
-				else {
-					menuPopup.fadeIn(50);
+			// Add individual menu items of the section.
+			$.each(section.items, function(i, item) {
+				item = $.extend({}, item);
+				if (sections.length > 1 && section_label !== null) {
+					item.ariaLabel = section_label + ', ' + item['label'];
 				}
+				$menu_popup.append(createMenuItem(item));
+			});
+		});
 
-				menuPopup.position({
-					of: position_target,
+		if (sections.length == 1) {
+			if (typeof sections[0].label === 'string' && sections[0].label.length) {
+				$menu_popup.attr({'aria-label': sections[0].label});
+			}
+		}
+	}
+
+	var defaultOptions = {
+		closeCallback: function(){}
+	};
+
+	var methods = {
+		init: function(sections, event, options) {
+			// Don't display empty menu.
+			if (!sections.length || !sections[0]['items'].length) {
+				return;
+			}
+
+			var $opener = $(this);
+
+			options = $.extend({
+				position: {
+					/*
+					 * Please note that click event is also triggered by hitting spacebar on the keyboard,
+					 * in which case the number of mouse clicks (stored in event.originalEvent.detail) will be zero.
+					 */
+					of: (event.type === 'click' && event.originalEvent.detail) ? event : event.target,
 					my: 'left top',
 					at: 'left bottom'
-				});
-			}
-			else {
-				id = new Date().getTime();
+				}
+			}, defaultOptions, options || {});
 
-				menuPopup = $('<ul>', {
-					'id': id,
+			// Close other action menus and prevent focus jumping before opening a new popup.
+			$('.menu-popup-top').menuPopup('close', null, false);
+
+			$opener.attr('aria-expanded', 'true');
+
+			var $menu_popup = $('<ul>', {
 					'role': 'menu',
 					'class': 'menu-popup menu-popup-top',
 					'tabindex': 0
 				});
 
-				// create sections
-				var sections_length = sections.length;
-				if (sections_length) {
-					$.each(sections, function(i, section) {
-						if ((typeof section.label !== 'undefined') && (section.label.length > 0)) {
-							var h3 = $('<h3>').text(section.label);
-							var sectionItem = $('<li>').append(h3);
-						}
-
-						// Add section delimited for all sections except first one.
-						if (i > 0) {
-							menuPopup.append($('<li>').append($('<div>')));
-						}
-						menuPopup.append(sectionItem);
-
-						$.each(section.items, function(i, item) {
-							if (sections_length > 1) {
-								item['ariaLabel'] = section.label + ', ' + item['label'];
-							}
-							menuPopup.append(createMenuItem(item));
-						});
-					});
-				}
-
-				if (sections_length == 1) {
-					menuPopup.attr({'aria-label': sections[0].label});
-				}
-
-				// Skip displaying empty menu sections.
-				if (menuPopup.children().length == 0) {
-					return;
-				}
-
-				// Set menu popup for map area.
-				if (opener.prop('tagName') === 'AREA') {
-					$('.menuPopupContainer').remove();
-
-					mapContainer = $('<div>', {
-						'class': 'menuPopupContainer',
-						'css': {
-							position: 'absolute',
-							top: event.pageY,
-							left: event.pageX
-						}
-					})
-					.append(menuPopup);
-
-					$('body').append(mapContainer);
-				}
-				// Set menu popup for common html elements.
-				else {
-					opener.data('menu-popup-id', id);
-
-					$('body').append(menuPopup);
-				}
-
-				// Hide current action menu sub-levels.
-				$('.menu-popup', menuPopup).css('display', 'none');
-
-				// display
-				menuPopup
-					.fadeIn(50)
-					.data('is-active', false)
-					.mouseenter(function() {
-						menuPopup.data('is-active', true);
-
-						clearTimeout(window.menuPopupTimeoutHandler);
-					})
-					.on('click', function(e) {
-						e.stopPropagation();
-					})
-					.position({
-						of: (opener.prop('tagName') === 'AREA') ? mapContainer : position_target,
-						my: 'left top',
-						at: 'left bottom'
-					});
+			// Add custom class, if specified.
+			if ('class' in options) {
+				$menu_popup.addClass(options.class);
 			}
 
+			$opener.data({
+				sections: sections,
+				menu_popup: $menu_popup
+			});
+			addMenuPopupItems($menu_popup, sections);
+
+			$menu_popup.data('menu_popup', options);
+
+			$('.wrapper').append($menu_popup);
+
+			// Position the menu (before hiding).
+			$menu_popup.position(options.position);
+
+			// Hide all action menu sub-levels, including the topmost, for fade effect to work.
+			$menu_popup.add('.menu-popup', $menu_popup).hide();
+
+			// Position and display the menu.
+			$menu_popup.fadeIn(50);
+
 			addToOverlaysStack('menu-popup', event.target, 'menu-popup');
 
-			$(document)
-				.on('click', {menu: menuPopup, opener: opener}, menuPopupDocumentCloseHandler)
-				.on('keydown', {menu: menuPopup}, menuPopupKeyDownHandler);
+			// Need to be postponed.
+			setTimeout(function() {
+				$(document)
+					.on('click', {menu: $menu_popup, opener: $opener}, menuPopupDocumentCloseHandler)
+					.on('keydown', {menu: $menu_popup}, menuPopupKeyDownHandler);
+			});
 
-			menuPopup.focus();
+			$menu_popup.focus();
 		},
-		close: function(trigger_elmnt, return_focus) {
-			var menuPopup = $(this);
-			if (!menuPopup.is(trigger_elmnt) && menuPopup.has(trigger_elmnt).length === 0) {
-				menuPopup.data('is-active', false);
-				$(trigger_elmnt).removeAttr('data-expanded');
-				menuPopup.fadeOut(0);
 
-				$('.highlighted', menuPopup).removeClass('highlighted');
-				$('[aria-expanded="true"]', menuPopup).attr({'aria-expanded': 'false'});
+		close: function(trigger_elem, return_focus) {
+			var menu_popup = $(this),
+				options = $(menu_popup).data('menu_popup') || {};
+
+			if (!menu_popup.is(trigger_elem) && menu_popup.has(trigger_elem).length === 0) {
+				$('[aria-expanded="true"]', trigger_elem).attr({'aria-expanded': 'false'});
+				menu_popup.fadeOut(0);
+
+				$('.highlighted', menu_popup).removeClass('highlighted');
+				$('[aria-expanded="true"]', menu_popup).attr({'aria-expanded': 'false'});
 
 				$(document)
 					.off('click', menuPopupDocumentCloseHandler)
 					.off('keydown', menuPopupKeyDownHandler);
 
-				removeFromOverlaysStack('menu-popup', return_focus);
-				menuPopup.remove();
+				var overlay = removeFromOverlaysStack('menu-popup', return_focus);
+
+				if (overlay && typeof overlay['element'] !== undefined) {
+					// Remove expanded attribute of the original opener.
+					$(overlay['element']).attr({'aria-expanded': 'false'});
+				}
+
+				menu_popup.remove();
+
+				// Call menu close callback function.
+				typeof options.closeCallback === 'function' && options.closeCallback.apply();
 			}
+		},
+
+		/**
+		 * Refresh popup menu, call refreshCallback for every item if defined. Refresh recreate item dom nodes.
+		 */
+		refresh: function(widget) {
+			var $opener = $(this),
+				sections = $opener.data('sections'),
+				$menu_popup = $opener.data('menu_popup');
+
+			$menu_popup.empty();
+			sections.forEach(
+				section => section.items && section.items.forEach(
+					item => item.refreshCallback && item.refreshCallback.call(item, widget)
+			));
+			addMenuPopupItems($menu_popup, sections);
 		}
 	};
 
@@ -1273,6 +1551,7 @@ jQuery(function($) {
 	 * @param string options['css']            Item class.
 	 * @param array  options['data']           Item data ("key" => "value").
 	 * @param array  options['items']          Item sub menu.
+	 * @param {bool} options['disabled']       Item disable status.
 	 * @param object options['clickCallback']  Item click callback.
 	 *
 	 * @return object
@@ -1281,7 +1560,8 @@ jQuery(function($) {
 		options = $.extend({
 			ariaLabel: options.label,
 			selected: false,
-			disabled: false
+			disabled: false,
+			class: false
 		}, options);
 
 		var item = $('<li>'),
@@ -1314,6 +1594,10 @@ jQuery(function($) {
 
 			if (typeof options.url !== 'undefined') {
 				link.attr('href', options.url);
+
+				if ('target' in options) {
+					link.attr('target', options.target);
+				}
 			}
 
 			if (typeof options.clickCallback !== 'undefined') {
@@ -1325,12 +1609,25 @@ jQuery(function($) {
 			link.addClass('selected');
 		}
 
+		if (options.class) {
+			link.addClass(options.class);
+		}
+
+		if ('dataAttributes' in options) {
+			$.each(options.dataAttributes, function(key, value) {
+				link.attr((key.substr(0, 5) === 'data-') ? key : 'data-' + key, value);
+			});
+		}
+
 		if (typeof options.items !== 'undefined' && options.items.length > 0) {
 			link.attr({
 				'aria-haspopup': 'true',
 				'aria-expanded': 'false',
 				'area-hidden': 'true'
 			});
+			link.on('click', function(e) {
+				e.stopPropagation();
+			});
 		}
 
 		item.append(link);
