@@ -1,6 +1,6 @@
 /*
  ** Zabbix
- ** Copyright (C) 2001-2019 Zabbix SIA
+ ** Copyright (C) 2001-2020 Zabbix SIA
  **
  ** This program is free software; you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -46,7 +46,9 @@ jQuery(function($) {
 			this.prepareNext(timeout);
 
 			this.$elem = $elem;
+			this.updateWidth();
 			this.$elem.on('mouseenter', this.hideMessage.bind(this));
+			$(window).on('resize', this.updateWidth.bind(this));
 		},
 
 		prepareNext: function(delay) {
@@ -101,6 +103,14 @@ jQuery(function($) {
 
 				this.$elem.fadeOut(200);
 			}
+		},
+
+		updateWidth: function() {
+			let $wrapper = $('.wrapper');
+			this.$elem.css({
+				left: $wrapper.offset().left + 10,
+				width: $wrapper[0].clientWidth - 20
+			});
 		}
 	};
 
