@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -64,19 +64,23 @@ $availableJScripts = [
 	'csvggraphwidget.js' => '',
 	'layout.mode.js' => '',
 	'textareaflexible.js' => '',
+	'inputsecret.js' => '',
+	'macrovalue.js' => '',
 	// vendors
-	'prototype.js' => 'vendors/',
 	'jquery.js' => 'vendors/',
 	'jquery-ui.js' => 'vendors/',
 	// classes
+	'component.z-select.js' => '',
+	'class.base-component.js' => '',
 	'class.bbcode.js' => '',
 	'class.calendar.js' => '',
 	'class.cclock.js' => '',
 	'class.cdate.js' => '',
 	'class.cdebug.js' => '',
 	'class.cmap.js' => '',
-	'class.promise.js' => '',
 	'class.localstorage.js' => '',
+	'class.menu.js' => '',
+	'class.menu-item.js' => '',
 	'class.notifications.js' => '',
 	'class.notification.js' => '',
 	'class.notification.collection.js' => '',
@@ -92,40 +96,53 @@ $availableJScripts = [
 	'class.ctree.js' => '',
 	'class.curl.js' => '',
 	'class.overlaycollection.js' => '',
+	'class.overlay.js' => '',
 	'class.cverticalaccordion.js' => '',
-	'class.mapwidget.js' => '',
+	'class.mapWidget.js' => '',
+	'class.scrollable.js' => '',
+	'class.sidebar.js' => '',
 	'class.svg.canvas.js' => 'vector/',
 	'class.svg.map.js' => 'vector/',
 	'class.cviewswitcher.js' => '',
 	'class.pmaster.js' => '',
 	'class.rpc.js' => '',
+	'class.tabfilter.js' => '',
+	'class.tabfilteritem.js' => '',
+	'class.template.js' => '',
 	'init.js' => '',
+	'class.tab-indicators.js' => '',
 	// templates
 	'sysmap.tpl.js' => 'templates/',
 	// page-specific scripts
 	'items.js' => 'pages/',
+	'setup.js' => 'pages/',
+	'popup.condition.common.js' => 'pages/',
+	'popup.operation.common.js' => 'pages/'
 ];
 
 $tranStrings = [
 	'gtlc.js' => [
 		'S_MINUTE_SHORT' => _x('m', 'minute short')
 	],
+	'class.overlay.js' => [
+		'Operation details' => _('Operation details')
+	],
 	'dashboard.grid.js' => [
 		'Edit widget' => _('Edit widget'),
 		'Add widget' => _('Add widget'),
+		'Paste widget' => _('Paste widget'),
+		'S_COPY' => _('Copy'),
+		'S_PASTE' => _('Paste'),
 		'Apply' => _('Apply'),
 		'Add' => _('Add'),
 		'Edit' => _('Edit'),
 		'Cancel' => _('Cancel'),
 		'Delete' => _('Delete'),
-		'You have unsaved changes.' => _('You have unsaved changes.'),
-		'Are you sure, you want to leave this page?' => _('Are you sure, you want to leave this page?'),
 		'Cannot add widgets in kiosk mode' => _('Cannot add widgets in kiosk mode'),
 		'You do not have permissions to edit dashboard' => _('You do not have permissions to edit dashboard'),
 		'Add a new widget' => _('Add a new widget'),
-		'Release to create a new widget.' => _('Release to create a new widget.'),
+		'Release to create a widget.' => _('Release to create a widget.'),
 		'Click and drag to desired size.' => _('Click and drag to desired size.'),
-		'Adjust widget refresh interval' => _('Adjust widget refresh interval'),
 		'Previous page' => _('Previous page'),
 		'Next page' => _('Next page'),
 		'Widget is too small for the specified number of columns and rows.' =>
@@ -212,8 +229,9 @@ $tranStrings = [
 		'S_PROBLEM_ON' => _('Problem on'),
 		'S_RESOLVED' => _('Resolved'),
 		'S_MUTE' => _('Mute'),
+		'S_CANNOT_SUPPORT_NOTIFICATION_AUDIO' => _('Cannot support notification audio for this device.'),
 		'S_UNMUTE' => _('Unmute'),
-		'S_CLEAR' => _('Clear'),
+		'S_CLOSE' => _('Close'),
 		'S_SNOOZE' => _('Snooze')
 	],
 	'class.cookie.js' => [
@@ -274,8 +292,9 @@ $tranStrings = [
 		'Go to' => _('Go to'),
 		'Graphs' => _('Graphs'),
 		'History' => _('History'),
-		'Host inventory' => _('Host inventory'),
-		'Host screens' => _('Host screens'),
+		'Host' => _('Host'),
+		'Inventory' => _('Inventory'),
+		'Screens' => _('Screens'),
 		'Latest data' => _('Latest data'),
 		'Latest values' => _('Latest values'),
 		'Last hour graph' => _('Last hour graph'),
@@ -289,6 +308,7 @@ $tranStrings = [
 		'S_TRIGGER' => _('Trigger'),
 		'URL' => _('URL'),
 		'URLs' => _('URLs'),
+		'Web' => _('Web'),
 		'No refresh' => _('No refresh'),
 		'10 seconds' => _n('%1$s second', '%1$s seconds', 10),
 		'30 seconds' => _n('%1$s second', '%1$s seconds', 30),
@@ -296,11 +316,17 @@ $tranStrings = [
 		'2 minutes' => _n('%1$s minute', '%1$s minutes', 2),
 		'10 minutes' => _n('%1$s minute', '%1$s minutes', 10),
 		'15 minutes' => _n('%1$s minute', '%1$s minutes', 15),
-		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader')
+		'S_SELECTED_SR' => _x('%1$s, selected', 'screen reader'),
+		'Download image' => _('Download image')
+	],
+	'init.js' => [
+		'Debug' => _('Debug'),
+		'Hide debug' => _('Hide debug')
 	],
 	'items.js' => [
 		'To set a host interface select a single item type for all items' => _('To set a host interface select a single item type for all items'),
-		'No interface found' => _('No interface found')
+		'No interface found' => _('No interface found'),
+		'Item type does not use interface' => _('Item type does not use interface')
 	],
 	'class.cnavtree.js' => [
 		'Edit' => _('Edit'),
@@ -323,24 +349,39 @@ $tranStrings = [
 	'common.js' => [
 		'Cancel' => _('Cancel')
 	],
+	'component.z-select.js' => [
+		'All' => _('All')
+	],
+	'macrovalue.js' => [
+		'Set new value' => _('Set new value'),
+		'path/to/secret:key' => _('path/to/secret:key'),
+		'value' => _('value')
+	]
 ];
 
 $js = '';
 if (empty($_GET['files'])) {
 
 	$files = [
-		'prototype.js',
 		'jquery.js',
 		'jquery-ui.js',
 		'common.js',
+		'component.z-select.js',
+		'class.base-component.js',
 		'class.cdebug.js',
 		'class.overlaycollection.js',
+		'class.overlay.js',
 		'class.cdate.js',
 		'class.cookie.js',
 		'class.curl.js',
+		'class.menu.js',
+		'class.menu-item.js',
 		'class.rpc.js',
 		'class.bbcode.js',
 		'class.csuggest.js',
+		'class.scrollable.js',
+		'class.sidebar.js',
+		'class.template.js',
 		'main.js',
 		'chkbxrange.js',
 		'functions.js',
@@ -348,16 +389,17 @@ if (empty($_GET['files'])) {
 		'init.js'
 	];
 
+	require_once dirname(__FILE__).'/include/defines.inc.php';
+	require_once dirname(__FILE__).'/include/classes/helpers/CCookieHelper.php';
+
+	if (CCookieHelper::has(ZBX_SESSION_NAME)) {
+		$session = json_decode(base64_decode(CCookieHelper::get(ZBX_SESSION_NAME)), true);
+		$js .= 'window.ZBX_SESSION_NAME = "'.crc32($session['sessionid']).'";';
+		$files[] = 'class.localstorage.js';
+	}
+
 	// load frontend messaging only for some pages
 	if (isset($_GET['showGuiMessaging']) && $_GET['showGuiMessaging']) {
-		require_once dirname(__FILE__).'/include/defines.inc.php';
-
-		if (array_key_exists(ZBX_SESSION_NAME, $_COOKIE)) {
-			$js .= 'window.ZBX_SESSION_NAME = "'.crc32($_COOKIE[ZBX_SESSION_NAME]).'";';
-		}
-
-		$files[] = 'class.promise.js';
-		$files[] = 'class.localstorage.js';
 		$files[] = 'class.browsertab.js';
 		$files[] = 'class.notification.collection.js';
 		$files[] = 'class.notifications.audio.js';
@@ -384,19 +426,6 @@ foreach ($files as $file) {
 	}
 }
 
-if (in_array('prototype.js', $files)) {
-	// This takes care of the Array toJSON incompatibility with JSON.stringify.
-	$js .=
-		'var _json_stringify = JSON.stringify;'.
-		'JSON.stringify = function(value) {'.
-			'var _array_tojson = Array.prototype.toJSON,'.
-				'ret;'.
-			'delete Array.prototype.toJSON;'.
-			'ret = _json_stringify(value);'.
-			'Array.prototype.toJSON = _array_tojson;'.
-			'return ret;'.
-		'};';
-}
 
 $etag = md5($js);
 /**
