@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -27,12 +27,10 @@ require_once dirname(__FILE__).'/include/blocks.inc.php';
 $page['title'] = _('Custom slides');
 $page['file'] = 'slides.php';
 $page['scripts'] = ['class.svg.canvas.js', 'class.svg.map.js', 'class.pmaster.js', 'class.calendar.js', 'gtlc.js',
-	'flickerfreescreen.js', 'layout.mode.js'
+	'multiselect.js', 'flickerfreescreen.js', 'layout.mode.js'
 ];
 $page['type'] = detect_page_type(PAGE_TYPE_HTML);
-
-CView::$has_web_layout_mode = true;
-$page['web_layout_mode'] = CView::getLayoutMode();
+$page['web_layout_mode'] = CViewHelper::loadLayoutMode();
 
 define('ZBX_PAGE_DO_JS_REFRESH', 1);
 
@@ -41,7 +39,6 @@ require_once dirname(__FILE__).'/include/page_header.php';
 
 // VAR	TYPE	OPTIONAL	FLAGS	VALIDATION	EXCEPTION
 $fields = [
-	'groupid' =>			[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,	null],
 	'hostid' =>				[T_ZBX_INT,			O_OPT, P_SYS,	DB_ID,	null],
 	'elementid' =>			[T_ZBX_INT,			O_OPT, P_SYS|P_NZERO, DB_ID, null],
 	'step' =>				[T_ZBX_INT,			O_OPT, P_SYS,	BETWEEN(0, 65535), null],
@@ -59,11 +56,8 @@ validateTimeSelectorPeriod(getRequest('from'), getRequest('to'));
 /*
  * Permissions
  */
-$data = [];
+$data = ['allowed_edit' => CWebUser::checkAccess(CRoleHelper::ACTIONS_EDIT_DASHBOARDS)];
 
-if (getRequest('groupid') && !isReadableHostGroups([getRequest('groupid')])) {
-	access_deny();
-}
 if (getRequest('hostid') && !isReadableHosts([getRequest('hostid')])) {
 	access_deny();
 }
@@ -78,10 +72,21 @@ if (!$data['screen']) {
 	else {
 		// Redirect to slide show list.
 		ob_end_clean();
-		redirect('slideconf.php');
+		redirect((new CUrl('slideconf.php'))
+			->setArgument('page', CPagerHelper::loadPage('slideconf.php', null))
+			->getUrl()
+		);
 	}
 }
 
+// Save dynamic item hostid.
+if (hasRequest('hostid')) {
+	CProfile::update('web.slides.hostid', getRequest('hostid'), PROFILE_TYPE_ID);
+}
+elseif (hasRequest('reset')) {
+	CProfile::delete('web.slides.hostid');
+}
+
 CProfile::update('web.slides.elementid', $data['elementId'], PROFILE_TYPE_ID);
 
 /*
@@ -143,7 +148,7 @@ if (hasRequest('widgetRefresh') || hasRequest('widgetRefreshRate')) {
 		$delay = timeUnitToSeconds(($screen['delay'] === '0') ? $data['screen']['delay'] : $screen['delay']);
 
 		$js = 'PMasters["slideshows"].dolls["'.WIDGET_SLIDESHOW.'"].frequency('.
-				CJs::encodeJson($delay * $widgetRefreshRate).
+				json_encode($delay * $widgetRefreshRate).
 			');'.
 			"\n".
 			'PMasters["slideshows"].dolls["'.WIDGET_SLIDESHOW.'"].restartDoll();';
@@ -176,26 +181,22 @@ updateTimeSelectorPeriod($timeselector_options);
 $data['timeline'] = getTimeSelectorPeriod($timeselector_options);
 $data['active_tab'] = CProfile::get('web.slides.filter.active', 1);
 
-// get groups and hosts
+// Dynamic item host selector.
 if (check_dynamic_items($data['elementId'], 1)) {
-	$data['isDynamicItems'] = true;
+	$data['has_dynamic_widgets'] = true;
+	$hostid = getRequest('hostid', CProfile::get('web.slides.hostid', 0));
+	$data['host'] = ($hostid > 0)
+		? CArrayHelper::renameObjectsKeys(API::Host()->get([
+			'output' => ['hostid', 'name'],
+			'hostids' => [$hostid],
+			'monitored_hosts' => 1
+		]), ['hostid' => 'id'])
+		: [];
 
-	$data['pageFilter'] = new CPageFilter([
-		'groups' => [
-			'monitored_hosts' => true,
-			'with_items' => true
-		],
-		'hosts' => [
-			'monitored_hosts' => true,
-			'with_items' => true,
-			'DDFirstLabel' => _('not selected')
-		],
-		'hostid' => getRequest('hostid'),
-		'groupid' => getRequest('groupid')
-	]);
-
-	$data['groupid'] = $data['pageFilter']->groupid;
-	$data['hostid'] = $data['pageFilter']->hostid;
+	$data['hostid'] = $data['host'] ? $data['host'][0]['id'] : 0;
+}
+else {
+	$data['has_dynamic_widgets'] = false;
 }
 
 // get element
@@ -212,8 +213,6 @@ show_messages();
 $data['refreshMultiplier'] = CProfile::get('web.slides.rf_rate.'.WIDGET_SLIDESHOW, 1, $data['elementId']);
 
 // render view
-$slidesView = new CView('monitoring.slides', $data);
-$slidesView->render();
-$slidesView->show();
+echo (new CView('monitoring.slides', $data))->getOutput();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
