@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -28,11 +28,9 @@ require_once dirname(__FILE__).'/include/html.inc.php';
 
 $page['title'] = _('Event details');
 $page['file'] = 'tr_events.php';
-$page['type'] = detect_page_type(PAGE_TYPE_HTML);
+$page['type'] = detect_page_type();
 $page['scripts'] = ['layout.mode.js'];
-
-CView::$has_web_layout_mode = true;
-$page['web_layout_mode'] = CView::getLayoutMode();
+$page['web_layout_mode'] = CViewHelper::loadLayoutMode();
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -87,6 +85,19 @@ if (!$events) {
 }
 $event = reset($events);
 
+$event['comments'] = ($trigger['comments'] !== '')
+	? CMacrosResolverHelper::resolveTriggerDescription(
+		[
+			'triggerid' => $trigger['triggerid'],
+			'expression' => $trigger['expression'],
+			'comments' => $trigger['comments'],
+			'clock' => $event['clock'],
+			'ns' => $event['ns']
+		],
+		['events' => true]
+	)
+	: '';
+
 if ($event['r_eventid'] != 0) {
 	$r_events = API::Event()->get([
 		'output' => ['correlationid', 'userid'],
@@ -104,15 +115,32 @@ if ($event['r_eventid'] != 0) {
 	}
 }
 
-$config = select_config();
-$severity_config = [
-	'severity_name_0' => $config['severity_name_0'],
-	'severity_name_1' => $config['severity_name_1'],
-	'severity_name_2' => $config['severity_name_2'],
-	'severity_name_3' => $config['severity_name_3'],
-	'severity_name_4' => $config['severity_name_4'],
-	'severity_name_5' => $config['severity_name_5']
-];
+if ($trigger['opdata'] !== '') {
+	$event['opdata'] = (new CCol(CMacrosResolverHelper::resolveTriggerOpdata(
+		[
+			'triggerid' => $trigger['triggerid'],
+			'expression' => $trigger['expression'],
+			'opdata' => $trigger['opdata'],
+			'clock' => $event['clock'],
+			'ns' => $event['ns']
+		],
+		[
+			'events' => true,
+			'html' => true
+		]
+	)))
+		->addClass('opdata')
+		->addClass(ZBX_STYLE_WORDWRAP);
+}
+else {
+	$db_items = API::Item()->get([
+		'output' => ['itemid', 'hostid', 'name', 'key_', 'value_type', 'units', 'valuemapid'],
+		'triggerids' => $event['objectid']
+	]);
+	$db_items = CMacrosResolverHelper::resolveItemNames($db_items);
+	$event['opdata'] = (new CCol(CScreenProblem::getLatestValues($db_items)))->addClass('latest-values');
+}
+
 $actions = getEventDetailsActions($event);
 $users = API::User()->get([
 	'output' => ['alias', 'name', 'surname'],
@@ -125,45 +153,60 @@ $mediatypes = API::Mediatype()->get([
 	'preservekeys' => true
 ]);
 
+$allowed = [
+	'ui_correlation' => CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_EVENT_CORRELATION),
+	'add_comments' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ADD_PROBLEM_COMMENTS),
+	'change_severity' => CWebUser::checkAccess(CRoleHelper::ACTIONS_CHANGE_SEVERITY),
+	'acknowledge' => CWebUser::checkAccess(CRoleHelper::ACTIONS_ACKNOWLEDGE_PROBLEMS),
+	'close' => ($trigger['manual_close'] == ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED
+			&& CWebUser::checkAccess(CRoleHelper::ACTIONS_CLOSE_PROBLEMS)
+	)
+];
+
 /*
  * Display
  */
-$eventTab = (new CTable())
-	->addRow([
-		new CDiv([
-			(new CUiWidget(WIDGET_HAT_TRIGGERDETAILS, make_trigger_details($trigger)))->setHeader(_('Trigger details')),
-			(new CUiWidget(WIDGET_HAT_EVENTDETAILS,
-				make_event_details($event,
-					$page['file'].'?triggerid='.getRequest('triggerid').'&eventid='.getRequest('eventid')
-				)
-			))->setHeader(_('Event details'))
-		]),
-		new CDiv([
-			(new CCollapsibleUiWidget(WIDGET_HAT_EVENTACTIONS,
-				makeEventDetailsActionsTable($actions, $users, $mediatypes, $severity_config)
-			))
-				->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state', true))
-				->setHeader(_('Actions'), [], false, 'tr_events.php'),
-			(new CCollapsibleUiWidget(WIDGET_HAT_EVENTLIST,
-				make_small_eventlist($event,
-					$page['file'].'?triggerid='.getRequest('triggerid').'&eventid='.getRequest('eventid')
-				)
-			))
-				->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state', true))
-				->setHeader(_('Event list [previous 20]'), [], false, 'tr_events.php')
-		])
-	]);
+$event_tab = (new CDiv([
+	new CDiv([
+		(new CUiWidget(WIDGET_HAT_TRIGGERDETAILS, make_trigger_details($trigger, $event['eventid'])))
+			->setHeader(_('Trigger details')),
+		(new CUiWidget(WIDGET_HAT_EVENTDETAILS, make_event_details($event, $allowed)))
+			->setHeader(_('Event details'))
+	]),
+	new CDiv([
+		(new CCollapsibleUiWidget(WIDGET_HAT_EVENTACTIONS,
+			makeEventDetailsActionsTable($actions, $users, $mediatypes)
+		))
+			->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state', true))
+			->setHeader(_('Actions'), [], 'web.tr_events.hats.'.WIDGET_HAT_EVENTACTIONS.'.state')
+			->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID),
+		(new CCollapsibleUiWidget(WIDGET_HAT_EVENTLIST, make_small_eventlist($event, $allowed)))
+			->setExpanded((bool) CProfile::get('web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state', true))
+			->setHeader(_('Event list [previous 20]'), [], 'web.tr_events.hats.'.WIDGET_HAT_EVENTLIST.'.state')
+			->addClass(ZBX_STYLE_DASHBRD_WIDGET_FLUID)
+	])
+]))
+	->addClass(ZBX_STYLE_COLUMNS)
+	->addClass(ZBX_STYLE_COLUMNS_2);
 
-$eventWidget = (new CWidget())
+$script = (new CScriptTag(
+	'$.subscribe("acknowledge.create", function(event, response, overlay) {'.
+		'postMessageOk(response.message);'.
+		'location.href = location.href;'.
+	'});'
+))->setOnDocumentReady();
+
+(new CWidget())
 	->setTitle(_('Event details'))
 	->setWebLayoutMode($page['web_layout_mode'])
 	->setControls((new CTag('nav', true,
 		(new CList())
-			->addItem(get_icon('fullscreen'))
+			->addItem(get_icon('kioskmode', ['mode' => $page['web_layout_mode']]))
 		))
 		->setAttribute('aria-label', _('Content controls'))
 	)
-	->addItem($eventTab)
+	->addItem($event_tab)
+	->addItem($script)
 	->show();
 
 require_once dirname(__FILE__).'/include/page_footer.php';
