@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@ require_once dirname(__FILE__).'/include/forms.inc.php';
 
 $page['title'] = _('Configuration of trigger prototypes');
 $page['file'] = 'trigger_prototypes.php';
+$page['scripts'] = ['multiselect.js', 'textareaflexible.js', 'class.tab-indicators.js'];
 
 require_once dirname(__FILE__).'/include/page_header.php';
 
@@ -35,6 +36,8 @@ $fields = [
 	'triggerid' =>								[T_ZBX_INT, O_OPT, P_SYS,	DB_ID,		'(isset({form}) && ({form} == "update"))'],
 	'type' =>									[T_ZBX_INT, O_OPT, null,	IN('0,1'),	null],
 	'description' =>							[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Name')],
+	'event_name' =>								[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
+	'opdata' =>									[T_ZBX_STR, O_OPT, null,	null,		'isset({add}) || isset({update})'],
 	'expression' =>								[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'isset({add}) || isset({update})', _('Expression')],
 	'recovery_expression' =>					[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,		'(isset({add}) || isset({update})) && isset({recovery_mode}) && {recovery_mode} == '.ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION.'', _('Recovery expression')],
 	'recovery_mode' =>							[T_ZBX_INT, O_OPT, null,	IN(ZBX_RECOVERY_MODE_EXPRESSION.','.ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION.','.ZBX_RECOVERY_MODE_NONE),	null],
@@ -44,6 +47,7 @@ $fields = [
 	'correlation_mode' =>						[T_ZBX_STR, O_OPT, null,	IN(ZBX_TRIGGER_CORRELATION_NONE.','.ZBX_TRIGGER_CORRELATION_TAG),	null],
 	'correlation_tag' =>						[T_ZBX_STR, O_OPT, null,	null,			'isset({add}) || isset({update})'],
 	'status' =>									[T_ZBX_STR, O_OPT, null,	null,		null],
+	'discover' =>								[T_ZBX_INT, O_OPT, null,	IN([ZBX_PROTOTYPE_DISCOVER, ZBX_PROTOTYPE_NO_DISCOVER]), null],
 	'expression_constructor' =>					[T_ZBX_INT, O_OPT, null,	NOT_EMPTY,	'isset({toggle_expression_constructor})'],
 	'recovery_expression_constructor' =>		[T_ZBX_INT, O_OPT, null,	NOT_EMPTY,		'isset({toggle_recovery_expression_constructor})'],
 	'expr_temp' =>								[T_ZBX_STR, O_OPT, null,	NOT_EMPTY,	'(isset({add_expression}) || isset({and_expression}) || isset({or_expression}) || isset({replace_expression}))', _('Expression')],
@@ -54,6 +58,11 @@ $fields = [
 	'new_dependency' =>							[T_ZBX_INT, O_OPT, null,	DB_ID.NOT_ZERO, 'isset({add_dependency})'],
 	'g_triggerid' =>							[T_ZBX_INT, O_OPT, null,	DB_ID,		null],
 	'tags' =>									[T_ZBX_STR, O_OPT, null,	null,		null],
+	'mass_update_tags'	=>						[T_ZBX_INT, O_OPT, null,
+													IN([ZBX_ACTION_ADD, ZBX_ACTION_REPLACE, ZBX_ACTION_REMOVE]),
+													null
+												],
+	'show_inherited_tags' =>					[T_ZBX_INT, O_OPT, null,	IN([0,1]),	null],
 	'manual_close' =>							[T_ZBX_INT, O_OPT, null,
 													IN([ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED,
 														ZBX_TRIGGER_MANUAL_CLOSE_ALLOWED
@@ -96,7 +105,7 @@ $fields = [
 	'form' =>									[T_ZBX_STR, O_OPT, P_SYS,	null,		null],
 	'form_refresh' =>							[T_ZBX_INT, O_OPT, null,	null,		null],
 	// sort and sortorder
-	'sort' =>									[T_ZBX_STR, O_OPT, P_SYS, IN('"description","priority","status"'),		null],
+	'sort' =>									[T_ZBX_STR, O_OPT, P_SYS, IN('"description","priority","status","discover"'),		null],
 	'sortorder' =>								[T_ZBX_STR, O_OPT, P_SYS, IN('"'.ZBX_SORT_DOWN.'","'.ZBX_SORT_UP.'"'),	null]
 ];
 
@@ -116,28 +125,33 @@ if (!$discoveryRule) {
 	access_deny();
 }
 
-$triggerPrototypeIds = getRequest('g_triggerid', []);
-if (!is_array($triggerPrototypeIds)) {
-	$triggerPrototypeIds = zbx_toArray($triggerPrototypeIds);
-}
-
-$triggerPrototypeId = getRequest('triggerid');
-if ($triggerPrototypeId !== null) {
-	$triggerPrototypeIds[] = $triggerPrototypeId;
-}
-
-if ($triggerPrototypeIds) {
+if (hasRequest('triggerid')) {
 	$triggerPrototypes = API::TriggerPrototype()->get([
-		'output' => ['triggerid'],
-		'triggerids' => $triggerPrototypeIds,
-		'editable' => true,
-		'preservekeys' => true
+		'output' => [],
+		'triggerids' => getRequest('triggerid'),
+		'editable' => true
 	]);
 
-	foreach ($triggerPrototypeIds as $triggerPrototypeId) {
-		if (!array_key_exists($triggerPrototypeId, $triggerPrototypes)) {
-			access_deny();
-		}
+	if (!$triggerPrototypes) {
+		access_deny();
+	}
+}
+
+
+$tags = getRequest('tags', []);
+foreach ($tags as $key => $tag) {
+	// remove empty new tag lines
+	if ($tag['tag'] === '' && $tag['value'] === '') {
+		unset($tags[$key]);
+		continue;
+	}
+
+	// remove inherited tags
+	if (array_key_exists('type', $tag) && !($tag['type'] & ZBX_PROPERTY_OWN)) {
+		unset($tags[$key]);
+	}
+	else {
+		unset($tags[$key]['type']);
 	}
 }
 
@@ -187,17 +201,10 @@ if (hasRequest('clone') && hasRequest('triggerid')) {
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	$tags = getRequest('tags', []);
 	$dependencies = zbx_toObject(getRequest('dependencies', []), 'triggerid');
-
-	// Remove empty new tag lines.
-	foreach ($tags as $key => $tag) {
-		if ($tag['tag'] === '' && $tag['value'] === '') {
-			unset($tags[$key]);
-		}
-	}
-
 	$description = getRequest('description', '');
+	$event_name = getRequest('event_name', '');
+	$opdata = getRequest('opdata', '');
 	$expression = getRequest('expression', '');
 	$recovery_mode = getRequest('recovery_mode', ZBX_RECOVERY_MODE_EXPRESSION);
 	$recovery_expression = getRequest('recovery_expression', '');
@@ -209,10 +216,13 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	$correlation_tag = getRequest('correlation_tag', '');
 	$manual_close = getRequest('manual_close', ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED);
 	$status = getRequest('status', TRIGGER_STATUS_ENABLED);
+	$discover = getRequest('discover', DB::getDefault('triggers', 'discover'));
 
 	if (hasRequest('add')) {
 		$trigger_prototype = [
 			'description' => $description,
+			'event_name' => $event_name,
+			'opdata' => $opdata,
 			'expression' => $expression,
 			'recovery_mode' => $recovery_mode,
 			'type' => $type,
@@ -222,7 +232,8 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			'tags' => $tags,
 			'manual_close' => $manual_close,
 			'dependencies' => $dependencies,
-			'status' => $status
+			'status' => $status,
+			'discover' => $discover
 		];
 		switch ($recovery_mode) {
 			case ZBX_RECOVERY_MODE_RECOVERY_EXPRESSION:
@@ -244,7 +255,8 @@ elseif (hasRequest('add') || hasRequest('update')) {
 	else {
 		$db_trigger_prototypes = API::TriggerPrototype()->get([
 			'output' => ['expression', 'description', 'url', 'status', 'priority', 'comments', 'templateid', 'type',
-				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close'
+				'recovery_mode', 'recovery_expression', 'correlation_mode', 'correlation_tag', 'manual_close', 'opdata',
+				'discover', 'event_name'
 			],
 			'selectDependencies' => ['triggerid'],
 			'selectTags' => ['tag', 'value'],
@@ -263,6 +275,12 @@ elseif (hasRequest('add') || hasRequest('update')) {
 			if ($db_trigger_prototype['description'] !== $description) {
 				$trigger_prototype['description'] = $description;
 			}
+			if ($db_trigger_prototype['event_name'] !== $event_name) {
+				$trigger_prototype['event_name'] = $event_name;
+			}
+			if ($db_trigger_prototype['opdata'] !== $opdata) {
+				$trigger_prototype['opdata'] = $opdata;
+			}
 			if ($db_trigger_prototype['expression'] !== $expression) {
 				$trigger_prototype['expression'] = $expression;
 			}
@@ -322,6 +340,9 @@ elseif (hasRequest('add') || hasRequest('update')) {
 		if ($db_trigger_prototype['status'] != $status) {
 			$trigger_prototype['status'] = $status;
 		}
+		if ($db_trigger_prototype['discover'] != $discover) {
+			$trigger_prototype['discover'] = $discover;
+		}
 
 		if ($trigger_prototype) {
 			$trigger_prototype['triggerid'] = getRequest('triggerid');
@@ -368,22 +389,31 @@ elseif (hasRequest('action') && getRequest('action') === 'triggerprototype.massu
 		$triggerids = getRequest('g_triggerid');
 		$triggers_to_update = [];
 
-		$triggers = API::TriggerPrototype()->get([
+		$options = [
 			'output' => ['triggerid', 'templateid'],
 			'triggerids' => $triggerids,
 			'preservekeys' => true
-		]);
+		];
 
-		if ($triggers) {
-			$tags = getRequest('tags', []);
+		if (array_key_exists('tags', $visible)) {
+			$mass_update_tags = getRequest('mass_update_tags', ZBX_ACTION_ADD);
 
-			// Remove empty new tag lines.
-			foreach ($tags as $key => $tag) {
-				if ($tag['tag'] === '' && $tag['value'] === '') {
-					unset($tags[$key]);
-				}
+			if ($mass_update_tags == ZBX_ACTION_ADD || $mass_update_tags == ZBX_ACTION_REMOVE) {
+				$options['selectTags'] = ['tag', 'value'];
 			}
 
+			$unique_tags = [];
+
+			foreach ($tags as $tag) {
+				$unique_tags[$tag['tag'].':'.$tag['value']] = $tag;
+			}
+
+			$tags = array_values($unique_tags);
+		}
+
+		$triggers = API::TriggerPrototype()->get($options);
+
+		if ($triggers) {
 			foreach ($triggerids as $triggerid) {
 				if (array_key_exists($triggerid, $triggers)) {
 					$trigger = ['triggerid' => $triggerid];
@@ -397,13 +427,43 @@ elseif (hasRequest('action') && getRequest('action') === 'triggerprototype.massu
 					}
 
 					if (array_key_exists('tags', $visible)) {
-						$trigger['tags'] = $tags;
+						if ($tags && $mass_update_tags == ZBX_ACTION_ADD) {
+							$unique_tags = [];
+
+							foreach (array_merge($triggers[$triggerid]['tags'], $tags) as $tag) {
+								$unique_tags[$tag['tag'].':'.$tag['value']] = $tag;
+							}
+
+							$trigger['tags'] = array_values($unique_tags);
+						}
+						elseif ($mass_update_tags == ZBX_ACTION_REPLACE) {
+							$trigger['tags'] = $tags;
+						}
+						elseif ($tags && $mass_update_tags == ZBX_ACTION_REMOVE) {
+							$diff_tags = [];
+
+							foreach ($triggers[$triggerid]['tags'] as $a) {
+								foreach ($tags as $b) {
+									if ($a['tag'] === $b['tag'] && $a['value'] === $b['value']) {
+										continue 2;
+									}
+								}
+
+								$diff_tags[] = $a;
+							}
+
+							$trigger['tags'] = $diff_tags;
+						}
 					}
 
 					if ($triggers[$triggerid]['templateid'] == 0 && array_key_exists('manual_close', $visible)) {
 						$trigger['manual_close'] = getRequest('manual_close');
 					}
 
+					if (array_key_exists('discover', $visible)) {
+						$trigger['discover'] = getRequest('discover');
+					}
+
 					$triggers_to_update[] = $trigger;
 				}
 			}
@@ -419,7 +479,7 @@ elseif (hasRequest('action') && getRequest('action') === 'triggerprototype.massu
 	show_messages($result, _('Trigger prototypes updated'), _('Cannot update trigger prototypes'));
 }
 elseif (getRequest('action') && str_in_array(getRequest('action'), ['triggerprototype.massenable', 'triggerprototype.massdisable']) && hasRequest('g_triggerid')) {
-	$status = (getRequest('action') == 'triggerprototype.massenable')
+	$status = (getRequest('action') === 'triggerprototype.massenable')
 		? TRIGGER_STATUS_ENABLED
 		: TRIGGER_STATUS_DISABLED;
 	$update = [];
@@ -456,7 +516,7 @@ elseif (getRequest('action') && str_in_array(getRequest('action'), ['triggerprot
 
 	show_messages($result, $messageSuccess, $messageFailed);
 }
-elseif (hasRequest('action') && getRequest('action') == 'triggerprototype.massdelete' && hasRequest('g_triggerid')) {
+elseif (hasRequest('action') && getRequest('action') === 'triggerprototype.massdelete' && hasRequest('g_triggerid')) {
 	$result = API::TriggerPrototype()->delete(getRequest('g_triggerid'));
 
 	if ($result) {
@@ -465,7 +525,16 @@ elseif (hasRequest('action') && getRequest('action') == 'triggerprototype.massde
 	show_messages($result, _('Trigger prototypes deleted'), _('Cannot delete trigger prototypes'));
 }
 
-$config = select_config();
+if (hasRequest('action') && getRequest('action') !== 'triggerprototype.massupdateform' && hasRequest('g_triggerid')
+		&& !$result) {
+	$triggerPrototypes = API::TriggerPrototype()->get([
+			'output' => [],
+			'triggerids' => getRequest('g_triggerid'),
+			'editable' => true
+		]);
+
+	uncheckTableRows(getRequest('parent_discoveryid'), zbx_objectValues($triggerPrototypes, 'triggerid'));
+}
 
 /*
  * Display
@@ -476,13 +545,11 @@ if ((getRequest('action') === 'triggerprototype.massupdateform' || hasRequest('m
 	$data['action'] = 'triggerprototype.massupdate';
 	$data['hostid'] = $discoveryRule['hostid'];
 
-	$triggersView = new CView('configuration.trigger.prototype.massupdate', $data);
-	$triggersView->render();
-	$triggersView->show();
+	// Render view.
+	echo (new CView('configuration.trigger.prototype.massupdate', $data))->getOutput();
 }
 elseif (isset($_REQUEST['form'])) {
 	$data = getTriggerFormData([
-		'config' => $config,
 		'form' => getRequest('form'),
 		'form_refresh' => getRequest('form_refresh'),
 		'parent_discoveryid' => getRequest('parent_discoveryid'),
@@ -495,27 +562,31 @@ elseif (isset($_REQUEST['form'])) {
 		'recovery_expr_temp' => getRequest('recovery_expr_temp', ''),
 		'recovery_mode' => getRequest('recovery_mode', ZBX_RECOVERY_MODE_EXPRESSION),
 		'description' => getRequest('description', ''),
+		'event_name' => getRequest('event_name', ''),
+		'opdata' => getRequest('opdata', ''),
 		'type' => getRequest('type', 0),
 		'priority' => getRequest('priority', TRIGGER_SEVERITY_NOT_CLASSIFIED),
 		'status' => getRequest('status', TRIGGER_STATUS_ENABLED),
+		'discover' => getRequest('discover', DB::getDefault('triggers', 'discover')),
 		'comments' => getRequest('comments', ''),
 		'url' => getRequest('url', ''),
 		'expression_constructor' => getRequest('expression_constructor', IM_ESTABLISHED),
 		'recovery_expression_constructor' => getRequest('recovery_expression_constructor', IM_ESTABLISHED),
 		'limited' => false,
 		'templates' => [],
+		'parent_templates' => [],
 		'hostid' => $discoveryRule['hostid'],
 		'expression_action' => $expression_action,
 		'recovery_expression_action' => $recovery_expression_action,
-		'tags' => getRequest('tags', []),
+		'tags' => array_values($tags),
+		'show_inherited_tags' => getRequest('show_inherited_tags', 0),
 		'correlation_mode' => getRequest('correlation_mode', ZBX_TRIGGER_CORRELATION_NONE),
 		'correlation_tag' => getRequest('correlation_tag', ''),
 		'manual_close' => getRequest('manual_close', ZBX_TRIGGER_MANUAL_CLOSE_NOT_ALLOWED)
 	]);
 
-	$triggersView = new CView('configuration.trigger.prototype.edit', $data);
-	$triggersView->render();
-	$triggersView->show();
+	// render view
+	echo (new CView('configuration.trigger.prototype.edit', $data))->getOutput();
 }
 else {
 	$sortField = getRequest('sort', CProfile::get('web.'.$page['file'].'.sort', 'description'));
@@ -531,7 +602,6 @@ else {
 		'triggers' => [],
 		'sort' => $sortField,
 		'sortorder' => $sortOrder,
-		'config' => $config,
 		'dependencyTriggers' => []
 	];
 
@@ -541,21 +611,32 @@ else {
 		'output' => ['triggerid', $sortField],
 		'discoveryids' => $data['parent_discoveryid'],
 		'sortfield' => $sortField,
-		'limit' => $config['search_limit'] + 1
+		'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT) + 1
 	];
 	$data['triggers'] = API::TriggerPrototype()->get($options);
 
 	order_result($data['triggers'], $sortField, $sortOrder);
 
-	// paging
-	$url = (new CUrl('trigger_prototypes.php'))
-		->setArgument('parent_discoveryid', $data['parent_discoveryid']);
+	// pager
+	if (hasRequest('page')) {
+		$page_num = getRequest('page');
+	}
+	elseif (isRequestMethod('get') && !hasRequest('cancel')) {
+		$page_num = 1;
+	}
+	else {
+		$page_num = CPagerHelper::loadPage($page['file']);
+	}
 
-	$data['paging'] = getPagingLine($data['triggers'], $sortOrder, $url);
+	CPagerHelper::savePage($page['file'], $page_num);
+
+	$data['paging'] = CPagerHelper::paginate($page_num, $data['triggers'], $sortOrder,
+		(new CUrl('trigger_prototypes.php'))->setArgument('parent_discoveryid', $data['parent_discoveryid'])
+	);
 
 	$data['triggers'] = API::TriggerPrototype()->get([
 		'output' => ['triggerid', 'expression', 'description', 'status', 'priority', 'templateid', 'recovery_mode',
-			'recovery_expression'
+			'recovery_expression', 'opdata', 'discover'
 		],
 		'selectHosts' => ['hostid', 'host'],
 		'selectDependencies' => ['triggerid', 'description'],
@@ -610,11 +691,10 @@ else {
 	}
 
 	$data['parent_templates'] = getTriggerParentTemplates($data['triggers'], ZBX_FLAG_DISCOVERY_PROTOTYPE);
+	$data['allowed_ui_conf_templates'] = CWebUser::checkAccess(CRoleHelper::UI_CONFIGURATION_TEMPLATES);
 
-	// render view
-	$triggersView = new CView('configuration.trigger.prototype.list', $data);
-	$triggersView->render();
-	$triggersView->show();
+	// Render view.
+	echo (new CView('configuration.trigger.prototype.list', $data))->getOutput();
 }
 
 require_once dirname(__FILE__).'/include/page_footer.php';
